@@ -6,6 +6,7 @@
 
 #include "d3d11_device_child.h"
 #include "d3d11_interfaces.h"
+#include "d3d11_resource.h"
 
 namespace dxvk {
   
@@ -13,6 +14,15 @@ namespace dxvk {
   class D3D11DeviceContext;
 
 
+  /**
+   * \brief Buffer map mode
+   */
+  enum D3D11_COMMON_BUFFER_MAP_MODE {
+    D3D11_COMMON_BUFFER_MAP_MODE_NONE,
+    D3D11_COMMON_BUFFER_MAP_MODE_DIRECT,
+  };
+
+
   /**
    * \brief Stream output buffer offset
    *
@@ -59,6 +69,12 @@ namespace dxvk {
       return &m_desc;
     }
 
+    D3D11_COMMON_BUFFER_MAP_MODE GetMapMode() const {
+      return (m_buffer->memFlags() & VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT)
+        ? D3D11_COMMON_BUFFER_MAP_MODE_DIRECT
+        : D3D11_COMMON_BUFFER_MAP_MODE_NONE;
+    }
+
     Rc<DxvkBuffer> GetBuffer() const {
       return m_buffer;
     }
@@ -76,7 +92,13 @@ namespace dxvk {
     }
 
     DxvkBufferSlice GetSOCounter() {
-      return m_soCounter;
+      return m_soCounter != nullptr
+        ? DxvkBufferSlice(m_soCounter)
+        : DxvkBufferSlice();
+    }
+    
+    DxvkBufferSliceHandle AllocSlice() {
+      return m_buffer->allocSlice();
     }
     
     DxvkBufferSliceHandle DiscardSlice() {
@@ -92,21 +114,35 @@ namespace dxvk {
       return &m_d3d10;
     }
 
+    /**
+     * \brief Normalizes buffer description
+     * 
+     * \param [in] pDesc Buffer description
+     * \returns \c S_OK if the parameters are valid
+     */
+    static HRESULT NormalizeBufferProperties(
+            D3D11_BUFFER_DESC*      pDesc);
+
   private:
     
     const Com<D3D11Device>      m_device;
     const D3D11_BUFFER_DESC     m_desc;
     
     Rc<DxvkBuffer>              m_buffer;
-    DxvkBufferSlice             m_soCounter;
+    Rc<DxvkBuffer>              m_soCounter;
     DxvkBufferSliceHandle       m_mapped;
 
+    D3D11DXGIResource           m_resource;
     D3D10Buffer                 m_d3d10;
 
     BOOL CheckFormatFeatureSupport(
             VkFormat              Format,
             VkFormatFeatureFlags  Features) const;
     
+    VkMemoryPropertyFlags GetMemoryFlags() const;
+
+    Rc<DxvkBuffer> CreateSoCounterBuffer();
+
   };
 
 
