@@ -16,6 +16,9 @@ namespace dxvk {
   
   
   HRESULT STDMETHODCALLTYPE D3D11CommandList::QueryInterface(REFIID riid, void** ppvObject) {
+    if (ppvObject == nullptr)
+      return E_POINTER;
+
     *ppvObject = nullptr;
     
     if (riid == __uuidof(IUnknown)
@@ -46,17 +49,28 @@ namespace dxvk {
   }
   
   
+  void D3D11CommandList::AddQuery(D3D11Query* pQuery) {
+    m_queries.emplace_back(pQuery);
+  }
+
+
   void D3D11CommandList::EmitToCommandList(ID3D11CommandList* pCommandList) {
     auto cmdList = static_cast<D3D11CommandList*>(pCommandList);
     
     for (const auto& chunk : m_chunks)
       cmdList->m_chunks.push_back(chunk);
-    
+
+    for (const auto& query : m_queries)
+      cmdList->m_queries.push_back(query);
+
     MarkSubmitted();
   }
   
   
   void D3D11CommandList::EmitToCsThread(DxvkCsThread* CsThread) {
+    for (const auto& query : m_queries)
+      query->DoDeferredEnd();
+
     for (const auto& chunk : m_chunks)
       CsThread->dispatchChunk(DxvkCsChunkRef(chunk));
     
