@@ -3,8 +3,9 @@
 #include "d3d11_device.h"
 #include "d3d11_texture.h"
 
-constexpr static uint32_t MinFlushIntervalUs = 1250;
-constexpr static uint32_t MaxPendingSubmits  = 3;
+constexpr static uint32_t MinFlushIntervalUs = 750;
+constexpr static uint32_t IncFlushIntervalUs = 250;
+constexpr static uint32_t MaxPendingSubmits  = 6;
 
 namespace dxvk {
   
@@ -13,8 +14,14 @@ namespace dxvk {
     const Rc<DxvkDevice>& Device)
   : D3D11DeviceContext(pParent, Device, DxvkCsChunkFlag::SingleUse),
     m_csThread(Device->createContext()) {
-    EmitCs([cDevice = m_device] (DxvkContext* ctx) {
+    EmitCs([
+      cDevice          = m_device,
+      cRelaxedBarriers = pParent->GetOptions()->relaxedBarriers
+    ] (DxvkContext* ctx) {
       ctx->beginRecording(cDevice->createCommandList());
+
+      if (cRelaxedBarriers)
+        ctx->setBarrierControl(DxvkBarrierControl::IgnoreWriteAfterWrite);
     });
     
     ClearState();
@@ -29,12 +36,18 @@ namespace dxvk {
   
   
   ULONG STDMETHODCALLTYPE D3D11ImmediateContext::AddRef() {
-    return m_parent->AddRef();
+    ULONG refCount = m_refCount++;
+    if (!refCount)
+      m_parent->AddRef();
+    return refCount + 1;
   }
   
   
   ULONG STDMETHODCALLTYPE D3D11ImmediateContext::Release() {
-    return m_parent->Release();
+    ULONG refCount = --m_refCount;
+    if (!refCount)
+      m_parent->Release();
+    return refCount;
   }
   
   
@@ -48,59 +61,106 @@ namespace dxvk {
   }
   
   
-  void STDMETHODCALLTYPE D3D11ImmediateContext::End(
-          ID3D11Asynchronous*               pAsync) {
-    D3D11DeviceContext::End(pAsync);
-
-    if (pAsync) {
-      D3D11_QUERY_DESC desc;
-      static_cast<D3D11Query*>(pAsync)->GetDesc(&desc);
-      
-      if (desc.Query == D3D11_QUERY_EVENT)
-        FlushImplicit(TRUE);
-    }
-  }
-
-
   HRESULT STDMETHODCALLTYPE D3D11ImmediateContext::GetData(
           ID3D11Asynchronous*               pAsync,
           void*                             pData,
           UINT                              DataSize,
           UINT                              GetDataFlags) {
-    if (!pAsync)
+    if (!pAsync || (DataSize && !pData))
       return E_INVALIDARG;
     
-    // Make sure that we can safely write to the memory
-    // location pointed to by pData if it is specified.
-    if (DataSize == 0)
-      pData = nullptr;
-    
-    if (pData && pAsync->GetDataSize() != DataSize) {
-      Logger::err(str::format(
-        "D3D11: GetData: Data size mismatch",
-        "\n  Expected: ", pAsync->GetDataSize(),
-        "\n  Got:      ", DataSize));
+    // Check whether the data size is actually correct
+    if (DataSize && DataSize != pAsync->GetDataSize())
       return E_INVALIDARG;
-    }
     
+    // Passing a non-null pData is actually allowed if
+    // DataSize is 0, but we should ignore that pointer
+    pData = DataSize ? pData : nullptr;
+
     // Get query status directly from the query object
-    HRESULT hr = static_cast<D3D11Query*>(pAsync)->GetData(pData, GetDataFlags);
+    auto query = static_cast<D3D11Query*>(pAsync);
+    HRESULT hr = query->GetData(pData, GetDataFlags);
     
     // If we're likely going to spin on the asynchronous object,
-    // flush the context so that we're keeping the GPU busy
-    if (hr == S_FALSE)
+    // flush the context so that we're keeping the GPU busy.
+    if (hr == S_FALSE) {
+      // Don't mark the event query as stalling if the app does
+      // not intend to spin on it. This reduces flushes on End.
+      if (!(GetDataFlags & D3D11_ASYNC_GETDATA_DONOTFLUSH))
+        query->NotifyStall();
+
+      // Ignore the DONOTFLUSH flag here as some games will spin
+      // on queries without ever flushing the context otherwise.
       FlushImplicit(FALSE);
+    }
     
     return hr;
   }
   
   
+  void STDMETHODCALLTYPE D3D11ImmediateContext::Begin(ID3D11Asynchronous* pAsync) {
+    D3D10DeviceLock lock = LockContext();
+
+    if (unlikely(!pAsync))
+      return;
+    
+    auto query = static_cast<D3D11Query*>(pAsync);
+
+    if (unlikely(!query->DoBegin()))
+      return;
+
+    EmitCs([cQuery = Com<D3D11Query, false>(query)]
+    (DxvkContext* ctx) {
+      cQuery->Begin(ctx);
+    });
+  }
+
+
+  void STDMETHODCALLTYPE D3D11ImmediateContext::End(ID3D11Asynchronous* pAsync) {
+    D3D10DeviceLock lock = LockContext();
+
+    if (unlikely(!pAsync))
+      return;
+    
+    auto query = static_cast<D3D11Query*>(pAsync);
+
+    if (unlikely(!query->DoEnd())) {
+      EmitCs([cQuery = Com<D3D11Query, false>(query)]
+      (DxvkContext* ctx) {
+        cQuery->Begin(ctx);
+      });
+    }
+
+    EmitCs([cQuery = Com<D3D11Query, false>(query)]
+    (DxvkContext* ctx) {
+      cQuery->End(ctx);
+    });
+
+    if (unlikely(query->IsEvent())) {
+      query->NotifyEnd();
+      query->IsStalling()
+        ? Flush()
+        : FlushImplicit(TRUE);
+    }
+  }
+
+
   void STDMETHODCALLTYPE D3D11ImmediateContext::Flush() {
+    Flush1(D3D11_CONTEXT_TYPE_ALL, nullptr);
+  }
+
+
+  void STDMETHODCALLTYPE D3D11ImmediateContext::Flush1(
+          D3D11_CONTEXT_TYPE          ContextType,
+          HANDLE                      hEvent) {
     m_parent->FlushInitContext();
+
+    if (hEvent)
+      Logger::warn("D3D11: Flush1: Ignoring event");
     
     D3D10DeviceLock lock = LockContext();
     
-    if (m_csIsBusy || m_csChunk->commandCount() != 0) {
+    if (m_csIsBusy || !m_csChunk->empty()) {
       // Add commands to flush the threaded
       // context, then flush the command list
       EmitCs([] (DxvkContext* ctx) {
@@ -110,12 +170,28 @@ namespace dxvk {
       FlushCsChunk();
       
       // Reset flush timer used for implicit flushes
-      m_lastFlush = std::chrono::high_resolution_clock::now();
+      m_lastFlush = dxvk::high_resolution_clock::now();
       m_csIsBusy  = false;
     }
   }
   
   
+  HRESULT STDMETHODCALLTYPE D3D11ImmediateContext::Signal(
+          ID3D11Fence*                pFence,
+          UINT64                      Value) {
+    Logger::err("D3D11ImmediateContext::Signal: Not implemented");
+    return E_NOTIMPL;
+  }
+
+
+  HRESULT STDMETHODCALLTYPE D3D11ImmediateContext::Wait(
+          ID3D11Fence*                pFence,
+          UINT64                      Value) {
+    Logger::err("D3D11ImmediateContext::Wait: Not implemented");
+    return E_NOTIMPL;
+  }
+
+
   void STDMETHODCALLTYPE D3D11ImmediateContext::ExecuteCommandList(
           ID3D11CommandList*  pCommandList,
           BOOL                RestoreContextState) {
@@ -164,7 +240,7 @@ namespace dxvk {
           D3D11_MAPPED_SUBRESOURCE*   pMappedResource) {
     D3D10DeviceLock lock = LockContext();
 
-    if (!pResource || !pMappedResource)
+    if (unlikely(!pResource))
       return E_INVALIDARG;
     
     D3D11_RESOURCE_DIMENSION resourceDim = D3D11_RESOURCE_DIMENSION_UNKNOWN;
@@ -172,7 +248,7 @@ namespace dxvk {
 
     HRESULT hr;
     
-    if (resourceDim == D3D11_RESOURCE_DIMENSION_BUFFER) {
+    if (likely(resourceDim == D3D11_RESOURCE_DIMENSION_BUFFER)) {
       hr = MapBuffer(
         static_cast<D3D11Buffer*>(pResource),
         MapType, MapFlags, pMappedResource);
@@ -183,11 +259,8 @@ namespace dxvk {
         pMappedResource);
     }
 
-    if (FAILED(hr)) {
-      pMappedResource->pData      = nullptr;
-      pMappedResource->RowPitch   = 0;
-      pMappedResource->DepthPitch = 0;
-    }
+    if (unlikely(FAILED(hr)))
+      *pMappedResource = D3D11_MAPPED_SUBRESOURCE();
 
     return hr;
   }
@@ -196,69 +269,14 @@ namespace dxvk {
   void STDMETHODCALLTYPE D3D11ImmediateContext::Unmap(
           ID3D11Resource*             pResource,
           UINT                        Subresource) {
-    D3D10DeviceLock lock = LockContext();
-
     D3D11_RESOURCE_DIMENSION resourceDim = D3D11_RESOURCE_DIMENSION_UNKNOWN;
     pResource->GetType(&resourceDim);
     
-    if (resourceDim != D3D11_RESOURCE_DIMENSION_BUFFER)
+    if (unlikely(resourceDim != D3D11_RESOURCE_DIMENSION_BUFFER)) {
+      D3D10DeviceLock lock = LockContext();
       UnmapImage(GetCommonTexture(pResource), Subresource);
+    }
   }
-  
-  
-  void STDMETHODCALLTYPE D3D11ImmediateContext::CopySubresourceRegion(
-          ID3D11Resource*                   pDstResource,
-          UINT                              DstSubresource,
-          UINT                              DstX,
-          UINT                              DstY,
-          UINT                              DstZ,
-          ID3D11Resource*                   pSrcResource,
-          UINT                              SrcSubresource,
-    const D3D11_BOX*                        pSrcBox) {
-    FlushImplicit(FALSE);
-
-    D3D11DeviceContext::CopySubresourceRegion(
-      pDstResource, DstSubresource, DstX, DstY, DstZ,
-      pSrcResource, SrcSubresource, pSrcBox);
-  }
-  
-
-  void STDMETHODCALLTYPE D3D11ImmediateContext::CopySubresourceRegion1(
-          ID3D11Resource*                   pDstResource,
-          UINT                              DstSubresource,
-          UINT                              DstX,
-          UINT                              DstY,
-          UINT                              DstZ,
-          ID3D11Resource*                   pSrcResource,
-          UINT                              SrcSubresource,
-    const D3D11_BOX*                        pSrcBox,
-          UINT                              CopyFlags) {
-    FlushImplicit(FALSE);
-
-    D3D11DeviceContext::CopySubresourceRegion1(
-      pDstResource, DstSubresource, DstX, DstY, DstZ,
-      pSrcResource, SrcSubresource, pSrcBox, CopyFlags);
-  }
-
-  
-  void STDMETHODCALLTYPE D3D11ImmediateContext::CopyResource(
-          ID3D11Resource*                   pDstResource,
-          ID3D11Resource*                   pSrcResource) {
-    FlushImplicit(FALSE);
-
-    D3D11DeviceContext::CopyResource(
-      pDstResource, pSrcResource);
-  }
-
-  
-  void STDMETHODCALLTYPE D3D11ImmediateContext::GenerateMips(
-          ID3D11ShaderResourceView*         pShaderResourceView) {
-    FlushImplicit(FALSE);
-
-    D3D11DeviceContext::GenerateMips(
-      pShaderResourceView);
-  }
-  
 
   void STDMETHODCALLTYPE D3D11ImmediateContext::UpdateSubresource(
           ID3D11Resource*                   pDstResource,
@@ -291,27 +309,12 @@ namespace dxvk {
       CopyFlags);
   }
   
-
-  void STDMETHODCALLTYPE D3D11ImmediateContext::ResolveSubresource(
-          ID3D11Resource*                   pDstResource,
-          UINT                              DstSubresource,
-          ID3D11Resource*                   pSrcResource,
-          UINT                              SrcSubresource,
-          DXGI_FORMAT                       Format) {
-    FlushImplicit(FALSE);
-
-    D3D11DeviceContext::ResolveSubresource(
-      pDstResource, DstSubresource,
-      pSrcResource, SrcSubresource,
-      Format);
-  }
-
-
+  
   void STDMETHODCALLTYPE D3D11ImmediateContext::OMSetRenderTargets(
           UINT                              NumViews,
           ID3D11RenderTargetView* const*    ppRenderTargetViews,
           ID3D11DepthStencilView*           pDepthStencilView) {
-    FlushImplicit(FALSE);
+    FlushImplicit(TRUE);
     
     D3D11DeviceContext::OMSetRenderTargets(
       NumViews, ppRenderTargetViews, pDepthStencilView);
@@ -326,7 +329,7 @@ namespace dxvk {
           UINT                              NumUAVs,
           ID3D11UnorderedAccessView* const* ppUnorderedAccessViews,
     const UINT*                             pUAVInitialCounts) {
-    FlushImplicit(FALSE);
+    FlushImplicit(TRUE);
 
     D3D11DeviceContext::OMSetRenderTargetsAndUnorderedAccessViews(
       NumRTVs, ppRenderTargetViews, pDepthStencilView,
@@ -340,9 +343,10 @@ namespace dxvk {
           D3D11_MAP                   MapType,
           UINT                        MapFlags,
           D3D11_MAPPED_SUBRESOURCE*   pMappedResource) {
-    Rc<DxvkBuffer> buffer = pResource->GetBuffer();
+    if (unlikely(!pMappedResource))
+      return E_INVALIDARG;
 
-    if (!(buffer->memFlags() & VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT)) {
+    if (unlikely(pResource->GetMapMode() == D3D11_COMMON_BUFFER_MAP_MODE_NONE)) {
       Logger::err("D3D11: Cannot map a device-local buffer");
       return E_INVALIDARG;
     }
@@ -357,8 +361,8 @@ namespace dxvk {
       pMappedResource->DepthPitch = pResource->Desc()->ByteWidth;
       
       EmitCs([
-        cBuffer      = std::move(buffer),
-        cBufferSlice = std::move(physSlice)
+        cBuffer      = pResource->GetBuffer(),
+        cBufferSlice = physSlice
       ] (DxvkContext* ctx) {
         ctx->invalidateBuffer(cBuffer, cBufferSlice);
       });
@@ -367,7 +371,7 @@ namespace dxvk {
     } else {
       // Wait until the resource is no longer in use
       if (MapType != D3D11_MAP_WRITE_NO_OVERWRITE) {
-        if (!WaitForResource(buffer, MapFlags))
+        if (!WaitForResource(pResource->GetBuffer(), MapType, MapFlags))
           return DXGI_ERROR_WAS_STILL_DRAWING;
       }
 
@@ -391,70 +395,52 @@ namespace dxvk {
           UINT                        MapFlags,
           D3D11_MAPPED_SUBRESOURCE*   pMappedResource) {
     const Rc<DxvkImage>  mappedImage  = pResource->GetImage();
-    const Rc<DxvkBuffer> mappedBuffer = pResource->GetMappedBuffer();
+    const Rc<DxvkBuffer> mappedBuffer = pResource->GetMappedBuffer(Subresource);
     
-    if (pResource->GetMapMode() == D3D11_COMMON_TEXTURE_MAP_MODE_NONE) {
+    if (unlikely(pResource->GetMapMode() == D3D11_COMMON_TEXTURE_MAP_MODE_NONE)) {
       Logger::err("D3D11: Cannot map a device-local image");
       return E_INVALIDARG;
     }
+
+    if (unlikely(Subresource >= pResource->CountSubresources()))
+      return E_INVALIDARG;
     
-    auto formatInfo = imageFormatInfo(mappedImage->info().format);
+    if (likely(pMappedResource != nullptr)) {
+      // Resources with an unknown memory layout cannot return a pointer
+      if (pResource->Desc()->Usage         == D3D11_USAGE_DEFAULT
+       && pResource->Desc()->TextureLayout == D3D11_TEXTURE_LAYOUT_UNDEFINED)
+        return E_INVALIDARG;
+    } else {
+      if (pResource->Desc()->Usage != D3D11_USAGE_DEFAULT)
+        return E_INVALIDARG;
+    }
+
+    VkFormat packedFormat = m_parent->LookupPackedFormat(
+      pResource->Desc()->Format, pResource->GetFormatMode()).Format;
+    
+    auto formatInfo = imageFormatInfo(packedFormat);
     auto subresource = pResource->GetSubresourceFromIndex(
-        formatInfo->aspectMask, Subresource);
-    
-    pResource->SetMappedSubresource(subresource, MapType);
+      formatInfo->aspectMask, Subresource);
     
     if (pResource->GetMapMode() == D3D11_COMMON_TEXTURE_MAP_MODE_DIRECT) {
       const VkImageType imageType = mappedImage->info().type;
       
       // Wait for the resource to become available
-      if (!WaitForResource(mappedImage, MapFlags))
+      if (!WaitForResource(mappedImage, MapType, MapFlags))
         return DXGI_ERROR_WAS_STILL_DRAWING;
       
+      // Mark the given subresource as mapped
+      pResource->SetMapType(Subresource, MapType);
+
       // Query the subresource's memory layout and hope that
       // the application respects the returned pitch values.
-      VkSubresourceLayout layout  = mappedImage->querySubresourceLayout(subresource);
-      pMappedResource->pData      = mappedImage->mapPtr(layout.offset);
-      pMappedResource->RowPitch   = imageType >= VK_IMAGE_TYPE_2D ? layout.rowPitch   : layout.size;
-      pMappedResource->DepthPitch = imageType >= VK_IMAGE_TYPE_3D ? layout.depthPitch : layout.size;
-      return S_OK;
-    } else if (formatInfo->aspectMask == (VK_IMAGE_ASPECT_DEPTH_BIT | VK_IMAGE_ASPECT_STENCIL_BIT)) {
-      VkExtent3D levelExtent = mappedImage->mipLevelExtent(subresource.mipLevel);
-
-      if (MapType != D3D11_MAP_READ) {
-        Logger::err(str::format("D3D11: Map type ", MapType, " not supported for depth-stencil images"));
-        return E_INVALIDARG;
+      if (pMappedResource) {
+        VkSubresourceLayout layout  = mappedImage->querySubresourceLayout(subresource);
+        pMappedResource->pData      = mappedImage->mapPtr(layout.offset);
+        pMappedResource->RowPitch   = imageType >= VK_IMAGE_TYPE_2D ? layout.rowPitch   : layout.size;
+        pMappedResource->DepthPitch = imageType >= VK_IMAGE_TYPE_3D ? layout.depthPitch : layout.size;
       }
 
-      // The actual Vulkan image format may differ
-      // from the format requested by the application
-      VkFormat packFormat = GetPackedDepthStencilFormat(pResource->Desc()->Format);
-      auto packFormatInfo = imageFormatInfo(packFormat);
-
-      // This is slow, but we have to dispatch a pack
-      // operation and then immediately synchronize.
-      EmitCs([
-        cImageBuffer = mappedBuffer,
-        cImage       = mappedImage,
-        cSubresource = subresource,
-        cFormat      = packFormat
-      ] (DxvkContext* ctx) {
-        auto layers = vk::makeSubresourceLayers(cSubresource);
-        auto x = cImage->mipLevelExtent(cSubresource.mipLevel);
-
-        VkOffset2D offset = { 0, 0 };
-        VkExtent2D extent = { x.width, x.height };
-
-        ctx->copyDepthStencilImageToPackedBuffer(
-          cImageBuffer, 0, cImage, layers, offset, extent, cFormat);
-      });
-
-      WaitForResource(mappedBuffer, 0);
-
-      DxvkBufferSliceHandle physSlice = mappedBuffer->getSliceHandle();
-      pMappedResource->pData      = physSlice.mapPtr;
-      pMappedResource->RowPitch   = packFormatInfo->elementSize * levelExtent.width;
-      pMappedResource->DepthPitch = packFormatInfo->elementSize * levelExtent.width * levelExtent.height;
       return S_OK;
     } else {
       VkExtent3D levelExtent = mappedImage->mipLevelExtent(subresource.mipLevel);
@@ -477,32 +463,29 @@ namespace dxvk {
         // When using any map mode which requires the image contents
         // to be preserved, and if the GPU has write access to the
         // image, copy the current image contents into the buffer.
-        const bool copyExistingData = pResource->Desc()->Usage == D3D11_USAGE_STAGING;
-        
-        if (copyExistingData) {
-          auto subresourceLayers = vk::makeSubresourceLayers(subresource);
-          
-          EmitCs([
-            cImageBuffer  = mappedBuffer,
-            cImage        = mappedImage,
-            cSubresources = subresourceLayers,
-            cLevelExtent  = levelExtent
-          ] (DxvkContext* ctx) {
-            ctx->copyImageToBuffer(
-              cImageBuffer, 0, VkExtent2D { 0u, 0u },
-              cImage, cSubresources, VkOffset3D { 0, 0, 0 },
-              cLevelExtent);
-          });
+        if (pResource->Desc()->Usage == D3D11_USAGE_STAGING
+         && !pResource->CanUpdateMappedBufferEarly()) {
+          UpdateMappedBuffer(pResource, subresource);
+          MapFlags &= ~D3D11_MAP_FLAG_DO_NOT_WAIT;
         }
         
-        WaitForResource(mappedBuffer, 0);
+        // Wait for mapped buffer to become available
+        if (!WaitForResource(mappedBuffer, MapType, MapFlags))
+          return DXGI_ERROR_WAS_STILL_DRAWING;
+        
         physSlice = mappedBuffer->getSliceHandle();
       }
       
+      // Mark the given subresource as mapped
+      pResource->SetMapType(Subresource, MapType);
+
       // Set up map pointer. Data is tightly packed within the mapped buffer.
-      pMappedResource->pData      = physSlice.mapPtr;
-      pMappedResource->RowPitch   = formatInfo->elementSize * blockCount.width;
-      pMappedResource->DepthPitch = formatInfo->elementSize * blockCount.width * blockCount.height;
+      if (pMappedResource) {
+        pMappedResource->pData      = physSlice.mapPtr;
+        pMappedResource->RowPitch   = formatInfo->elementSize * blockCount.width;
+        pMappedResource->DepthPitch = formatInfo->elementSize * blockCount.width * blockCount.height;
+      }
+
       return S_OK;
     }
   }
@@ -511,17 +494,26 @@ namespace dxvk {
   void D3D11ImmediateContext::UnmapImage(
           D3D11CommonTexture*         pResource,
           UINT                        Subresource) {
-    if (pResource->GetMapType() == D3D11_MAP_READ)
+    D3D11_MAP mapType = pResource->GetMapType(Subresource);
+    pResource->SetMapType(Subresource, D3D11_MAP(~0u));
+
+    if (mapType == D3D11_MAP(~0u)
+     || mapType == D3D11_MAP_READ)
       return;
     
     if (pResource->GetMapMode() == D3D11_COMMON_TEXTURE_MAP_MODE_BUFFER) {
       // Now that data has been written into the buffer,
       // we need to copy its contents into the image
       const Rc<DxvkImage>  mappedImage  = pResource->GetImage();
-      const Rc<DxvkBuffer> mappedBuffer = pResource->GetMappedBuffer();
-      
-      VkImageSubresource subresource = pResource->GetMappedSubresource();
-      
+      const Rc<DxvkBuffer> mappedBuffer = pResource->GetMappedBuffer(Subresource);
+
+      VkFormat packedFormat = m_parent->LookupPackedFormat(
+        pResource->Desc()->Format, pResource->GetFormatMode()).Format;
+
+      auto formatInfo = imageFormatInfo(packedFormat);
+      auto subresource = pResource->GetSubresourceFromIndex(
+        formatInfo->aspectMask, Subresource);
+
       VkExtent3D levelExtent = mappedImage
         ->mipLevelExtent(subresource.mipLevel);
       
@@ -534,18 +526,51 @@ namespace dxvk {
         cSrcBuffer      = mappedBuffer,
         cDstImage       = mappedImage,
         cDstLayers      = subresourceLayers,
-        cDstLevelExtent = levelExtent
+        cDstLevelExtent = levelExtent,
+        cPackedFormat   = GetPackedDepthStencilFormat(pResource->Desc()->Format)
       ] (DxvkContext* ctx) {
-        ctx->copyBufferToImage(cDstImage, cDstLayers,
-          VkOffset3D { 0, 0, 0 }, cDstLevelExtent,
-          cSrcBuffer, 0, { 0u, 0u });
+        if (cPackedFormat == VK_FORMAT_UNDEFINED) {
+          ctx->copyBufferToImage(cDstImage, cDstLayers,
+            VkOffset3D { 0, 0, 0 }, cDstLevelExtent,
+            cSrcBuffer, 0, { 0u, 0u });
+        } else {
+          ctx->copyPackedBufferToDepthStencilImage(
+            cDstImage, cDstLayers,
+            VkOffset2D { 0, 0 },
+            VkExtent2D { cDstLevelExtent.width, cDstLevelExtent.height },
+            cSrcBuffer, 0, cPackedFormat);
+        }
       });
     }
-    
-    pResource->ClearMappedSubresource();
   }
   
   
+  void STDMETHODCALLTYPE D3D11ImmediateContext::SwapDeviceContextState(
+          ID3DDeviceContextState*           pState,
+          ID3DDeviceContextState**          ppPreviousState) {
+    InitReturnPtr(ppPreviousState);
+
+    if (!pState)
+      return;
+    
+    Com<D3D11DeviceContextState> oldState = std::move(m_stateObject);
+    Com<D3D11DeviceContextState> newState = static_cast<D3D11DeviceContextState*>(pState);
+
+    if (oldState == nullptr)
+      oldState = new D3D11DeviceContextState(m_parent);
+    
+    if (ppPreviousState)
+      *ppPreviousState = oldState.ref();
+    
+    m_stateObject = newState;
+
+    oldState->SetState(m_state);
+    newState->GetState(m_state);
+
+    RestoreState();
+  }
+
+
   void D3D11ImmediateContext::SynchronizeCsThread() {
     D3D10DeviceLock lock = LockContext();
 
@@ -553,7 +578,8 @@ namespace dxvk {
     // recorded prior to this function will be run
     FlushCsChunk();
     
-    m_csThread.synchronize();
+    if (m_csThread.isBusy())
+      m_csThread.synchronize();
   }
   
   
@@ -564,18 +590,20 @@ namespace dxvk {
   
   bool D3D11ImmediateContext::WaitForResource(
     const Rc<DxvkResource>&                 Resource,
+          D3D11_MAP                         MapType,
           UINT                              MapFlags) {
-    // Some games (e.g. The Witcher 3) do not work correctly
-    // when a map fails with D3D11_MAP_FLAG_DO_NOT_WAIT set
-    if (!m_parent->GetOptions()->allowMapFlagNoWait)
-      MapFlags &= ~D3D11_MAP_FLAG_DO_NOT_WAIT;
+    // Determine access type to wait for based on map mode
+    DxvkAccess access = MapType == D3D11_MAP_READ
+      ? DxvkAccess::Write
+      : DxvkAccess::Read;
     
     // Wait for the any pending D3D11 command to be executed
     // on the CS thread so that we can determine whether the
     // resource is currently in use or not.
-    SynchronizeCsThread();
+    if (!Resource->isInUse(access))
+      SynchronizeCsThread();
     
-    if (Resource->isInUse()) {
+    if (Resource->isInUse(access)) {
       if (MapFlags & D3D11_MAP_FLAG_DO_NOT_WAIT) {
         // We don't have to wait, but misbehaving games may
         // still try to spin on `Map` until the resource is
@@ -588,7 +616,7 @@ namespace dxvk {
         Flush();
         SynchronizeCsThread();
         
-        while (Resource->isInUse())
+        while (Resource->isInUse(access))
           dxvk::this_thread::yield();
       }
     }
@@ -606,11 +634,16 @@ namespace dxvk {
   void D3D11ImmediateContext::FlushImplicit(BOOL StrongHint) {
     // Flush only if the GPU is about to go idle, in
     // order to keep the number of submissions low.
-    if (StrongHint || m_device->pendingSubmissions() <= MaxPendingSubmits) {
-      auto now = std::chrono::high_resolution_clock::now();
+    uint32_t pending = m_device->pendingSubmissions();
+
+    if (StrongHint || pending <= MaxPendingSubmits) {
+      auto now = dxvk::high_resolution_clock::now();
+
+      uint32_t delay = MinFlushIntervalUs
+                     + IncFlushIntervalUs * pending;
 
       // Prevent flushing too often in short intervals.
-      if (now - m_lastFlush >= std::chrono::microseconds(MinFlushIntervalUs))
+      if (now - m_lastFlush >= std::chrono::microseconds(delay))
         Flush();
     }
   }
