@@ -4,19 +4,14 @@
 
 #include "../dxgi/dxgi_options.h"
 
+#include "../dxvk/dxvk_device.h"
+
 #include "d3d11_include.h"
 
 namespace dxvk {
   
   struct D3D11Options {
-    D3D11Options(const Config& config);
-    /// Handle D3D11_MAP_FLAG_DO_NOT_WAIT properly.
-    /// 
-    /// This can offer substantial speedups, but some games
-    /// (The Witcher 3, Elder Scrolls Online, possibly others)
-    /// seem to make incorrect assumptions about when a map
-    /// operation succeeds when that flag is set.
-    bool allowMapFlagNoWait;
+    D3D11Options(const Config& config, const Rc<DxvkDevice>& device);
 
     /// Enables speed hack for mapping on deferred contexts
     ///
@@ -25,12 +20,27 @@ namespace dxvk {
     /// than once.
     bool dcSingleUseMode;
 
+    /// Enables workaround to replace NaN render target
+    /// outputs with zero
+    bool enableRtOutputNanFixup;
+
+    /// Enables out-of-bounds access check for constant
+    /// buffers. Workaround for a few broken games that
+    /// access random data inside their shaders.
+    bool constantBufferRangeCheck;
+
     /// Zero-initialize workgroup memory
     ///
     /// Workargound for games that don't initialize
     /// TGSM in compute shaders before reading it.
     bool zeroInitWorkgroupMemory;
 
+    /// Use relaxed memory barriers
+    ///
+    /// May improve performance in some games,
+    /// but might also cause rendering issues.
+    bool relaxedBarriers;
+
     /// Maximum tessellation factor.
     ///
     /// Limits tessellation factors in tessellation
@@ -44,6 +54,9 @@ namespace dxvk {
     /// given anisotropy value for all samplers.
     int32_t samplerAnisotropy;
     
+    /// Declare vertex positions in shaders as invariant
+    bool invariantPosition;
+
     /// Back buffer count for the Vulkan swap chain.
     /// Overrides DXGI_SWAP_CHAIN_DESC::BufferCount.
     int32_t numBackBuffers;
@@ -60,6 +73,10 @@ namespace dxvk {
     /// fixes issues with games that create multiple swap chains
     /// for a single window that may interfere with each other.
     bool deferSurfaceCreation;
+
+    /// Apitrace mode: Maps all buffers in cached memory.
+    /// Enabled automatically if dxgitrace.dll is attached.
+    bool apitraceMode;
   };
   
 }
\ No newline at end of file
