@@ -5,36 +5,24 @@ namespace dxvk {
   
   D3D11RasterizerState::D3D11RasterizerState(
           D3D11Device*                    device,
-    const D3D11_RASTERIZER_DESC1&         desc)
+    const D3D11_RASTERIZER_DESC2&         desc)
   : m_device(device), m_desc(desc), m_d3d10(this) {
     // Polygon mode. Determines whether the rasterizer fills
     // a polygon or renders lines connecting the vertices.
-    m_state.polygonMode = VK_POLYGON_MODE_FILL;
-    
     switch (desc.FillMode) {
-      case D3D11_FILL_WIREFRAME: m_state.polygonMode = VK_POLYGON_MODE_LINE; break;
-      case D3D11_FILL_SOLID:     m_state.polygonMode = VK_POLYGON_MODE_FILL; break;
-      
       default:
-        Logger::err(str::format(
-          "D3D11RasterizerState: Unsupported fill mode: ",
-          desc.FillMode));
+      case D3D11_FILL_SOLID:     m_state.polygonMode = VK_POLYGON_MODE_FILL; break;
+      case D3D11_FILL_WIREFRAME: m_state.polygonMode = VK_POLYGON_MODE_LINE; break;
     }
     
     // Face culling properties. The rasterizer may discard
     // polygons that are facing towards or away from the
     // viewer, depending on the options below.
-    m_state.cullMode = VK_CULL_MODE_NONE;
-    
     switch (desc.CullMode) {
+      default:
       case D3D11_CULL_NONE:  m_state.cullMode = VK_CULL_MODE_NONE;      break;
       case D3D11_CULL_FRONT: m_state.cullMode = VK_CULL_MODE_FRONT_BIT; break;
       case D3D11_CULL_BACK:  m_state.cullMode = VK_CULL_MODE_BACK_BIT;  break;
-      
-      default:
-        Logger::err(str::format(
-          "D3D11RasterizerState: Unsupported cull mode: ",
-          desc.CullMode));
     }
     
     m_state.frontFace = desc.FrontCounterClockwise
@@ -62,13 +50,33 @@ namespace dxvk {
   }
   
   
+  ULONG STDMETHODCALLTYPE D3D11RasterizerState::AddRef() {
+    ULONG refCount = m_refCount++;
+    if (!refCount)
+      m_device->AddRef();
+    return refCount + 1;
+  }
+
+
+  ULONG STDMETHODCALLTYPE D3D11RasterizerState::Release() {
+    ULONG refCount = --m_refCount;
+    if (!refCount)
+      m_device->Release();
+    return refCount;
+  }
+
+
   HRESULT STDMETHODCALLTYPE D3D11RasterizerState::QueryInterface(REFIID riid, void** ppvObject) {
+    if (ppvObject == nullptr)
+      return E_POINTER;
+
     *ppvObject = nullptr;
     
     if (riid == __uuidof(IUnknown)
      || riid == __uuidof(ID3D11DeviceChild)
      || riid == __uuidof(ID3D11RasterizerState)
-     || riid == __uuidof(ID3D11RasterizerState1)) {
+     || riid == __uuidof(ID3D11RasterizerState1)
+     || riid == __uuidof(ID3D11RasterizerState2)) {
       *ppvObject = ref(this);
       return S_OK;
     }
@@ -105,10 +113,25 @@ namespace dxvk {
   
   
   void STDMETHODCALLTYPE D3D11RasterizerState::GetDesc1(D3D11_RASTERIZER_DESC1* pDesc) {
-    *pDesc = m_desc;
+    pDesc->FillMode               = m_desc.FillMode;
+    pDesc->CullMode               = m_desc.CullMode;
+    pDesc->FrontCounterClockwise  = m_desc.FrontCounterClockwise;
+    pDesc->DepthBias              = m_desc.DepthBias;
+    pDesc->DepthBiasClamp         = m_desc.DepthBiasClamp;
+    pDesc->SlopeScaledDepthBias   = m_desc.SlopeScaledDepthBias;
+    pDesc->DepthClipEnable        = m_desc.DepthClipEnable;
+    pDesc->ScissorEnable          = m_desc.ScissorEnable;
+    pDesc->MultisampleEnable      = m_desc.MultisampleEnable;
+    pDesc->AntialiasedLineEnable  = m_desc.AntialiasedLineEnable;
+    pDesc->ForcedSampleCount      = m_desc.ForcedSampleCount;
   }
   
   
+  void STDMETHODCALLTYPE D3D11RasterizerState::GetDesc2(D3D11_RASTERIZER_DESC2* pDesc) {
+    *pDesc = m_desc;
+  }
+
+  
   void D3D11RasterizerState::BindToContext(const Rc<DxvkContext>& ctx) {
     ctx->setRasterizerState(m_state);
     
@@ -117,26 +140,9 @@ namespace dxvk {
   }
   
   
-  D3D11_RASTERIZER_DESC1 D3D11RasterizerState::DefaultDesc() {
-    D3D11_RASTERIZER_DESC1 dstDesc;
-    dstDesc.FillMode              = D3D11_FILL_SOLID;
-    dstDesc.CullMode              = D3D11_CULL_BACK;
-    dstDesc.FrontCounterClockwise = FALSE;
-    dstDesc.DepthBias             = 0;
-    dstDesc.SlopeScaledDepthBias  = 0.0f;
-    dstDesc.DepthBiasClamp        = 0.0f;
-    dstDesc.DepthClipEnable       = TRUE;
-    dstDesc.ScissorEnable         = FALSE;
-    dstDesc.MultisampleEnable     = FALSE;
-    dstDesc.AntialiasedLineEnable = FALSE;
-    dstDesc.ForcedSampleCount     = 0;
-    return dstDesc;
-  }
-  
-  
-  D3D11_RASTERIZER_DESC1 D3D11RasterizerState::PromoteDesc(
+  D3D11_RASTERIZER_DESC2 D3D11RasterizerState::PromoteDesc(
     const D3D11_RASTERIZER_DESC*  pSrcDesc) {
-    D3D11_RASTERIZER_DESC1 dstDesc;
+    D3D11_RASTERIZER_DESC2 dstDesc;
     dstDesc.FillMode              = pSrcDesc->FillMode;
     dstDesc.CullMode              = pSrcDesc->CullMode;
     dstDesc.FrontCounterClockwise = pSrcDesc->FrontCounterClockwise;
@@ -148,12 +154,32 @@ namespace dxvk {
     dstDesc.MultisampleEnable     = pSrcDesc->MultisampleEnable;
     dstDesc.AntialiasedLineEnable = pSrcDesc->AntialiasedLineEnable;
     dstDesc.ForcedSampleCount     = 0;
+    dstDesc.ConservativeRaster    = D3D11_CONSERVATIVE_RASTERIZATION_MODE_OFF;
+    return dstDesc;
+  }
+  
+  
+  D3D11_RASTERIZER_DESC2 D3D11RasterizerState::PromoteDesc(
+    const D3D11_RASTERIZER_DESC1*  pSrcDesc) {
+    D3D11_RASTERIZER_DESC2 dstDesc;
+    dstDesc.FillMode              = pSrcDesc->FillMode;
+    dstDesc.CullMode              = pSrcDesc->CullMode;
+    dstDesc.FrontCounterClockwise = pSrcDesc->FrontCounterClockwise;
+    dstDesc.DepthBias             = pSrcDesc->DepthBias;
+    dstDesc.DepthBiasClamp        = pSrcDesc->DepthBiasClamp;
+    dstDesc.SlopeScaledDepthBias  = pSrcDesc->SlopeScaledDepthBias;
+    dstDesc.DepthClipEnable       = pSrcDesc->DepthClipEnable;
+    dstDesc.ScissorEnable         = pSrcDesc->ScissorEnable;
+    dstDesc.MultisampleEnable     = pSrcDesc->MultisampleEnable;
+    dstDesc.AntialiasedLineEnable = pSrcDesc->AntialiasedLineEnable;
+    dstDesc.ForcedSampleCount     = 0;
+    dstDesc.ConservativeRaster    = D3D11_CONSERVATIVE_RASTERIZATION_MODE_OFF;
     return dstDesc;
   }
   
   
   HRESULT D3D11RasterizerState::NormalizeDesc(
-          D3D11_RASTERIZER_DESC1* pDesc) {
+          D3D11_RASTERIZER_DESC2* pDesc) {
     if (pDesc->FillMode < D3D11_FILL_WIREFRAME
      || pDesc->FillMode > D3D11_FILL_SOLID)
       return E_INVALIDARG;
@@ -181,6 +207,10 @@ namespace dxvk {
       if (FAILED(DecodeSampleCount(pDesc->ForcedSampleCount, nullptr)))
         return E_INVALIDARG;
     }
+
+    // Conservative rasterization currently not supported
+    if (pDesc->ConservativeRaster != D3D11_CONSERVATIVE_RASTERIZATION_MODE_OFF)
+      return E_INVALIDARG;
     
     return S_OK;
   }
