@@ -24,7 +24,10 @@ namespace dxvk {
     m_window    (hWnd),
     m_desc      (*pDesc),
     m_device    (pDevice->GetDXVKDevice()),
-    m_context   (m_device->createContext()) {
+    m_context   (m_device->createContext()),
+    m_frameLatencyCap(pDevice->GetOptions()->maxFrameLatency) {
+    CreateFrameLatencyEvent();
+
     if (!pDevice->GetOptions()->deferSurfaceCreation)
       CreatePresenter();
     
@@ -34,22 +37,26 @@ namespace dxvk {
     InitRenderState();
     InitSamplers();
     InitShaders();
-
-    SetGammaControl(0, nullptr);
   }
 
 
   D3D11SwapChain::~D3D11SwapChain() {
+    m_device->waitForSubmission(&m_presentStatus);
     m_device->waitForIdle();
     
     if (m_backBuffer)
       m_backBuffer->ReleasePrivate();
+
+    DestroyFrameLatencyEvent();
   }
 
 
   HRESULT STDMETHODCALLTYPE D3D11SwapChain::QueryInterface(
           REFIID                  riid,
           void**                  ppvObject) {
+    if (ppvObject == nullptr)
+      return E_POINTER;
+
     InitReturnPtr(ppvObject);
 
     if (riid == __uuidof(IUnknown)
@@ -104,13 +111,24 @@ namespace dxvk {
   }
 
 
+  UINT STDMETHODCALLTYPE D3D11SwapChain::GetFrameLatency() {
+    return m_frameLatency;
+  }
+
+
+  HANDLE STDMETHODCALLTYPE D3D11SwapChain::GetFrameLatencyEvent() {
+    return m_frameLatencyEvent;
+  }
+
+
   HRESULT STDMETHODCALLTYPE D3D11SwapChain::ChangeProperties(
     const DXGI_SWAP_CHAIN_DESC1*  pDesc) {
 
     m_dirty |= m_desc.Format      != pDesc->Format
             || m_desc.Width       != pDesc->Width
             || m_desc.Height      != pDesc->Height
-            || m_desc.BufferCount != pDesc->BufferCount;
+            || m_desc.BufferCount != pDesc->BufferCount
+            || m_desc.Flags       != pDesc->Flags;
 
     m_desc = *pDesc;
     CreateBackBuffer();
@@ -128,31 +146,47 @@ namespace dxvk {
   HRESULT STDMETHODCALLTYPE D3D11SwapChain::SetGammaControl(
           UINT                      NumControlPoints,
     const DXGI_RGB*                 pControlPoints) {
-    if (NumControlPoints > 0) {
+    bool isIdentity = true;
+
+    if (NumControlPoints > 1) {
       std::array<D3D11_VK_GAMMA_CP, 1025> cp;
 
       if (NumControlPoints > cp.size())
         return E_INVALIDARG;
       
       for (uint32_t i = 0; i < NumControlPoints; i++) {
+        uint16_t identity = MapGammaControlPoint(float(i) / float(NumControlPoints - 1));
+
         cp[i].R = MapGammaControlPoint(pControlPoints[i].Red);
         cp[i].G = MapGammaControlPoint(pControlPoints[i].Green);
         cp[i].B = MapGammaControlPoint(pControlPoints[i].Blue);
         cp[i].A = 0;
+
+        isIdentity &= cp[i].R == identity
+                   && cp[i].G == identity
+                   && cp[i].B == identity;
       }
 
-      CreateGammaTexture(NumControlPoints, cp.data());
-    } else {
-      std::array<D3D11_VK_GAMMA_CP, 256> cp;
-
-      for (uint32_t i = 0; i < cp.size(); i++) {
-        const uint16_t value = 257 * i;
-        cp[i] = { value, value, value, 0 };
-      }
-
-      CreateGammaTexture(cp.size(), cp.data());
+      if (!isIdentity)
+        CreateGammaTexture(NumControlPoints, cp.data());
     }
 
+    if (isIdentity)
+      DestroyGammaTexture();
+
+    return S_OK;
+  }
+
+
+  HRESULT STDMETHODCALLTYPE D3D11SwapChain::SetFrameLatency(
+          UINT                      MaxLatency) {
+    if (MaxLatency == 0 || MaxLatency > DXGI_MAX_SWAP_CHAIN_BUFFERS)
+      return DXGI_ERROR_INVALID_CALL;
+
+    m_frameLatency = MaxLatency;
+    m_frameLatencySignal->wait(m_frameId - GetActualFrameLatency());
+
+    SignalFrameLatencyEvent();
     return S_OK;
   }
 
@@ -165,58 +199,65 @@ namespace dxvk {
 
     if (options->syncInterval >= 0)
       SyncInterval = options->syncInterval;
-    
-    bool vsync = SyncInterval != 0;
 
-    m_dirty |= vsync != m_vsync;
-    m_vsync  = vsync;
+    if (!(PresentFlags & DXGI_PRESENT_TEST)) {
+      bool vsync = SyncInterval != 0;
+
+      m_dirty |= vsync != m_vsync;
+      m_vsync  = vsync;
+    }
 
     if (m_presenter == nullptr)
       CreatePresenter();
 
-    if (std::exchange(m_dirty, false))
-      RecreateSwapChain(vsync);
-    
-    FlushImmediateContext();
+    HRESULT hr = S_OK;
 
+    if (!m_presenter->hasSwapChain()) {
+      RecreateSwapChain(m_vsync);
+      m_dirty = false;
+    }
+
+    if (!m_presenter->hasSwapChain())
+      hr = DXGI_STATUS_OCCLUDED;
+
+    if (m_device->getDeviceStatus() != VK_SUCCESS)
+      hr = DXGI_ERROR_DEVICE_RESET;
+
+    if ((PresentFlags & DXGI_PRESENT_TEST) || hr != S_OK)
+      return hr;
+
+    if (std::exchange(m_dirty, false))
+      RecreateSwapChain(m_vsync);
+    
     try {
       PresentImage(SyncInterval);
-      return S_OK;
     } catch (const DxvkError& e) {
       Logger::err(e.message());
-      return E_FAIL;
+      hr = E_FAIL;
     }
+
+    return hr;
   }
 
 
-  void D3D11SwapChain::PresentImage(UINT SyncInterval) {
-    // Wait for the sync event so that we
-    // respect the maximum frame latency
-    Rc<DxvkEvent> syncEvent = m_dxgiDevice->GetFrameSyncEvent();
-    syncEvent->wait();
-    
-    if (m_hud != nullptr)
-      m_hud->update();
+  HRESULT D3D11SwapChain::PresentImage(UINT SyncInterval) {
+    Com<ID3D11DeviceContext> deviceContext = nullptr;
+    m_parent->GetImmediateContext(&deviceContext);
 
+    // Flush pending rendering commands before
+    auto immediateContext = static_cast<D3D11ImmediateContext*>(deviceContext.ptr());
+    immediateContext->Flush();
+
+    // Wait for the sync event so that we respect the maximum frame latency
+    uint64_t frameId = ++m_frameId;
+    m_frameLatencySignal->wait(frameId - GetActualFrameLatency());
+    
     for (uint32_t i = 0; i < SyncInterval || i < 1; i++) {
-      m_context->beginRecording(
-        m_device->createCommandList());
-      
-      // Resolve back buffer if it is multisampled. We
-      // only have to do it only for the first frame.
-      if (m_swapImageResolve != nullptr && i == 0) {
-        VkImageSubresourceLayers resolveSubresources;
-        resolveSubresources.aspectMask      = VK_IMAGE_ASPECT_COLOR_BIT;
-        resolveSubresources.mipLevel        = 0;
-        resolveSubresources.baseArrayLayer  = 0;
-        resolveSubresources.layerCount      = 1;
-        
-        m_context->resolveImage(
-          m_swapImageResolve, resolveSubresources,
-          m_swapImage,        resolveSubresources,
-          VK_FORMAT_UNDEFINED);
-      }
-      
+      SynchronizePresent();
+
+      if (!m_presenter->hasSwapChain())
+        return DXGI_STATUS_OCCLUDED;
+
       // Presentation semaphores and WSI swap chain image
       vk::PresenterInfo info = m_presenter->info();
       vk::PresenterSync sync = m_presenter->getSyncSemaphores();
@@ -224,18 +265,45 @@ namespace dxvk {
       uint32_t imageIndex = 0;
 
       VkResult status = m_presenter->acquireNextImage(
-        sync.acquire, imageIndex);
+        sync.acquire, VK_NULL_HANDLE, imageIndex);
 
       while (status != VK_SUCCESS && status != VK_SUBOPTIMAL_KHR) {
         RecreateSwapChain(m_vsync);
+
+        if (!m_presenter->hasSwapChain())
+          return DXGI_STATUS_OCCLUDED;
         
         info = m_presenter->info();
         sync = m_presenter->getSyncSemaphores();
 
         status = m_presenter->acquireNextImage(
-          sync.acquire, imageIndex);
+          sync.acquire, VK_NULL_HANDLE, imageIndex);
       }
 
+      // Resolve back buffer if it is multisampled. We
+      // only have to do it only for the first frame.
+      m_context->beginRecording(
+        m_device->createCommandList());
+      
+      if (m_swapImageResolve != nullptr && i == 0) {
+        VkImageSubresourceLayers resolveSubresource;
+        resolveSubresource.aspectMask      = VK_IMAGE_ASPECT_COLOR_BIT;
+        resolveSubresource.mipLevel        = 0;
+        resolveSubresource.baseArrayLayer  = 0;
+        resolveSubresource.layerCount      = 1;
+
+        VkImageResolve resolveRegion;
+        resolveRegion.srcSubresource = resolveSubresource;
+        resolveRegion.srcOffset      = VkOffset3D { 0, 0, 0 };
+        resolveRegion.dstSubresource = resolveSubresource;
+        resolveRegion.dstOffset      = VkOffset3D { 0, 0, 0 };
+        resolveRegion.extent         = m_swapImage->info().extent;
+        
+        m_context->resolveImage(
+          m_swapImageResolve, m_swapImage,
+          resolveRegion, VK_FORMAT_UNDEFINED);
+      }
+      
       // Use an appropriate texture filter depending on whether
       // the back buffer size matches the swap image size
       bool fitSize = m_swapImage->info().extent.width  == info.imageExtent.width
@@ -247,7 +315,7 @@ namespace dxvk {
       DxvkRenderTargets renderTargets;
       renderTargets.color[0].view   = m_imageViews.at(imageIndex);
       renderTargets.color[0].layout = VK_IMAGE_LAYOUT_COLOR_ATTACHMENT_OPTIMAL;
-      m_context->bindRenderTargets(renderTargets, false);
+      m_context->bindRenderTargets(renderTargets);
 
       VkViewport viewport;
       viewport.x        = 0.0f;
@@ -274,43 +342,80 @@ namespace dxvk {
       m_context->setInputAssemblyState(m_iaState);
       m_context->setInputLayout(0, nullptr, 0, nullptr);
 
-      m_context->bindResourceSampler(BindingIds::Sampler, fitSize ? m_samplerFitting : m_samplerScaling);
-      m_context->bindResourceSampler(BindingIds::GammaSmp, m_gammaSampler);
+      m_context->bindResourceSampler(BindingIds::Image, fitSize ? m_samplerFitting : m_samplerScaling);
+      m_context->bindResourceSampler(BindingIds::Gamma, m_gammaSampler);
 
-      m_context->bindResourceView(BindingIds::Texture, m_swapImageView, nullptr);
-      m_context->bindResourceView(BindingIds::GammaTex, m_gammaTextureView, nullptr);
+      m_context->bindResourceView(BindingIds::Image, m_swapImageView, nullptr);
+      m_context->bindResourceView(BindingIds::Gamma, m_gammaTextureView, nullptr);
 
-      m_context->draw(4, 1, 0, 0);
+      m_context->draw(3, 1, 0, 0);
 
       if (m_hud != nullptr)
-        m_hud->render(m_context, info.imageExtent);
+        m_hud->render(m_context, info.format, info.imageExtent);
       
-      if (i + 1 >= SyncInterval) {
-        DxvkEventRevision eventRev;
-        eventRev.event    = syncEvent;
-        eventRev.revision = syncEvent->reset();
-        m_context->signalEvent(eventRev);
-      }
+      if (i + 1 >= SyncInterval)
+        m_context->signal(m_frameLatencySignal, frameId);
 
-      m_device->submitCommandList(
-        m_context->endRecording(),
-        sync.acquire, sync.present);
-      
-      status = m_device->presentImage(
-        m_presenter, sync.present);
-      
-      if (status != VK_SUCCESS)
-        RecreateSwapChain(m_vsync);
+      SubmitPresent(immediateContext, sync, i);
     }
+
+    SignalFrameLatencyEvent();
+    return S_OK;
   }
 
-  
+
+  void D3D11SwapChain::SubmitPresent(
+          D3D11ImmediateContext*  pContext,
+    const vk::PresenterSync&      Sync,
+          uint32_t                FrameId) {
+    auto lock = pContext->LockContext();
+
+    // Present from CS thread so that we don't
+    // have to synchronize with it first.
+    m_presentStatus.result = VK_NOT_READY;
+
+    pContext->EmitCs([this,
+      cFrameId     = FrameId,
+      cSync        = Sync,
+      cHud         = m_hud,
+      cCommandList = m_context->endRecording()
+    ] (DxvkContext* ctx) {
+      m_device->submitCommandList(cCommandList,
+        cSync.acquire, cSync.present);
+
+      if (cHud != nullptr && !cFrameId)
+        cHud->update();
+
+      m_device->presentImage(m_presenter,
+        cSync.present, &m_presentStatus);
+    });
+
+    pContext->FlushCsChunk();
+  }
+
+
+  void D3D11SwapChain::SynchronizePresent() {
+    // Recreate swap chain if the previous present call failed
+    VkResult status = m_device->waitForSubmission(&m_presentStatus);
+    
+    if (status != VK_SUCCESS)
+      RecreateSwapChain(m_vsync);
+  }
+
+
   void D3D11SwapChain::RecreateSwapChain(BOOL Vsync) {
+    // Ensure that we can safely destroy the swap chain
+    m_device->waitForSubmission(&m_presentStatus);
+    m_device->waitForIdle();
+
+    m_presentStatus.result = VK_SUCCESS;
+
     vk::PresenterDesc presenterDesc;
     presenterDesc.imageExtent     = { m_desc.Width, m_desc.Height };
-    presenterDesc.imageCount      = PickImageCount(m_desc.BufferCount);
+    presenterDesc.imageCount      = PickImageCount(m_desc.BufferCount + 1);
     presenterDesc.numFormats      = PickFormats(m_desc.Format, presenterDesc.formats);
     presenterDesc.numPresentModes = PickPresentModes(Vsync, presenterDesc.presentModes);
+    presenterDesc.fullScreenExclusive = PickFullscreenMode();
 
     if (m_presenter->recreateSwapChain(presenterDesc) != VK_SUCCESS)
       throw DxvkError("D3D11SwapChain: Failed to recreate swap chain");
@@ -319,19 +424,29 @@ namespace dxvk {
   }
 
 
+  void D3D11SwapChain::CreateFrameLatencyEvent() {
+    m_frameLatencySignal = new sync::Win32Fence(m_frameId);
+
+    if (m_desc.Flags & DXGI_SWAP_CHAIN_FLAG_FRAME_LATENCY_WAITABLE_OBJECT)
+      m_frameLatencyEvent = CreateEvent(nullptr, false, true, nullptr);
+  }
+
+
   void D3D11SwapChain::CreatePresenter() {
-    DxvkDeviceQueue graphicsQueue = m_device->graphicsQueue();
+    DxvkDeviceQueue graphicsQueue = m_device->queues().graphics;
 
     vk::PresenterDevice presenterDevice;
     presenterDevice.queueFamily   = graphicsQueue.queueFamily;
     presenterDevice.queue         = graphicsQueue.queueHandle;
     presenterDevice.adapter       = m_device->adapter()->handle();
+    presenterDevice.features.fullScreenExclusive = m_device->extensions().extFullScreenExclusive;
 
     vk::PresenterDesc presenterDesc;
     presenterDesc.imageExtent     = { m_desc.Width, m_desc.Height };
-    presenterDesc.imageCount      = PickImageCount(m_desc.BufferCount);
+    presenterDesc.imageCount      = PickImageCount(m_desc.BufferCount + 1);
     presenterDesc.numFormats      = PickFormats(m_desc.Format, presenterDesc.formats);
     presenterDesc.numPresentModes = PickPresentModes(false, presenterDesc.presentModes);
+    presenterDesc.fullScreenExclusive = PickFullscreenMode();
 
     m_presenter = new vk::Presenter(m_window,
       m_device->adapter()->vki(),
@@ -385,18 +500,6 @@ namespace dxvk {
   }
 
 
-  void D3D11SwapChain::FlushImmediateContext() {
-    Com<ID3D11DeviceContext> deviceContext = nullptr;
-    m_parent->GetImmediateContext(&deviceContext);
-    
-    // The presentation code is run from the main rendering thread
-    // rather than the command stream thread, so we synchronize.
-    auto immediateContext = static_cast<D3D11ImmediateContext*>(deviceContext.ptr());
-    immediateContext->Flush();
-    immediateContext->SynchronizeCsThread();
-  }
-
-
   void D3D11SwapChain::CreateBackBuffer() {
     // Explicitly destroy current swap image before
     // creating a new one to free up resources
@@ -422,10 +525,14 @@ namespace dxvk {
                             | D3D11_BIND_SHADER_RESOURCE;
     desc.CPUAccessFlags     = 0;
     desc.MiscFlags          = 0;
+    desc.TextureLayout      = D3D11_TEXTURE_LAYOUT_UNDEFINED;
 
     if (m_desc.BufferUsage & DXGI_USAGE_UNORDERED_ACCESS)
       desc.BindFlags |= D3D11_BIND_UNORDERED_ACCESS;
     
+    if (m_desc.Flags & DXGI_SWAP_CHAIN_FLAG_GDI_COMPATIBLE)
+      desc.MiscFlags |= D3D11_RESOURCE_MISC_GDI_COMPATIBLE;
+    
     m_backBuffer = new D3D11Texture2D(m_parent, &desc);
     m_backBuffer->AddRefPrivate();
 
@@ -559,8 +666,22 @@ namespace dxvk {
   }
 
 
+  void D3D11SwapChain::DestroyFrameLatencyEvent() {
+    CloseHandle(m_frameLatencyEvent);
+  }
+
+
+  void D3D11SwapChain::DestroyGammaTexture() {
+    m_gammaTexture     = nullptr;
+    m_gammaTextureView = nullptr;
+  }
+
+
   void D3D11SwapChain::CreateHud() {
     m_hud = hud::Hud::createHud(m_device);
+
+    if (m_hud != nullptr)
+      m_hud->addItem<hud::HudClientApiItem>("api", 1, GetApiName());
   }
 
 
@@ -578,7 +699,6 @@ namespace dxvk {
     
     m_msState.sampleMask            = 0xffffffff;
     m_msState.enableAlphaToCoverage = VK_FALSE;
-    m_msState.enableAlphaToOne      = VK_FALSE;
     
     VkStencilOpState stencilOp;
     stencilOp.failOp      = VK_STENCIL_OP_KEEP;
@@ -598,7 +718,7 @@ namespace dxvk {
     
     m_loState.enableLogicOp = VK_FALSE;
     m_loState.logicOp       = VK_LOGIC_OP_NO_OP;
-    
+
     m_blendMode.enableBlending  = VK_FALSE;
     m_blendMode.colorSrcFactor  = VK_BLEND_FACTOR_ONE;
     m_blendMode.colorDstFactor  = VK_BLEND_FACTOR_ONE_MINUS_SRC_ALPHA;
@@ -647,11 +767,9 @@ namespace dxvk {
     const SpirvCodeBuffer vsCode(dxgi_presenter_vert);
     const SpirvCodeBuffer fsCode(dxgi_presenter_frag);
     
-    const std::array<DxvkResourceSlot, 4> fsResourceSlots = {{
-      { BindingIds::Sampler,  VK_DESCRIPTOR_TYPE_SAMPLER,        VK_IMAGE_VIEW_TYPE_MAX_ENUM },
-      { BindingIds::Texture,  VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE,  VK_IMAGE_VIEW_TYPE_2D       },
-      { BindingIds::GammaSmp, VK_DESCRIPTOR_TYPE_SAMPLER,        VK_IMAGE_VIEW_TYPE_MAX_ENUM },
-      { BindingIds::GammaTex, VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE,  VK_IMAGE_VIEW_TYPE_1D       },
+    const std::array<DxvkResourceSlot, 2> fsResourceSlots = {{
+      { BindingIds::Image, VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER, VK_IMAGE_VIEW_TYPE_2D },
+      { BindingIds::Gamma, VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER, VK_IMAGE_VIEW_TYPE_1D },
     }};
 
     m_vertShader = m_device->createShader(
@@ -667,6 +785,28 @@ namespace dxvk {
   }
 
 
+  void D3D11SwapChain::SignalFrameLatencyEvent() {
+    if (m_frameLatencyEvent) {
+      // Signal event with the same value that we'd wait for during the next present.
+      m_frameLatencySignal->setEvent(m_frameLatencyEvent, m_frameId - GetActualFrameLatency() + 1);
+    }
+  }
+
+
+  uint32_t D3D11SwapChain::GetActualFrameLatency() {
+    uint32_t maxFrameLatency = m_frameLatency;
+
+    if (!(m_desc.Flags & DXGI_SWAP_CHAIN_FLAG_FRAME_LATENCY_WAITABLE_OBJECT))
+      m_dxgiDevice->GetMaximumFrameLatency(&maxFrameLatency);
+
+    if (m_frameLatencyCap)
+      maxFrameLatency = std::min(maxFrameLatency, m_frameLatencyCap);
+
+    maxFrameLatency = std::min(maxFrameLatency, m_desc.BufferCount + 1);
+    return maxFrameLatency;
+  }
+
+
   uint32_t D3D11SwapChain::PickFormats(
           DXGI_FORMAT               Format,
           VkSurfaceFormatKHR*       pDstFormats) {
@@ -725,4 +865,25 @@ namespace dxvk {
     return option > 0 ? uint32_t(option) : uint32_t(Preferred);
   }
 
+
+  VkFullScreenExclusiveEXT D3D11SwapChain::PickFullscreenMode() {
+    return m_desc.Flags & DXGI_SWAP_CHAIN_FLAG_ALLOW_MODE_SWITCH
+      ? VK_FULL_SCREEN_EXCLUSIVE_ALLOWED_EXT
+      : VK_FULL_SCREEN_EXCLUSIVE_DISALLOWED_EXT;
+  }
+
+
+  std::string D3D11SwapChain::GetApiName() const {
+    Com<IDXGIDXVKDevice> device;
+    m_parent->QueryInterface(__uuidof(IDXGIDXVKDevice), reinterpret_cast<void**>(&device));
+
+    uint32_t apiVersion = device->GetAPIVersion();
+    uint32_t featureLevel = m_parent->GetFeatureLevel();
+
+    uint32_t flHi = (featureLevel >> 12);
+    uint32_t flLo = (featureLevel >> 8) & 0x7;
+
+    return str::format("D3D", apiVersion, " FL", flHi, "_", flLo);
+  }
+
 }
\ No newline at end of file
