@@ -1,4 +1,5 @@
 #include "d3d11_device.h"
+#include "d3d11_gdi.h"
 #include "d3d11_texture.h"
 
 namespace dxvk {
@@ -29,6 +30,7 @@ namespace dxvk {
                               | VK_ACCESS_TRANSFER_WRITE_BIT;
     imageInfo.tiling          = VK_IMAGE_TILING_OPTIMAL;
     imageInfo.layout          = VK_IMAGE_LAYOUT_GENERAL;
+    imageInfo.initialLayout   = VK_IMAGE_LAYOUT_UNDEFINED;
 
     DecodeSampleCount(m_desc.SampleDesc.Count, &imageInfo.sampleCount);
 
@@ -112,7 +114,7 @@ namespace dxvk {
     
     if (Dimension == D3D11_RESOURCE_DIMENSION_TEXTURE3D &&
         (m_desc.BindFlags & D3D11_BIND_RENDER_TARGET))
-      imageInfo.flags |= VK_IMAGE_CREATE_2D_ARRAY_COMPATIBLE_BIT_KHR;
+      imageInfo.flags |= VK_IMAGE_CREATE_2D_ARRAY_COMPATIBLE_BIT;
     
     // Some image formats (i.e. the R32G32B32 ones) are
     // only supported with linear tiling on most GPUs
@@ -126,14 +128,8 @@ namespace dxvk {
     // to enable linear tiling, and DXVK needs to be aware that
     // the image can be accessed by the host.
     if (m_mapMode == D3D11_COMMON_TEXTURE_MAP_MODE_DIRECT) {
-      imageInfo.stages |= VK_PIPELINE_STAGE_HOST_BIT;
-      imageInfo.tiling  = VK_IMAGE_TILING_LINEAR;
-      
-      if (m_desc.CPUAccessFlags & D3D11_CPU_ACCESS_WRITE)
-        imageInfo.access |= VK_ACCESS_HOST_WRITE_BIT;
-      
-      if (m_desc.CPUAccessFlags & D3D11_CPU_ACCESS_READ)
-        imageInfo.access |= VK_ACCESS_HOST_READ_BIT;
+      imageInfo.tiling        = VK_IMAGE_TILING_LINEAR;
+      imageInfo.initialLayout = VK_IMAGE_LAYOUT_PREINITIALIZED;
     }
     
     // We must keep LINEAR images in GENERAL layout, but we
@@ -164,8 +160,14 @@ namespace dxvk {
     }
     
     // If necessary, create the mapped linear buffer
-    if (m_mapMode == D3D11_COMMON_TEXTURE_MAP_MODE_BUFFER)
-      m_buffer = CreateMappedBuffer();
+    for (uint32_t i = 0; i < m_desc.ArraySize; i++) {
+      for (uint32_t j = 0; j < m_desc.MipLevels; j++) {
+        if (m_mapMode == D3D11_COMMON_TEXTURE_MAP_MODE_BUFFER)
+          m_buffers.push_back(CreateMappedBuffer(j));
+        if (m_mapMode != D3D11_COMMON_TEXTURE_MAP_MODE_NONE)
+          m_mapTypes.push_back(D3D11_MAP(~0u));
+      }
+    }
     
     // Create the image on a host-visible memory type
     // in case it is going to be mapped directly.
@@ -265,12 +267,28 @@ namespace dxvk {
   
   
   HRESULT D3D11CommonTexture::NormalizeTextureProperties(D3D11_COMMON_TEXTURE_DESC* pDesc) {
-    if (pDesc->Width == 0 || pDesc->Height == 0 || pDesc->Depth == 0)
+    if (pDesc->Width == 0 || pDesc->Height == 0 || pDesc->Depth == 0 || pDesc->ArraySize == 0)
       return E_INVALIDARG;
     
     if (FAILED(DecodeSampleCount(pDesc->SampleDesc.Count, nullptr)))
       return E_INVALIDARG;
     
+    if ((pDesc->MiscFlags & D3D11_RESOURCE_MISC_GDI_COMPATIBLE)
+     && (pDesc->Usage == D3D11_USAGE_STAGING
+      || (pDesc->Format != DXGI_FORMAT_B8G8R8A8_TYPELESS
+       && pDesc->Format != DXGI_FORMAT_B8G8R8A8_UNORM
+       && pDesc->Format != DXGI_FORMAT_B8G8R8A8_UNORM_SRGB)))
+      return E_INVALIDARG;
+
+    if ((pDesc->MiscFlags & D3D11_RESOURCE_MISC_GENERATE_MIPS)
+     && (pDesc->BindFlags & (D3D11_BIND_SHADER_RESOURCE | D3D11_BIND_RENDER_TARGET))
+                         != (D3D11_BIND_SHADER_RESOURCE | D3D11_BIND_RENDER_TARGET))
+      return E_INVALIDARG;
+
+    // TILE_POOL is invalid, but we don't support TILED either
+    if (pDesc->MiscFlags & (D3D11_RESOURCE_MISC_TILE_POOL | D3D11_RESOURCE_MISC_TILED))
+      return E_INVALIDARG;
+
     // Use the maximum possible mip level count if the supplied
     // mip level count is either unspecified (0) or invalid
     const uint32_t maxMipLevelCount = pDesc->SampleDesc.Count <= 1
@@ -280,6 +298,16 @@ namespace dxvk {
     if (pDesc->MipLevels == 0 || pDesc->MipLevels > maxMipLevelCount)
       pDesc->MipLevels = maxMipLevelCount;
     
+    // Row-major is only supported for textures with one single
+    // subresource and one sample and cannot have bind flags.
+    if (pDesc->TextureLayout == D3D11_TEXTURE_LAYOUT_ROW_MAJOR
+     && (pDesc->MipLevels != 1 || pDesc->SampleDesc.Count != 1 || pDesc->BindFlags))
+      return E_INVALIDARG;
+
+    // Standard swizzle is unsupported
+    if (pDesc->TextureLayout == D3D11_TEXTURE_LAYOUT_64K_STANDARD_SWIZZLE)
+      return E_INVALIDARG;
+
     return S_OK;
   }
   
@@ -332,6 +360,9 @@ namespace dxvk {
                         |  VK_FORMAT_FEATURE_COLOR_ATTACHMENT_BIT;
     }
 
+    if (Format == VK_FORMAT_D32_SFLOAT_S8_UINT || Format == VK_FORMAT_D24_UNORM_S8_UINT)
+      requestedFeatures |= VK_FORMAT_FEATURE_DEPTH_STENCIL_ATTACHMENT_BIT;
+
     if (requestedFeatures == 0)
       return 0;
 
@@ -386,7 +417,9 @@ namespace dxvk {
     // 2. Since the image will most likely be read for rendering by the GPU,
     //    writing the image to device-local image may be more efficient than
     //    reading its contents from host-visible memory.
-    if (m_desc.Usage == D3D11_USAGE_DYNAMIC)
+    if (m_desc.Usage         == D3D11_USAGE_DYNAMIC
+     && m_desc.BindFlags     != 0
+     && m_desc.TextureLayout != D3D11_TEXTURE_LAYOUT_ROW_MAJOR)
       return D3D11_COMMON_TEXTURE_MAP_MODE_BUFFER;
     
     // Depth-stencil formats in D3D11 can be mapped and follow special
@@ -404,13 +437,16 @@ namespace dxvk {
   }
   
   
-  Rc<DxvkBuffer> D3D11CommonTexture::CreateMappedBuffer() const {
+  Rc<DxvkBuffer> D3D11CommonTexture::CreateMappedBuffer(UINT MipLevel) const {
     const DxvkFormatInfo* formatInfo = imageFormatInfo(
-      m_device->LookupFormat(m_desc.Format, GetFormatMode()).Format);
+      m_device->LookupPackedFormat(m_desc.Format, GetFormatMode()).Format);
+    
+    const VkExtent3D mipExtent = util::computeMipLevelExtent(
+      VkExtent3D { m_desc.Width, m_desc.Height, m_desc.Depth },
+      MipLevel);
     
     const VkExtent3D blockCount = util::computeBlockCount(
-      VkExtent3D { m_desc.Width, m_desc.Height, m_desc.Depth },
-      formatInfo->blockSize);
+      mipExtent, formatInfo->blockSize);
     
     DxvkBufferCreateInfo info;
     info.size   = formatInfo->elementSize
@@ -418,14 +454,19 @@ namespace dxvk {
                 * blockCount.height
                 * blockCount.depth;
     info.usage  = VK_BUFFER_USAGE_TRANSFER_SRC_BIT
-                | VK_BUFFER_USAGE_TRANSFER_DST_BIT;
+                | VK_BUFFER_USAGE_TRANSFER_DST_BIT
+                | VK_BUFFER_USAGE_STORAGE_BUFFER_BIT;
     info.stages = VK_PIPELINE_STAGE_TRANSFER_BIT;
     info.access = VK_ACCESS_TRANSFER_READ_BIT
                 | VK_ACCESS_TRANSFER_WRITE_BIT;
     
-    return m_device->GetDXVKDevice()->createBuffer(info,
-      VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT |
-      VK_MEMORY_PROPERTY_HOST_COHERENT_BIT);
+    VkMemoryPropertyFlags memType = VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT
+                                  | VK_MEMORY_PROPERTY_HOST_COHERENT_BIT;
+    
+    if (m_desc.Usage == D3D11_USAGE_STAGING)
+      memType |= VK_MEMORY_PROPERTY_HOST_CACHED_BIT;
+    
+    return m_device->GetDXVKDevice()->createBuffer(info, memType);
   }
   
   
@@ -471,11 +512,192 @@ namespace dxvk {
   }
   
   
-  D3D11VkInteropSurface::D3D11VkInteropSurface(
-          ID3D11DeviceChild*  pContainer,
+
+
+  D3D11DXGISurface::D3D11DXGISurface(
+          ID3D11Resource*     pResource,
           D3D11CommonTexture* pTexture)
-  : m_container (pContainer),
-    m_texture   (pTexture) {
+  : m_resource  (pResource),
+    m_texture   (pTexture),
+    m_gdiSurface(nullptr) {
+    if (pTexture->Desc()->MiscFlags & D3D11_RESOURCE_MISC_GDI_COMPATIBLE)
+      m_gdiSurface = new D3D11GDISurface(m_resource, 0);
+  }
+
+  
+  D3D11DXGISurface::~D3D11DXGISurface() {
+    if (m_gdiSurface)
+      delete m_gdiSurface;
+  }
+
+  
+  ULONG STDMETHODCALLTYPE D3D11DXGISurface::AddRef() {
+    return m_resource->AddRef();
+  }
+
+  
+  ULONG STDMETHODCALLTYPE D3D11DXGISurface::Release() {
+    return m_resource->Release();
+  }
+
+  
+  HRESULT STDMETHODCALLTYPE D3D11DXGISurface::QueryInterface(
+          REFIID                  riid,
+          void**                  ppvObject) {
+    return m_resource->QueryInterface(riid, ppvObject);
+  }
+
+
+  HRESULT STDMETHODCALLTYPE D3D11DXGISurface::GetPrivateData(
+          REFGUID                 Name,
+          UINT*                   pDataSize,
+          void*                   pData) {
+    return m_resource->GetPrivateData(Name, pDataSize, pData);
+  }
+
+  
+  HRESULT STDMETHODCALLTYPE D3D11DXGISurface::SetPrivateData(
+          REFGUID                 Name,
+          UINT                    DataSize,
+    const void*                   pData) {
+    return m_resource->SetPrivateData(Name, DataSize, pData);
+  }
+
+  
+  HRESULT STDMETHODCALLTYPE D3D11DXGISurface::SetPrivateDataInterface(
+          REFGUID                 Name,
+    const IUnknown*               pUnknown) {
+    return m_resource->SetPrivateDataInterface(Name, pUnknown);
+  }
+
+  
+  HRESULT STDMETHODCALLTYPE D3D11DXGISurface::GetParent(
+          REFIID                  riid,
+          void**                  ppParent) {
+    return GetDevice(riid, ppParent);
+  }
+
+  
+  HRESULT STDMETHODCALLTYPE D3D11DXGISurface::GetDevice(
+          REFIID                  riid,
+          void**                  ppDevice) {
+    Com<ID3D11Device> device;
+    m_resource->GetDevice(&device);
+    return device->QueryInterface(riid, ppDevice);
+  }
+
+  
+  HRESULT STDMETHODCALLTYPE D3D11DXGISurface::GetDesc(
+          DXGI_SURFACE_DESC*      pDesc) {
+    if (!pDesc)
+      return DXGI_ERROR_INVALID_CALL;
+
+    auto desc = m_texture->Desc();
+    pDesc->Width      = desc->Width;
+    pDesc->Height     = desc->Height;
+    pDesc->Format     = desc->Format;
+    pDesc->SampleDesc = desc->SampleDesc;
+    return S_OK;
+  }
+
+  
+  HRESULT STDMETHODCALLTYPE D3D11DXGISurface::Map(
+            DXGI_MAPPED_RECT*       pLockedRect,
+            UINT                    MapFlags) {
+    Com<ID3D11Device>        device;
+    Com<ID3D11DeviceContext> context;
+
+    m_resource->GetDevice(&device);
+    device->GetImmediateContext(&context);
+
+    if (pLockedRect) {
+      pLockedRect->Pitch = 0;
+      pLockedRect->pBits = nullptr;
+    }
+
+    D3D11_MAP mapType;
+
+    if (MapFlags & (DXGI_MAP_READ | DXGI_MAP_WRITE))
+      mapType = D3D11_MAP_READ_WRITE;
+    else if (MapFlags & DXGI_MAP_READ)
+      mapType = D3D11_MAP_READ;
+    else if (MapFlags & (DXGI_MAP_WRITE | DXGI_MAP_DISCARD))
+      mapType = D3D11_MAP_WRITE_DISCARD;
+    else if (MapFlags & DXGI_MAP_WRITE)
+      mapType = D3D11_MAP_WRITE;
+    else
+      return DXGI_ERROR_INVALID_CALL;
+    
+    D3D11_MAPPED_SUBRESOURCE sr;
+    HRESULT hr = context->Map(m_resource, 0,
+      mapType, 0, pLockedRect ? &sr : nullptr);
+
+    if (hr != S_OK)
+      return hr;
+
+    pLockedRect->Pitch = sr.RowPitch;
+    pLockedRect->pBits = reinterpret_cast<unsigned char*>(sr.pData);
+    return hr;
+  }
+
+  
+  HRESULT STDMETHODCALLTYPE D3D11DXGISurface::Unmap() {
+    Com<ID3D11Device>        device;
+    Com<ID3D11DeviceContext> context;
+
+    m_resource->GetDevice(&device);
+    device->GetImmediateContext(&context);
+    
+    context->Unmap(m_resource, 0);
+    return S_OK;
+  }
+
+
+  HRESULT STDMETHODCALLTYPE D3D11DXGISurface::GetDC(
+          BOOL                    Discard,
+          HDC*                    phdc) {
+    if (!m_gdiSurface)
+      return DXGI_ERROR_INVALID_CALL;
+    
+    return m_gdiSurface->Acquire(Discard, phdc);
+  }
+
+
+  HRESULT STDMETHODCALLTYPE D3D11DXGISurface::ReleaseDC(
+          RECT*                   pDirtyRect) {
+    if (!m_gdiSurface)
+      return DXGI_ERROR_INVALID_CALL;
+
+    return m_gdiSurface->Release(pDirtyRect);
+  }
+
+  
+  HRESULT STDMETHODCALLTYPE D3D11DXGISurface::GetResource(
+          REFIID                  riid,
+          void**                  ppParentResource,
+          UINT*                   pSubresourceIndex) {
+    HRESULT hr = m_resource->QueryInterface(riid, ppParentResource);
+    if (pSubresourceIndex)
+      *pSubresourceIndex = 0;
+    return hr;
+  }
+  
+  
+  bool D3D11DXGISurface::isSurfaceCompatible() const {
+    auto desc = m_texture->Desc();
+
+    return desc->ArraySize == 1
+        && desc->MipLevels == 1;
+  }
+
+
+
+
+  D3D11VkInteropSurface::D3D11VkInteropSurface(
+          ID3D11Resource*     pResource,
+          D3D11CommonTexture* pTexture)
+  : m_resource(pResource),
+    m_texture (pTexture) {
       
   }
   
@@ -486,26 +708,26 @@ namespace dxvk {
   
   
   ULONG STDMETHODCALLTYPE D3D11VkInteropSurface::AddRef() {
-    return m_container->AddRef();
+    return m_resource->AddRef();
   }
   
   
   ULONG STDMETHODCALLTYPE D3D11VkInteropSurface::Release() {
-    return m_container->Release();
+    return m_resource->Release();
   }
   
   
   HRESULT STDMETHODCALLTYPE D3D11VkInteropSurface::QueryInterface(
           REFIID                  riid,
           void**                  ppvObject) {
-    return m_container->QueryInterface(riid, ppvObject);
+    return m_resource->QueryInterface(riid, ppvObject);
   }
   
   
   HRESULT STDMETHODCALLTYPE D3D11VkInteropSurface::GetDevice(
           IDXGIVkInteropDevice**  ppDevice) {
     Com<ID3D11Device> device;
-    m_container->GetDevice(&device);
+    m_resource->GetDevice(&device);
     
     return device->QueryInterface(
       __uuidof(IDXGIVkInteropDevice),
@@ -557,7 +779,9 @@ namespace dxvk {
     const D3D11_COMMON_TEXTURE_DESC*  pDesc)
   : m_texture (pDevice, pDesc, D3D11_RESOURCE_DIMENSION_TEXTURE1D),
     m_interop (this, &m_texture),
-    m_d3d10   (this, pDevice->GetD3D10Interface()) {
+    m_surface (this, &m_texture),
+    m_resource(this),
+    m_d3d10   (this) {
     
   }
   
@@ -568,6 +792,9 @@ namespace dxvk {
   
   
   HRESULT STDMETHODCALLTYPE D3D11Texture1D::QueryInterface(REFIID riid, void** ppvObject) {
+    if (ppvObject == nullptr)
+      return E_POINTER;
+
     *ppvObject = nullptr;
     
     if (riid == __uuidof(IUnknown)
@@ -585,6 +812,22 @@ namespace dxvk {
       return S_OK;
     }
     
+    if (m_surface.isSurfaceCompatible()
+     && (riid == __uuidof(IDXGISurface)
+      || riid == __uuidof(IDXGISurface1)
+      || riid == __uuidof(IDXGISurface2))) {
+      *ppvObject = ref(&m_surface);
+      return S_OK;
+    }
+
+    if (riid == __uuidof(IDXGIObject)
+     || riid == __uuidof(IDXGIDeviceSubObject)
+     || riid == __uuidof(IDXGIResource)
+     || riid == __uuidof(IDXGIResource1)) {
+       *ppvObject = ref(&m_resource);
+       return S_OK;
+    }
+    
     if (riid == __uuidof(IDXGIVkInteropSurface)) {
       *ppvObject = ref(&m_interop);
       return S_OK;
@@ -638,7 +881,9 @@ namespace dxvk {
     const D3D11_COMMON_TEXTURE_DESC*  pDesc)
   : m_texture (pDevice, pDesc, D3D11_RESOURCE_DIMENSION_TEXTURE2D),
     m_interop (this, &m_texture),
-    m_d3d10   (this, pDevice->GetD3D10Interface()) {
+    m_surface (this, &m_texture),
+    m_resource(this),
+    m_d3d10   (this) {
     
   }
   
@@ -649,12 +894,16 @@ namespace dxvk {
   
   
   HRESULT STDMETHODCALLTYPE D3D11Texture2D::QueryInterface(REFIID riid, void** ppvObject) {
+    if (ppvObject == nullptr)
+      return E_POINTER;
+
     *ppvObject = nullptr;
     
     if (riid == __uuidof(IUnknown)
      || riid == __uuidof(ID3D11DeviceChild)
      || riid == __uuidof(ID3D11Resource)
-     || riid == __uuidof(ID3D11Texture2D)) {
+     || riid == __uuidof(ID3D11Texture2D)
+     || riid == __uuidof(ID3D11Texture2D1)) {
       *ppvObject = ref(this);
       return S_OK;
     }
@@ -665,6 +914,22 @@ namespace dxvk {
       *ppvObject = ref(&m_d3d10);
       return S_OK;
     }
+
+    if (m_surface.isSurfaceCompatible()
+     && (riid == __uuidof(IDXGISurface)
+      || riid == __uuidof(IDXGISurface1)
+      || riid == __uuidof(IDXGISurface2))) {
+      *ppvObject = ref(&m_surface);
+      return S_OK;
+    }
+    
+    if (riid == __uuidof(IDXGIObject)
+     || riid == __uuidof(IDXGIDeviceSubObject)
+     || riid == __uuidof(IDXGIResource)
+     || riid == __uuidof(IDXGIResource1)) {
+       *ppvObject = ref(&m_resource);
+       return S_OK;
+    }
     
     if (riid == __uuidof(IDXGIVkInteropSurface)) {
       *ppvObject = ref(&m_interop);
@@ -700,7 +965,7 @@ namespace dxvk {
   }
   
   
-  void STDMETHODCALLTYPE D3D11Texture2D::GetDesc(D3D11_TEXTURE2D_DESC *pDesc) {
+  void STDMETHODCALLTYPE D3D11Texture2D::GetDesc(D3D11_TEXTURE2D_DESC* pDesc) {
     pDesc->Width          = m_texture.Desc()->Width;
     pDesc->Height         = m_texture.Desc()->Height;
     pDesc->MipLevels      = m_texture.Desc()->MipLevels;
@@ -714,6 +979,21 @@ namespace dxvk {
   }
   
   
+  void STDMETHODCALLTYPE D3D11Texture2D::GetDesc1(D3D11_TEXTURE2D_DESC1* pDesc) {
+    pDesc->Width          = m_texture.Desc()->Width;
+    pDesc->Height         = m_texture.Desc()->Height;
+    pDesc->MipLevels      = m_texture.Desc()->MipLevels;
+    pDesc->ArraySize      = m_texture.Desc()->ArraySize;
+    pDesc->Format         = m_texture.Desc()->Format;
+    pDesc->SampleDesc     = m_texture.Desc()->SampleDesc;
+    pDesc->Usage          = m_texture.Desc()->Usage;
+    pDesc->BindFlags      = m_texture.Desc()->BindFlags;
+    pDesc->CPUAccessFlags = m_texture.Desc()->CPUAccessFlags;
+    pDesc->MiscFlags      = m_texture.Desc()->MiscFlags;
+    pDesc->TextureLayout  = m_texture.Desc()->TextureLayout;
+  }
+  
+  
   ///////////////////////////////////////////
   //      D 3 D 1 1 T E X T U R E 3 D
   D3D11Texture3D::D3D11Texture3D(
@@ -721,7 +1001,8 @@ namespace dxvk {
     const D3D11_COMMON_TEXTURE_DESC*  pDesc)
   : m_texture (pDevice, pDesc, D3D11_RESOURCE_DIMENSION_TEXTURE3D),
     m_interop (this, &m_texture),
-    m_d3d10   (this, pDevice->GetD3D10Interface()) {
+    m_resource(this),
+    m_d3d10   (this) {
     
   }
   
@@ -732,12 +1013,16 @@ namespace dxvk {
   
   
   HRESULT STDMETHODCALLTYPE D3D11Texture3D::QueryInterface(REFIID riid, void** ppvObject) {
+    if (ppvObject == nullptr)
+      return E_POINTER;
+
     *ppvObject = nullptr;
     
     if (riid == __uuidof(IUnknown)
      || riid == __uuidof(ID3D11DeviceChild)
      || riid == __uuidof(ID3D11Resource)
-     || riid == __uuidof(ID3D11Texture3D)) {
+     || riid == __uuidof(ID3D11Texture3D)
+     || riid == __uuidof(ID3D11Texture3D1)) {
       *ppvObject = ref(this);
       return S_OK;
     }
@@ -749,6 +1034,14 @@ namespace dxvk {
       return S_OK;
     }
     
+    if (riid == __uuidof(IDXGIObject)
+     || riid == __uuidof(IDXGIDeviceSubObject)
+     || riid == __uuidof(IDXGIResource)
+     || riid == __uuidof(IDXGIResource1)) {
+       *ppvObject = ref(&m_resource);
+       return S_OK;
+    }
+    
     if (riid == __uuidof(IDXGIVkInteropSurface)) {
       *ppvObject = ref(&m_interop);
       return S_OK;
@@ -783,7 +1076,7 @@ namespace dxvk {
   }
   
   
-  void STDMETHODCALLTYPE D3D11Texture3D::GetDesc(D3D11_TEXTURE3D_DESC *pDesc) {
+  void STDMETHODCALLTYPE D3D11Texture3D::GetDesc(D3D11_TEXTURE3D_DESC* pDesc) {
     pDesc->Width          = m_texture.Desc()->Width;
     pDesc->Height         = m_texture.Desc()->Height;
     pDesc->Depth          = m_texture.Desc()->Depth;
@@ -796,6 +1089,18 @@ namespace dxvk {
   }
   
   
+  void STDMETHODCALLTYPE D3D11Texture3D::GetDesc1(D3D11_TEXTURE3D_DESC1* pDesc) {
+    pDesc->Width          = m_texture.Desc()->Width;
+    pDesc->Height         = m_texture.Desc()->Height;
+    pDesc->Depth          = m_texture.Desc()->Depth;
+    pDesc->MipLevels      = m_texture.Desc()->MipLevels;
+    pDesc->Format         = m_texture.Desc()->Format;
+    pDesc->Usage          = m_texture.Desc()->Usage;
+    pDesc->BindFlags      = m_texture.Desc()->BindFlags;
+    pDesc->CPUAccessFlags = m_texture.Desc()->CPUAccessFlags;
+    pDesc->MiscFlags      = m_texture.Desc()->MiscFlags;
+  }
+  
   
   D3D11CommonTexture* GetCommonTexture(ID3D11Resource* pResource) {
     D3D11_RESOURCE_DIMENSION dimension = D3D11_RESOURCE_DIMENSION_UNKNOWN;
