@@ -31,9 +31,6 @@ namespace dxvk {
   VkCompareOp DecodeCompareOp(
           D3D11_COMPARISON_FUNC     Mode);
   
-  VkMemoryPropertyFlags GetMemoryFlagsForUsage(
-          D3D11_USAGE               Usage);
-  
   VkShaderStageFlagBits GetShaderStage(
           DxbcProgramType           ProgramType);
   
