@@ -9,14 +9,19 @@ namespace dxvk {
   D3D11ShaderResourceView::D3D11ShaderResourceView(
           D3D11Device*                      pDevice,
           ID3D11Resource*                   pResource,
-    const D3D11_SHADER_RESOURCE_VIEW_DESC*  pDesc)
+    const D3D11_SHADER_RESOURCE_VIEW_DESC1* pDesc)
   : m_device(pDevice), m_resource(pResource), m_desc(*pDesc), m_d3d10(this) {
     ResourceAddRefPrivate(m_resource);
 
-    D3D11_RESOURCE_DIMENSION resourceDim = D3D11_RESOURCE_DIMENSION_UNKNOWN;
-    pResource->GetType(&resourceDim);
+    D3D11_COMMON_RESOURCE_DESC resourceDesc;
+    GetCommonResourceDesc(pResource, &resourceDesc);
 
-    if (resourceDim == D3D11_RESOURCE_DIMENSION_BUFFER) {
+    // Basic view resource info
+    m_info.pResource = pResource;
+    m_info.Dimension = resourceDesc.Dim;
+    m_info.BindFlags = resourceDesc.BindFlags;
+
+    if (resourceDesc.Dim == D3D11_RESOURCE_DIMENSION_BUFFER) {
       auto buffer = static_cast<D3D11Buffer*>(pResource);
 
       // Move buffer description to a common struct to
@@ -57,6 +62,10 @@ namespace dxvk {
         viewInfo.rangeLength = formatInfo->elementSize * bufInfo.NumElements;
       }
 
+      // Populate view info struct
+      m_info.Buffer.Offset = viewInfo.rangeOffset;
+      m_info.Buffer.Length = viewInfo.rangeLength;
+
       // Create underlying buffer view object
       m_bufferView = pDevice->GetDXVKDevice()->createBufferView(
         buffer->GetBuffer(), viewInfo);
@@ -154,6 +163,13 @@ namespace dxvk {
           throw DxvkError("D3D11: Invalid view dimension for image SRV");
       }
       
+      // Populate view info struct
+      m_info.Image.Aspects   = viewInfo.aspect;
+      m_info.Image.MinLevel  = viewInfo.minLevel;
+      m_info.Image.MinLayer  = viewInfo.minLayer;
+      m_info.Image.NumLevels = viewInfo.numLevels;
+      m_info.Image.NumLayers = viewInfo.numLayers;
+
       // Create the underlying image view object
       m_imageView = pDevice->GetDXVKDevice()->createImageView(
         GetCommonTexture(pResource)->GetImage(), viewInfo);
@@ -167,12 +183,16 @@ namespace dxvk {
   
   
   HRESULT STDMETHODCALLTYPE D3D11ShaderResourceView::QueryInterface(REFIID riid, void** ppvObject) {
+    if (ppvObject == nullptr)
+      return E_POINTER;
+
     *ppvObject = nullptr;
     
     if (riid == __uuidof(IUnknown)
      || riid == __uuidof(ID3D11DeviceChild)
      || riid == __uuidof(ID3D11View)
-     || riid == __uuidof(ID3D11ShaderResourceView)) {
+     || riid == __uuidof(ID3D11ShaderResourceView)
+     || riid == __uuidof(ID3D11ShaderResourceView1)) {
       *ppvObject = ref(this);
       return S_OK;
     }
@@ -202,13 +222,72 @@ namespace dxvk {
   
   
   void STDMETHODCALLTYPE D3D11ShaderResourceView::GetDesc(D3D11_SHADER_RESOURCE_VIEW_DESC* pDesc) {
+    pDesc->Format            = m_desc.Format;
+    pDesc->ViewDimension     = m_desc.ViewDimension;
+
+    switch (m_desc.ViewDimension) {
+      case D3D11_SRV_DIMENSION_UNKNOWN:
+        break;
+
+      case D3D11_SRV_DIMENSION_BUFFER:
+        pDesc->Buffer = m_desc.Buffer;
+        break;
+
+      case D3D11_SRV_DIMENSION_TEXTURE1D:
+        pDesc->Texture1D = m_desc.Texture1D;
+        break;
+
+      case D3D11_SRV_DIMENSION_TEXTURE1DARRAY:
+        pDesc->Texture1DArray = m_desc.Texture1DArray;
+        break;
+
+      case D3D11_SRV_DIMENSION_TEXTURE2D:
+        pDesc->Texture2D.MostDetailedMip = m_desc.Texture2D.MostDetailedMip;
+        pDesc->Texture2D.MipLevels       = m_desc.Texture2D.MipLevels;
+        break;
+
+      case D3D11_SRV_DIMENSION_TEXTURE2DARRAY:
+        pDesc->Texture2DArray.MostDetailedMip = m_desc.Texture2DArray.MostDetailedMip;
+        pDesc->Texture2DArray.MipLevels       = m_desc.Texture2DArray.MipLevels;
+        pDesc->Texture2DArray.FirstArraySlice = m_desc.Texture2DArray.FirstArraySlice;
+        pDesc->Texture2DArray.ArraySize       = m_desc.Texture2DArray.ArraySize;
+        break;
+
+      case D3D11_SRV_DIMENSION_TEXTURE2DMS:
+        pDesc->Texture2DMS = m_desc.Texture2DMS;
+        break;
+
+      case D3D11_SRV_DIMENSION_TEXTURE2DMSARRAY:
+        pDesc->Texture2DMSArray = m_desc.Texture2DMSArray;
+        break;
+
+      case D3D11_SRV_DIMENSION_TEXTURE3D:
+        pDesc->Texture3D = m_desc.Texture3D;
+        break;
+
+      case D3D11_SRV_DIMENSION_TEXTURECUBE:
+        pDesc->TextureCube = m_desc.TextureCube;
+        break;
+
+      case D3D11_SRV_DIMENSION_TEXTURECUBEARRAY:
+        pDesc->TextureCubeArray = m_desc.TextureCubeArray;
+        break;
+
+      case D3D11_SRV_DIMENSION_BUFFEREX:
+        pDesc->BufferEx = m_desc.BufferEx;
+        break;
+    }
+  }
+  
+  
+  void STDMETHODCALLTYPE D3D11ShaderResourceView::GetDesc1(D3D11_SHADER_RESOURCE_VIEW_DESC1* pDesc) {
     *pDesc = m_desc;
   }
   
   
   HRESULT D3D11ShaderResourceView::GetDescFromResource(
           ID3D11Resource*                   pResource,
-          D3D11_SHADER_RESOURCE_VIEW_DESC*  pDesc) {
+          D3D11_SHADER_RESOURCE_VIEW_DESC1* pDesc) {
     D3D11_RESOURCE_DIMENSION resourceDim = D3D11_RESOURCE_DIMENSION_UNKNOWN;
     pResource->GetType(&resourceDim);
     
@@ -256,12 +335,14 @@ namespace dxvk {
             pDesc->ViewDimension = D3D11_SRV_DIMENSION_TEXTURE2D;
             pDesc->Texture2D.MostDetailedMip = 0;
             pDesc->Texture2D.MipLevels       = resourceDesc.MipLevels;
+            pDesc->Texture2D.PlaneSlice      = 0;
           } else {
             pDesc->ViewDimension = D3D11_SRV_DIMENSION_TEXTURE2DARRAY;
             pDesc->Texture2DArray.MostDetailedMip = 0;
             pDesc->Texture2DArray.MipLevels       = resourceDesc.MipLevels;
             pDesc->Texture2DArray.FirstArraySlice = 0;
             pDesc->Texture2DArray.ArraySize       = resourceDesc.ArraySize;
+            pDesc->Texture2DArray.PlaneSlice      = 0;
           }
         } else {
           if (resourceDesc.ArraySize == 1) {
@@ -293,9 +374,74 @@ namespace dxvk {
   }
   
   
+  D3D11_SHADER_RESOURCE_VIEW_DESC1 D3D11ShaderResourceView::PromoteDesc(
+    const D3D11_SHADER_RESOURCE_VIEW_DESC*  pDesc) {
+    D3D11_SHADER_RESOURCE_VIEW_DESC1 dstDesc;
+    dstDesc.Format            = pDesc->Format;
+    dstDesc.ViewDimension     = pDesc->ViewDimension;
+
+    switch (pDesc->ViewDimension) {
+      case D3D11_SRV_DIMENSION_UNKNOWN:
+        break;
+
+      case D3D11_SRV_DIMENSION_BUFFER:
+        dstDesc.Buffer = pDesc->Buffer;
+        break;
+
+      case D3D11_SRV_DIMENSION_TEXTURE1D:
+        dstDesc.Texture1D = pDesc->Texture1D;
+        break;
+
+      case D3D11_SRV_DIMENSION_TEXTURE1DARRAY:
+        dstDesc.Texture1DArray = pDesc->Texture1DArray;
+        break;
+
+      case D3D11_SRV_DIMENSION_TEXTURE2D:
+        dstDesc.Texture2D.MostDetailedMip = pDesc->Texture2D.MostDetailedMip;
+        dstDesc.Texture2D.MipLevels       = pDesc->Texture2D.MipLevels;
+        dstDesc.Texture2D.PlaneSlice      = 0;
+        break;
+
+      case D3D11_SRV_DIMENSION_TEXTURE2DARRAY:
+        dstDesc.Texture2DArray.MostDetailedMip = pDesc->Texture2DArray.MostDetailedMip;
+        dstDesc.Texture2DArray.MipLevels       = pDesc->Texture2DArray.MipLevels;
+        dstDesc.Texture2DArray.FirstArraySlice = pDesc->Texture2DArray.FirstArraySlice;
+        dstDesc.Texture2DArray.ArraySize       = pDesc->Texture2DArray.ArraySize;
+        dstDesc.Texture2DArray.PlaneSlice      = 0;
+        break;
+
+      case D3D11_SRV_DIMENSION_TEXTURE2DMS:
+        dstDesc.Texture2DMS = pDesc->Texture2DMS;
+        break;
+
+      case D3D11_SRV_DIMENSION_TEXTURE2DMSARRAY:
+        dstDesc.Texture2DMSArray = pDesc->Texture2DMSArray;
+        break;
+
+      case D3D11_SRV_DIMENSION_TEXTURE3D:
+        dstDesc.Texture3D = pDesc->Texture3D;
+        break;
+
+      case D3D11_SRV_DIMENSION_TEXTURECUBE:
+        dstDesc.TextureCube = pDesc->TextureCube;
+        break;
+
+      case D3D11_SRV_DIMENSION_TEXTURECUBEARRAY:
+        dstDesc.TextureCubeArray = pDesc->TextureCubeArray;
+        break;
+
+      case D3D11_SRV_DIMENSION_BUFFEREX:
+        dstDesc.BufferEx = pDesc->BufferEx;
+        break;
+    }
+
+    return dstDesc;
+  }
+
+
   HRESULT D3D11ShaderResourceView::NormalizeDesc(
           ID3D11Resource*                   pResource,
-          D3D11_SHADER_RESOURCE_VIEW_DESC*  pDesc) {
+          D3D11_SHADER_RESOURCE_VIEW_DESC1* pDesc) {
     D3D11_RESOURCE_DIMENSION resourceDim = D3D11_RESOURCE_DIMENSION_UNKNOWN;
     pResource->GetType(&resourceDim);
     
@@ -368,6 +514,16 @@ namespace dxvk {
       pDesc->Format = format;
     
     switch (pDesc->ViewDimension) {
+      case D3D11_SRV_DIMENSION_BUFFER:
+        if (pDesc->Buffer.NumElements == 0)
+          return E_INVALIDARG;
+        break;
+
+      case D3D11_SRV_DIMENSION_BUFFEREX:
+        if (pDesc->BufferEx.NumElements == 0)
+          return E_INVALIDARG;
+        break;
+
       case D3D11_SRV_DIMENSION_TEXTURE1D:
         if (pDesc->Texture1D.MipLevels > mipLevels - pDesc->Texture1D.MostDetailedMip)
           pDesc->Texture1D.MipLevels = mipLevels - pDesc->Texture1D.MostDetailedMip;
@@ -383,6 +539,9 @@ namespace dxvk {
       case D3D11_SRV_DIMENSION_TEXTURE2D:
         if (pDesc->Texture2D.MipLevels > mipLevels - pDesc->Texture2D.MostDetailedMip)
           pDesc->Texture2D.MipLevels = mipLevels - pDesc->Texture2D.MostDetailedMip;
+        
+        if (pDesc->Texture2D.PlaneSlice != 0)
+          return E_INVALIDARG;
         break;
       
       case D3D11_SRV_DIMENSION_TEXTURE2DARRAY:
@@ -390,6 +549,9 @@ namespace dxvk {
           pDesc->Texture2DArray.MipLevels = mipLevels - pDesc->Texture2DArray.MostDetailedMip;
         if (pDesc->Texture2DArray.ArraySize > numLayers - pDesc->Texture2DArray.FirstArraySlice)
           pDesc->Texture2DArray.ArraySize = numLayers - pDesc->Texture2DArray.FirstArraySlice;
+        
+        if (pDesc->Texture2DArray.PlaneSlice != 0)
+          return E_INVALIDARG;
         break;
       
       case D3D11_SRV_DIMENSION_TEXTURE2DMSARRAY:
