@@ -160,6 +160,7 @@ namespace dxvk {
     uint32_t builtinLayer         = 0;
     uint32_t builtinViewportId    = 0;
     uint32_t builtinInvocationId  = 0;
+    uint32_t invocationCount      = 0;
   };
   
   
@@ -171,6 +172,7 @@ namespace dxvk {
     
     uint32_t builtinFragCoord     = 0;
     uint32_t builtinDepth         = 0;
+    uint32_t builtinStencilRef    = 0;
     uint32_t builtinIsFrontFace   = 0;
     uint32_t builtinSampleId      = 0;
     uint32_t builtinSampleMaskIn  = 0;
@@ -178,8 +180,10 @@ namespace dxvk {
     uint32_t builtinLayer         = 0;
     uint32_t builtinViewportId    = 0;
     
-    uint32_t invocationMask       = 0;
+    uint32_t builtinLaneId        = 0;
     uint32_t killState            = 0;
+
+    uint32_t specRsSampleCount    = 0;
   };
   
   
@@ -345,6 +349,7 @@ namespace dxvk {
     uint32_t varId;
     uint32_t specId;
     uint32_t stride;
+    uint32_t align;
   };
   
 
@@ -468,17 +473,13 @@ namespace dxvk {
     // Function state tracking. Required in order
     // to properly end functions in some cases.
     bool m_insideFunction = false;
-    
-    ///////////////////////////////////////////////
-    // Specialization constants. These are defined
-    // as needed by the getSpecConstant method.
-    std::array<DxbcRegisterValue,
-      uint32_t(DxvkSpecConstantId::SpecConstantIdMax) -
-      uint32_t(DxvkSpecConstantId::SpecConstantIdMin) + 1> m_specConstants;
-    
+
     ///////////////////////////////////////////////////////////
     // Array of input values. Since v# registers are indexable
     // in DXBC, we need to copy them into an array first.
+    uint32_t m_vArrayLength   = 0;
+    uint32_t m_vArrayLengthId = 0;
+
     uint32_t m_vArray = 0;
     
     ////////////////////////////////////////////////////
@@ -509,6 +510,10 @@ namespace dxvk {
     uint32_t m_uavCtrStructType  = 0;
     uint32_t m_uavCtrPointerType = 0;
     
+    ////////////////////////////////
+    // Function IDs for subroutines
+    std::unordered_map<uint32_t, uint32_t> m_subroutines;
+    
     ///////////////////////////////////////////////////
     // Entry point description - we'll need to declare
     // the function ID and all input/output variables.
@@ -570,7 +575,8 @@ namespace dxvk {
     void emitDclConstantBufferVar(
             uint32_t                regIdx,
             uint32_t                numConstants,
-      const char*                   name);
+      const char*                   name,
+            bool                    asSsbo);
     
     void emitDclSampler(
       const DxbcShaderInstruction&  ins);
@@ -662,6 +668,9 @@ namespace dxvk {
     void emitVectorImul(
       const DxbcShaderInstruction&  ins);
     
+    void emitVectorMsad(
+      const DxbcShaderInstruction&  ins);
+    
     void emitVectorShift(
       const DxbcShaderInstruction&  ins);
     
@@ -784,6 +793,15 @@ namespace dxvk {
     void emitControlFlowDiscard(
       const DxbcShaderInstruction&  ins);
     
+    void emitControlFlowLabel(
+      const DxbcShaderInstruction&  ins);
+
+    void emitControlFlowCall(
+      const DxbcShaderInstruction&  ins);
+    
+    void emitControlFlowCallc(
+      const DxbcShaderInstruction&  ins);
+    
     void emitControlFlow(
       const DxbcShaderInstruction&  ins);
     
@@ -972,6 +990,10 @@ namespace dxvk {
     DxbcRegisterValue emitRegisterLoadRaw(
       const DxbcRegister&           reg);
     
+    DxbcRegisterValue emitConstantBufferLoad(
+      const DxbcRegister&           reg,
+            DxbcRegMask             writeMask);
+    
     DxbcRegisterValue emitRegisterLoad(
       const DxbcRegister&           reg,
             DxbcRegMask             writeMask);
@@ -982,12 +1004,12 @@ namespace dxvk {
     
     ////////////////////////////////////////
     // Spec constant declaration and access
-    DxbcRegisterValue getSpecConstant(
-            DxvkSpecConstantId      specId);
-    
-    DxbcSpecConstant getSpecConstantProperties(
-            DxvkSpecConstantId      specId);
-    
+    uint32_t emitNewSpecConstant(
+            DxvkSpecConstantId      specId,
+            DxbcScalarType          type,
+            uint32_t                value,
+      const char*                   name);
+
     ////////////////////////////
     // Input/output preparation
     void emitInputSetup();
@@ -1214,6 +1236,9 @@ namespace dxvk {
       const DxbcRegisterInfo& type);
     
     uint32_t getPerVertexBlockId();
+
+    uint32_t getFunctionId(
+            uint32_t          functionNr);
     
     DxbcCompilerHsForkJoinPhase* getCurrentHsForkJoinPhase();
     
