@@ -1,7 +1,9 @@
 #pragma once
 
 //for some reason we need to specify __declspec(dllexport) for MinGW
-#if defined(_MSC_VER) || defined(__WINE__)
+#if defined(__WINE__)
+  #define DLLEXPORT __attribute__((visibility("default")))
+#elif defined(_MSC_VER)
   #define DLLEXPORT
 #else
   #define DLLEXPORT __declspec(dllexport)
@@ -25,7 +27,7 @@
 #include "../util/util_math.h"
 #include "../util/util_string.h"
 
-#include <dxgi1_4.h>
+#include <dxgi1_5.h>
 
 // For some reason, these are not exposed
 #ifndef DXGI_RESOURCE_PRIORITY_NORMAL
@@ -35,3 +37,11 @@
   #define DXGI_RESOURCE_PRIORITY_HIGH (0xa0000000)
   #define DXGI_RESOURCE_PRIORITY_MAXIMUM (0xc8000000)
 #endif
+
+#ifndef DXGI_CPU_ACCESS_NONE
+  #define DXGI_CPU_ACCESS_NONE       (0)
+  #define DXGI_CPU_ACCESS_DYNAMIC    (1)
+  #define DXGI_CPU_ACCESS_READ_WRITE (2)
+  #define DXGI_CPU_ACCESS_SCRATCH    (3)
+  #define DXGI_CPU_ACCESS_FIELD      (0xf)
+#endif
\ No newline at end of file
