@@ -29,9 +29,6 @@ namespace dxvk {
     VkDeviceSize maxDeviceMemory;
     VkDeviceSize maxSharedMemory;
 
-    /// Enables D3D10 support
-    bool d3d10Enable;
-
     /// Enables nvapi workaround
     bool nvapiHack;
   };
