@@ -1,4 +1,5 @@
 #include <algorithm>
+#include <numeric>
 
 #include <cstdlib>
 #include <cstring>
@@ -7,6 +8,7 @@
 #include <string>
 
 #include "dxgi_adapter.h"
+#include "dxgi_factory.h"
 #include "dxgi_output.h"
 #include "dxgi_swapchain.h"
 
@@ -15,25 +17,25 @@
 namespace dxvk {
   
   DxgiOutput::DxgiOutput(
+    const Com<DxgiFactory>& factory,
     const Com<DxgiAdapter>& adapter,
               HMONITOR      monitor)
-  : m_adapter(adapter),
+  : m_monitorInfo(factory->GetMonitorInfo()),
+    m_adapter(adapter),
     m_monitor(monitor) {
-    // Init output data if necessary
-    if (FAILED(m_adapter->GetOutputData(m_monitor, nullptr))) {
-      DXGI_VK_OUTPUT_DATA outputData;
-      outputData.FrameStats = DXGI_FRAME_STATISTICS();
-      outputData.GammaCurve.Scale  = { 1.0f, 1.0f, 1.0f };
-      outputData.GammaCurve.Offset = { 0.0f, 0.0f, 0.0f };
-      
-      for (uint32_t i = 0; i < DXGI_VK_GAMMA_CP_COUNT; i++) {
-        const float value = GammaControlPointLocation(i);
-        outputData.GammaCurve.GammaCurve[i] = { value, value, value };
-      }
-      
-      outputData.GammaDirty = FALSE;
-      m_adapter->SetOutputData(m_monitor, &outputData);
+    // Init monitor info if necessary
+    DXGI_VK_MONITOR_DATA monitorData;
+    monitorData.pSwapChain = nullptr;
+    monitorData.FrameStats = DXGI_FRAME_STATISTICS();
+    monitorData.GammaCurve.Scale  = { 1.0f, 1.0f, 1.0f };
+    monitorData.GammaCurve.Offset = { 0.0f, 0.0f, 0.0f };
+    
+    for (uint32_t i = 0; i < DXGI_VK_GAMMA_CP_COUNT; i++) {
+      const float value = GammaControlPointLocation(i);
+      monitorData.GammaCurve.GammaCurve[i] = { value, value, value };
     }
+    
+    m_monitorInfo->InitMonitorData(monitor, &monitorData);    
   }
   
   
@@ -43,6 +45,9 @@ namespace dxvk {
   
   
   HRESULT STDMETHODCALLTYPE DxgiOutput::QueryInterface(REFIID riid, void** ppvObject) {
+    if (ppvObject == nullptr)
+      return E_POINTER;
+
     *ppvObject = nullptr;
     
     if (riid == __uuidof(IUnknown)
@@ -51,7 +56,8 @@ namespace dxvk {
      || riid == __uuidof(IDXGIOutput1)
      || riid == __uuidof(IDXGIOutput2)
      || riid == __uuidof(IDXGIOutput3)
-     || riid == __uuidof(IDXGIOutput4)) {
+     || riid == __uuidof(IDXGIOutput4)
+     || riid == __uuidof(IDXGIOutput5)) {
       *ppvObject = ref(this);
       return S_OK;
     }
@@ -110,23 +116,46 @@ namespace dxvk {
 
     if (pModeToMatch->Format == DXGI_FORMAT_UNKNOWN && !pConcernedDevice)
       return DXGI_ERROR_INVALID_CALL;
-    
-    // If no format was specified, fall back to a standard
-    // SRGB format, which is supported on all devices.
+
+    // Both or neither must be zero
+    if ((pModeToMatch->Width == 0) ^ (pModeToMatch->Height == 0))
+      return DXGI_ERROR_INVALID_CALL;
+
+    DXGI_MODE_DESC activeMode = { };
+    GetMonitorDisplayMode(m_monitor, ENUM_CURRENT_SETTINGS, &activeMode);
+
+    DXGI_MODE_DESC1 defaultMode;
+    defaultMode.Width            = 0;
+    defaultMode.Height           = 0;
+    defaultMode.RefreshRate      = { 0, 0 };
+    defaultMode.Format           = DXGI_FORMAT_UNKNOWN;
+    defaultMode.ScanlineOrdering = DXGI_MODE_SCANLINE_ORDER_UNSPECIFIED;
+    defaultMode.Scaling          = DXGI_MODE_SCALING_UNSPECIFIED;
+    defaultMode.Stereo           = pModeToMatch->Stereo;
+
+    if (pModeToMatch->ScanlineOrdering == DXGI_MODE_SCANLINE_ORDER_UNSPECIFIED)
+      defaultMode.ScanlineOrdering = activeMode.ScanlineOrdering;
+
+    if (pModeToMatch->Scaling == DXGI_MODE_SCALING_UNSPECIFIED)
+      defaultMode.Scaling = activeMode.Scaling;
+
     DXGI_FORMAT targetFormat = pModeToMatch->Format;
-    
-    if (targetFormat == DXGI_FORMAT_UNKNOWN)
-      targetFormat = DXGI_FORMAT_R8G8B8A8_UNORM_SRGB;
-    
-    UINT targetRefreshRate = 0;
-    
-    if (pModeToMatch->RefreshRate.Denominator != 0) {
-      targetRefreshRate = pModeToMatch->RefreshRate.Numerator
-                        / pModeToMatch->RefreshRate.Denominator;
+
+    if (pModeToMatch->Format == DXGI_FORMAT_UNKNOWN) {
+      defaultMode.Format = activeMode.Format;
+      targetFormat       = activeMode.Format;
     }
-    
-    // List all supported modes and filter
-    // out those we don't actually need
+
+    if (!pModeToMatch->Width) {
+      defaultMode.Width  = activeMode.Width;
+      defaultMode.Height = activeMode.Height;
+    }
+
+    if (!pModeToMatch->RefreshRate.Numerator || !pModeToMatch->RefreshRate.Denominator) {
+      defaultMode.RefreshRate.Numerator   = activeMode.RefreshRate.Numerator;
+      defaultMode.RefreshRate.Denominator = activeMode.RefreshRate.Denominator;
+    }
+
     UINT modeCount = 0;
     GetDisplayModeList1(targetFormat, DXGI_ENUM_MODES_SCALING, &modeCount, nullptr);
     
@@ -137,57 +166,22 @@ namespace dxvk {
 
     std::vector<DXGI_MODE_DESC1> modes(modeCount);
     GetDisplayModeList1(targetFormat, DXGI_ENUM_MODES_SCALING, &modeCount, modes.data());
-    
-    for (auto it = modes.begin(); it != modes.end(); ) {
-      bool skipMode = false;
-      
-      // Remove modes with a different refresh rate
-      if (targetRefreshRate != 0) {
-        UINT modeRefreshRate = it->RefreshRate.Numerator
-                             / it->RefreshRate.Denominator;
-        skipMode |= modeRefreshRate != targetRefreshRate;
-      }
-      
-      // Remove modes with incorrect scaling
-      if (pModeToMatch->Scaling != DXGI_MODE_SCALING_UNSPECIFIED)
-        skipMode |= it->Scaling != pModeToMatch->Scaling;
-      
-      // Remove modes with incorrect stereo mode
-      skipMode |= it->Stereo != pModeToMatch->Stereo;
-      
-      it = skipMode ? modes.erase(it) : ++it;
-    }
-    
-    // No matching modes found
-    if (modes.size() == 0)
+
+    FilterModesByDesc(modes, *pModeToMatch);
+    FilterModesByDesc(modes, defaultMode);
+
+    if (modes.empty())
       return DXGI_ERROR_NOT_FOUND;
 
-    // If no valid resolution is specified, find the
-    // closest match for the current display resolution
-    UINT targetWidth  = pModeToMatch->Width;
-    UINT targetHeight = pModeToMatch->Height;
-
-    if (targetWidth == 0 || targetHeight == 0) {
-      DXGI_MODE_DESC activeMode = { };
-      GetDisplayMode(&activeMode, ENUM_CURRENT_SETTINGS);
-
-      targetWidth  = activeMode.Width;
-      targetHeight = activeMode.Height;
-    }
-
-    // Select mode with minimal height+width difference
-    UINT minDifference = std::numeric_limits<unsigned int>::max();
-    
-    for (auto mode : modes) {
-      UINT currDifference = std::abs(int(targetWidth  - mode.Width))
-                          + std::abs(int(targetHeight - mode.Height));
-
-      if (currDifference <= minDifference) {
-        minDifference = currDifference;
-        *pClosestMatch = mode;
-      }
-    }
+    *pClosestMatch = modes[0];
 
+    Logger::debug(str::format(
+      "DXGI: For mode ",
+        pModeToMatch->Width, "x", pModeToMatch->Height, "@",
+        pModeToMatch->RefreshRate.Denominator ? (pModeToMatch->RefreshRate.Numerator / pModeToMatch->RefreshRate.Denominator) : 0,
+      " found closest mode ",
+        pClosestMatch->Width, "x", pClosestMatch->Height, "@",
+        pClosestMatch->RefreshRate.Denominator ? (pClosestMatch->RefreshRate.Numerator / pClosestMatch->RefreshRate.Denominator) : 0));
     return S_OK;
   }
 
@@ -225,7 +219,7 @@ namespace dxvk {
     std::vector<DXGI_MODE_DESC1> modes;
 
     if (pDesc)
-      modes.resize(*pNumModes);
+      modes.resize(std::max(1u, *pNumModes));
     
     HRESULT hr = GetDisplayModeList1(
       EnumFormat, Flags, pNumModes,
@@ -252,6 +246,12 @@ namespace dxvk {
     if (pNumModes == nullptr)
       return DXGI_ERROR_INVALID_CALL;
     
+    // Special case, just return zero modes
+    if (EnumFormat == DXGI_FORMAT_UNKNOWN) {
+      *pNumModes = 0;
+      return S_OK;
+    }
+
     // Query monitor info to get the device name
     ::MONITORINFOEXW monInfo;
     monInfo.cbSize = sizeof(monInfo);
@@ -263,7 +263,8 @@ namespace dxvk {
     
     // Walk over all modes that the display supports and
     // return those that match the requested format etc.
-    DEVMODEW devMode;
+    DEVMODEW devMode = { };
+    devMode.dmSize = sizeof(DEVMODEW);
     
     uint32_t srcModeId = 0;
     uint32_t dstModeId = 0;
@@ -276,7 +277,7 @@ namespace dxvk {
         continue;
       
       // Skip modes with incompatible formats
-      if (devMode.dmBitsPerPel != GetFormatBpp(EnumFormat))
+      if (devMode.dmBitsPerPel != GetMonitorFormatBpp(EnumFormat))
         continue;
       
       if (pDesc != nullptr) {
@@ -330,27 +331,27 @@ namespace dxvk {
   
   
   HRESULT STDMETHODCALLTYPE DxgiOutput::GetFrameStatistics(DXGI_FRAME_STATISTICS* pStats) {
-    DXGI_VK_OUTPUT_DATA outputData;
+    DXGI_VK_MONITOR_DATA* monitorInfo = nullptr;
+    HRESULT hr = m_monitorInfo->AcquireMonitorData(m_monitor, &monitorInfo);
+
+    if (FAILED(hr))
+      return hr;
     
-    if (FAILED(m_adapter->GetOutputData(m_monitor, &outputData))) {
-      Logger::err("DXGI: Failed to query output data");
-      return E_FAIL;
-    }
-    
-    *pStats = outputData.FrameStats;
+    *pStats = monitorInfo->FrameStats;
+    m_monitorInfo->ReleaseMonitorData();
     return S_OK;
   }
   
   
   HRESULT STDMETHODCALLTYPE DxgiOutput::GetGammaControl(DXGI_GAMMA_CONTROL* pArray) {
-    DXGI_VK_OUTPUT_DATA outputData;
+    DXGI_VK_MONITOR_DATA* monitorInfo = nullptr;
+    HRESULT hr = m_monitorInfo->AcquireMonitorData(m_monitor, &monitorInfo);
+
+    if (FAILED(hr))
+      return hr;
     
-    if (FAILED(m_adapter->GetOutputData(m_monitor, &outputData))) {
-      Logger::err("DXGI: Failed to query output data");
-      return E_FAIL;
-    }
-    
-    *pArray = outputData.GammaCurve;
+    *pArray = monitorInfo->GammaCurve;
+    m_monitorInfo->ReleaseMonitorData();
     return S_OK;
   }
   
@@ -385,20 +386,21 @@ namespace dxvk {
   
   
   HRESULT STDMETHODCALLTYPE DxgiOutput::SetGammaControl(const DXGI_GAMMA_CONTROL* pArray) {
-    DXGI_VK_OUTPUT_DATA outputData;
+    DXGI_VK_MONITOR_DATA* monitorInfo = nullptr;
+    HRESULT hr = m_monitorInfo->AcquireMonitorData(m_monitor, &monitorInfo);
+
+    if (FAILED(hr))
+      return hr;
     
-    if (FAILED(m_adapter->GetOutputData(m_monitor, &outputData))) {
-      Logger::err("DXGI: Failed to query output data");
-      return E_FAIL;
+    monitorInfo->GammaCurve = *pArray;
+
+    if (monitorInfo->pSwapChain) {
+      hr = monitorInfo->pSwapChain->SetGammaControl(
+        DXGI_VK_GAMMA_CP_COUNT, pArray->GammaCurve);
     }
-    
-    outputData.GammaCurve = *pArray;
-    outputData.GammaDirty = TRUE;
-    
-    if (FAILED(m_adapter->SetOutputData(m_monitor, &outputData))) {
-      Logger::err("DXGI: Failed to update output data");
-      return E_FAIL;
-    } return S_OK;
+
+    m_monitorInfo->ReleaseMonitorData();
+    return hr;
   }
   
   
@@ -422,12 +424,28 @@ namespace dxvk {
   HRESULT STDMETHODCALLTYPE DxgiOutput::DuplicateOutput(
           IUnknown*                 pDevice,
           IDXGIOutputDuplication**  ppOutputDuplication) {
+    return DuplicateOutput1(pDevice, 0, 0, nullptr, ppOutputDuplication);
+  }
+
+
+  HRESULT STDMETHODCALLTYPE DxgiOutput::DuplicateOutput1(
+          IUnknown*                 pDevice,
+          UINT                      Flags,
+          UINT                      SupportedFormatsCount,
+    const DXGI_FORMAT*              pSupportedFormats,
+          IDXGIOutputDuplication**  ppOutputDuplication) {
+    InitReturnPtr(ppOutputDuplication);
+
+    if (!pDevice)
+      return E_INVALIDARG;
+    
     static bool s_errorShown = false;
 
     if (!std::exchange(s_errorShown, true))
-      Logger::warn("DxgiOutput::DuplicateOutput: Stub");
+      Logger::err("DxgiOutput::DuplicateOutput1: Not implemented");
     
-    return E_NOTIMPL;
+    // At least return a valid error code
+    return DXGI_ERROR_UNSUPPORTED;
   }
 
 
@@ -453,69 +471,77 @@ namespace dxvk {
     Logger::warn("DxgiOutput: CheckOverlayColorSpaceSupport: Stub");
     return DXGI_ERROR_UNSUPPORTED;
   }
+  
 
+  void DxgiOutput::FilterModesByDesc(
+          std::vector<DXGI_MODE_DESC1>& Modes,
+    const DXGI_MODE_DESC1&              TargetMode) {
+    uint32_t minDiffResolution  = 0;
+    uint32_t minDiffRefreshRate = 0;
 
-  HRESULT DxgiOutput::GetDisplayMode(DXGI_MODE_DESC* pMode, DWORD ModeNum) {
-    ::MONITORINFOEXW monInfo;
-    monInfo.cbSize = sizeof(monInfo);
-
-    if (!::GetMonitorInfoW(m_monitor, reinterpret_cast<MONITORINFO*>(&monInfo))) {
-      Logger::err("DXGI: Failed to query monitor info");
-      return E_FAIL;
+    if (TargetMode.Width) {
+      minDiffResolution = std::accumulate(
+        Modes.begin(), Modes.end(), std::numeric_limits<uint32_t>::max(),
+        [&TargetMode] (uint32_t current, const DXGI_MODE_DESC1& mode) {
+          uint32_t diff = std::abs(int32_t(TargetMode.Width  - mode.Width))
+                        + std::abs(int32_t(TargetMode.Height - mode.Height));
+          return std::min(current, diff);
+        });
     }
-    
-    DEVMODEW devMode = { };
-    devMode.dmSize = sizeof(devMode);
-    
-    if (!::EnumDisplaySettingsW(monInfo.szDevice, ModeNum, &devMode))
-      return DXGI_ERROR_NOT_FOUND;
-    
-    pMode->Width            = devMode.dmPelsWidth;
-    pMode->Height           = devMode.dmPelsHeight;
-    pMode->RefreshRate      = { devMode.dmDisplayFrequency, 1 };
-    pMode->Format           = DXGI_FORMAT_R8G8B8A8_UNORM_SRGB; // FIXME
-    pMode->ScanlineOrdering = DXGI_MODE_SCANLINE_ORDER_PROGRESSIVE;
-    pMode->Scaling          = DXGI_MODE_SCALING_UNSPECIFIED;
-    return S_OK;
-  }
-  
-    
-  HRESULT DxgiOutput::SetDisplayMode(const DXGI_MODE_DESC* pMode) {
-    ::MONITORINFOEXW monInfo;
-    monInfo.cbSize = sizeof(monInfo);
 
-    if (!::GetMonitorInfoW(m_monitor, reinterpret_cast<MONITORINFO*>(&monInfo))) {
-      Logger::err("DXGI: Failed to query monitor info");
-      return E_FAIL;
+    if (TargetMode.RefreshRate.Numerator && TargetMode.RefreshRate.Denominator) {
+      minDiffRefreshRate = std::accumulate(
+        Modes.begin(), Modes.end(), std::numeric_limits<uint64_t>::max(),
+        [&TargetMode] (uint64_t current, const DXGI_MODE_DESC1& mode) {
+          uint64_t rate = uint64_t(mode.RefreshRate.Numerator)
+                        * uint64_t(TargetMode.RefreshRate.Denominator)
+                        / uint64_t(mode.RefreshRate.Denominator);
+          uint64_t diff = std::abs(int64_t(rate - uint64_t(TargetMode.RefreshRate.Numerator)));
+          return std::min(current, diff);
+        });
     }
-    
-    DEVMODEW devMode = { };
-    devMode.dmSize       = sizeof(devMode);
-    devMode.dmFields     = DM_PELSWIDTH | DM_PELSHEIGHT | DM_BITSPERPEL;
-    devMode.dmPelsWidth  = pMode->Width;
-    devMode.dmPelsHeight = pMode->Height;
-    devMode.dmBitsPerPel = GetFormatBpp(pMode->Format);
-    
-    if (pMode->RefreshRate.Numerator != 0)  {
-      devMode.dmFields |= DM_DISPLAYFREQUENCY;
-      devMode.dmDisplayFrequency = pMode->RefreshRate.Numerator
-                                 / pMode->RefreshRate.Denominator;
+
+    bool testScanlineOrder = false;
+    bool testScaling       = false;
+    bool testFormat        = false;
+
+    for (const auto& mode : Modes) {
+      testScanlineOrder |= TargetMode.ScanlineOrdering != DXGI_MODE_SCANLINE_ORDER_UNSPECIFIED
+                        && TargetMode.ScanlineOrdering == mode.ScanlineOrdering;
+      testScaling       |= TargetMode.Scaling != DXGI_MODE_SCALING_UNSPECIFIED
+                        && TargetMode.Scaling == mode.Scaling;
+      testFormat        |= TargetMode.Format != DXGI_FORMAT_UNKNOWN
+                        && TargetMode.Format == mode.Format;
+    }
+
+    for (auto it = Modes.begin(); it != Modes.end(); ) {
+      bool skipMode = it->Stereo != TargetMode.Stereo;
+
+      if (testScanlineOrder)
+        skipMode |= it->ScanlineOrdering != TargetMode.ScanlineOrdering;
+
+      if (testScaling)
+        skipMode |= it->Scaling != TargetMode.Scaling;
+
+      if (testFormat)
+        skipMode |= it->Format != TargetMode.Format;
+
+      if (TargetMode.Width) {
+        uint32_t diff = std::abs(int32_t(TargetMode.Width  - it->Width))
+                      + std::abs(int32_t(TargetMode.Height - it->Height));
+        skipMode |= diff != minDiffResolution;
+      }
+
+      if (TargetMode.RefreshRate.Numerator && TargetMode.RefreshRate.Denominator) {
+        uint64_t rate = uint64_t(it->RefreshRate.Numerator)
+                      * uint64_t(TargetMode.RefreshRate.Denominator)
+                      / uint64_t(it->RefreshRate.Denominator);
+        uint64_t diff = std::abs(int64_t(rate - uint64_t(TargetMode.RefreshRate.Numerator)));
+        skipMode |= diff != minDiffRefreshRate;
+      }
+
+      it = skipMode ? Modes.erase(it) : ++it;
     }
-    
-    Logger::info(str::format("DXGI: Setting display mode: ",
-      devMode.dmPelsWidth, "x", devMode.dmPelsHeight, "@",
-      devMode.dmDisplayFrequency));
-    
-    LONG status = ::ChangeDisplaySettingsExW(
-      monInfo.szDevice, &devMode, nullptr, CDS_FULLSCREEN, nullptr);
-    
-    return status == DISP_CHANGE_SUCCESSFUL ? S_OK : DXGI_ERROR_NOT_CURRENTLY_AVAILABLE;;
   }
-  
-  
-  uint32_t DxgiOutput::GetFormatBpp(DXGI_FORMAT Format) const {
-    DXGI_VK_FORMAT_INFO formatInfo = m_adapter->LookupFormat(Format, DXGI_VK_FORMAT_MODE_ANY);
-    return imageFormatInfo(formatInfo.Format)->elementSize * 8;
-  }
-  
+
 }
