@@ -26,6 +26,9 @@ namespace dxvk {
     if (FAILED(m_presenter->GetAdapter(__uuidof(IDXGIAdapter), reinterpret_cast<void**>(&m_adapter))))
       throw DxvkError("DXGI: Failed to get adapter for present device");
     
+    // Query monitor info form DXVK's DXGI factory, if available
+    m_factory->QueryInterface(__uuidof(IDXGIVkMonitorInfo), reinterpret_cast<void**>(&m_monitorInfo));
+    
     // Apply initial window mode and fullscreen state
     if (!m_descFs.Windowed && FAILED(EnterFullscreenMode(nullptr)))
       throw DxvkError("DXGI: Failed to set initial fullscreen state");
@@ -48,6 +51,9 @@ namespace dxvk {
   
   
   HRESULT STDMETHODCALLTYPE DxgiSwapChain::QueryInterface(REFIID riid, void** ppvObject) {
+    if (ppvObject == nullptr)
+      return E_POINTER;
+
     *ppvObject = nullptr;
     
     if (riid == __uuidof(IUnknown)
@@ -56,7 +62,8 @@ namespace dxvk {
      || riid == __uuidof(IDXGISwapChain)
      || riid == __uuidof(IDXGISwapChain1)
      || riid == __uuidof(IDXGISwapChain2)
-     || riid == __uuidof(IDXGISwapChain3)) {
+     || riid == __uuidof(IDXGISwapChain3)
+     || riid == __uuidof(IDXGISwapChain4)) {
       *ppvObject = ref(this);
       return S_OK;
     }
@@ -93,6 +100,11 @@ namespace dxvk {
     if (!IsWindow(m_window))
       return DXGI_ERROR_INVALID_CALL;
     
+    if (m_target != nullptr) {
+      *ppOutput = m_target.ref();
+      return S_OK;
+    }
+
     RECT windowRect = { 0, 0, 0, 0 };
     ::GetWindowRect(m_window, &windowRect);
     
@@ -106,8 +118,8 @@ namespace dxvk {
   
   
   HRESULT STDMETHODCALLTYPE DxgiSwapChain::GetDesc(DXGI_SWAP_CHAIN_DESC* pDesc) {
-    if (pDesc == nullptr)
-      return DXGI_ERROR_INVALID_CALL;
+    if (!pDesc)
+      return E_INVALIDARG;
     
     pDesc->BufferDesc.Width     = m_desc.Width;
     pDesc->BufferDesc.Height    = m_desc.Height;
@@ -128,7 +140,7 @@ namespace dxvk {
   
   HRESULT STDMETHODCALLTYPE DxgiSwapChain::GetDesc1(DXGI_SWAP_CHAIN_DESC1* pDesc) {
     if (pDesc == nullptr)
-      return DXGI_ERROR_INVALID_CALL;
+      return E_INVALIDARG;
     
     *pDesc = m_desc;
     return S_OK;
@@ -160,7 +172,7 @@ namespace dxvk {
   
   HRESULT STDMETHODCALLTYPE DxgiSwapChain::GetFrameStatistics(DXGI_FRAME_STATISTICS* pStats) {
     if (pStats == nullptr)
-      return DXGI_ERROR_INVALID_CALL;
+      return E_INVALIDARG;
     
     *pStats = m_stats;
     return S_OK;
@@ -170,21 +182,14 @@ namespace dxvk {
   HRESULT STDMETHODCALLTYPE DxgiSwapChain::GetFullscreenState(
           BOOL*         pFullscreen,
           IDXGIOutput** ppTarget) {
-    if (!IsWindow(m_window))
-      return DXGI_ERROR_INVALID_CALL;
-    
     HRESULT hr = S_OK;
     
     if (pFullscreen != nullptr)
       *pFullscreen = !m_descFs.Windowed;
     
-    if (ppTarget != nullptr) {
-      *ppTarget = nullptr;
-      
-      if (!m_descFs.Windowed)
-        hr = GetOutputFromMonitor(m_monitor, ppTarget);
-    }
-    
+    if (ppTarget != nullptr)
+      *ppTarget = m_target.ref();
+
     return hr;
   }
   
@@ -192,7 +197,7 @@ namespace dxvk {
   HRESULT STDMETHODCALLTYPE DxgiSwapChain::GetFullscreenDesc(
           DXGI_SWAP_CHAIN_FULLSCREEN_DESC* pDesc) {
     if (pDesc == nullptr)
-      return DXGI_ERROR_INVALID_CALL;
+      return E_INVALIDARG;
     
     *pDesc = m_descFs;
     return S_OK;
@@ -202,7 +207,7 @@ namespace dxvk {
   HRESULT STDMETHODCALLTYPE DxgiSwapChain::GetHwnd(
           HWND*                     pHwnd) {
     if (pHwnd == nullptr)
-      return DXGI_ERROR_INVALID_CALL;
+      return E_INVALIDARG;
     
     *pHwnd = m_window;
     return S_OK;
@@ -221,7 +226,7 @@ namespace dxvk {
   
   HRESULT STDMETHODCALLTYPE DxgiSwapChain::GetLastPresentCount(UINT* pLastPresentCount) {
     if (pLastPresentCount == nullptr)
-      return DXGI_ERROR_INVALID_CALL;
+      return E_INVALIDARG;
     
     *pLastPresentCount = m_stats.PresentCount;
     return S_OK;
@@ -247,17 +252,11 @@ namespace dxvk {
     if (!IsWindow(m_window))
       return DXGI_ERROR_INVALID_CALL;
     
-    if (PresentFlags & DXGI_PRESENT_TEST)
-      return S_OK;
-    
-    std::lock_guard<std::mutex> lockWin(m_lockWindow);
-    std::lock_guard<std::mutex> lockBuf(m_lockBuffer);
+    if (SyncInterval > 4)
+      return DXGI_ERROR_INVALID_CALL;
 
-    // Higher values are not allowed according to the Microsoft documentation:
-    // 
-    //   "1 through 4 - Synchronize presentation after the nth vertical blank."
-    //   https://msdn.microsoft.com/en-us/library/windows/desktop/bb174576(v=vs.85).aspx
-    SyncInterval = std::min<UINT>(SyncInterval, 4);
+    std::lock_guard<std::recursive_mutex> lockWin(m_lockWindow);
+    std::lock_guard<std::mutex> lockBuf(m_lockBuffer);
 
     try {
       return m_presenter->Present(SyncInterval, PresentFlags, nullptr);
@@ -276,6 +275,11 @@ namespace dxvk {
           UINT        SwapChainFlags) {
     if (!IsWindow(m_window))
       return DXGI_ERROR_INVALID_CALL;
+
+    constexpr UINT PreserveFlags = DXGI_SWAP_CHAIN_FLAG_FRAME_LATENCY_WAITABLE_OBJECT;
+
+    if ((m_desc.Flags & PreserveFlags) != (SwapChainFlags & PreserveFlags))
+      return DXGI_ERROR_INVALID_CALL;
     
     std::lock_guard<std::mutex> lock(m_lockBuffer);
     m_desc.Width  = Width;
@@ -314,7 +318,7 @@ namespace dxvk {
 
 
   HRESULT STDMETHODCALLTYPE DxgiSwapChain::ResizeTarget(const DXGI_MODE_DESC* pNewTargetParameters) {
-    std::lock_guard<std::mutex> lock(m_lockWindow);
+    std::lock_guard<std::recursive_mutex> lock(m_lockWindow);
 
     if (pNewTargetParameters == nullptr)
       return DXGI_ERROR_INVALID_CALL;
@@ -372,7 +376,10 @@ namespace dxvk {
   HRESULT STDMETHODCALLTYPE DxgiSwapChain::SetFullscreenState(
           BOOL          Fullscreen,
           IDXGIOutput*  pTarget) {
-    std::lock_guard<std::mutex> lock(m_lockWindow);
+    std::lock_guard<std::recursive_mutex> lock(m_lockWindow);
+
+    if (!Fullscreen && pTarget)
+      return DXGI_ERROR_INVALID_CALL;
 
     if (m_descFs.Windowed && Fullscreen)
       return this->EnterFullscreenMode(pTarget);
@@ -398,8 +405,10 @@ namespace dxvk {
   
   
   HANDLE STDMETHODCALLTYPE DxgiSwapChain::GetFrameLatencyWaitableObject() {
-    Logger::err("DxgiSwapChain::GetFrameLatencyWaitableObject: Not implemented");
-    return nullptr;
+    if (!(m_desc.Flags & DXGI_SWAP_CHAIN_FLAG_FRAME_LATENCY_WAITABLE_OBJECT))
+      return nullptr;
+
+    return m_presenter->GetFrameLatencyEvent();
   }
 
 
@@ -413,8 +422,12 @@ namespace dxvk {
   
   HRESULT STDMETHODCALLTYPE DxgiSwapChain::GetMaximumFrameLatency(
           UINT*                     pMaxLatency) {
-    Logger::err("DxgiSwapChain::GetMaximumFrameLatency: Not implemented");
-    return DXGI_ERROR_INVALID_CALL;
+    if (!(m_desc.Flags & DXGI_SWAP_CHAIN_FLAG_FRAME_LATENCY_WAITABLE_OBJECT))
+      return DXGI_ERROR_INVALID_CALL;
+
+    std::lock_guard<std::recursive_mutex> lock(m_lockWindow);
+    *pMaxLatency = m_presenter->GetFrameLatency();
+    return S_OK;
   }
 
   
@@ -438,8 +451,11 @@ namespace dxvk {
   
   HRESULT STDMETHODCALLTYPE DxgiSwapChain::SetMaximumFrameLatency(
           UINT                      MaxLatency) {
-    Logger::err("DxgiSwapChain::SetMaximumFrameLatency: Not implemented");
-    return DXGI_ERROR_INVALID_CALL;
+    if (!(m_desc.Flags & DXGI_SWAP_CHAIN_FLAG_FRAME_LATENCY_WAITABLE_OBJECT))
+      return DXGI_ERROR_INVALID_CALL;
+
+    std::lock_guard<std::recursive_mutex> lock(m_lockWindow);
+    return m_presenter->SetFrameLatency(MaxLatency);
   }
 
 
@@ -460,18 +476,59 @@ namespace dxvk {
   
 
   HRESULT STDMETHODCALLTYPE DxgiSwapChain::CheckColorSpaceSupport(
-    DXGI_COLOR_SPACE_TYPE           ColorSpace,
-    UINT*                           pColorSpaceSupport) {
-    Logger::err("DxgiSwapChain::CheckColorSpaceSupport: Not implemented");
+          DXGI_COLOR_SPACE_TYPE           ColorSpace,
+          UINT*                           pColorSpaceSupport) {
+    if (!pColorSpaceSupport)
+      return E_INVALIDARG;
+    
+    UINT supportFlags = 0;
 
-    *pColorSpaceSupport = 0;
-    return E_NOTIMPL;
+    if (ColorSpace == DXGI_COLOR_SPACE_RGB_FULL_G22_NONE_P709)
+      supportFlags |= DXGI_SWAP_CHAIN_COLOR_SPACE_SUPPORT_FLAG_PRESENT;
+
+    *pColorSpaceSupport = supportFlags;
+    return S_OK;
   }
 
 
   HRESULT STDMETHODCALLTYPE DxgiSwapChain::SetColorSpace1(DXGI_COLOR_SPACE_TYPE ColorSpace) {
-    Logger::err("DxgiSwapChain::SetColorSpace1: Not implemented");
-    return E_NOTIMPL;
+    UINT support = 0;
+
+    HRESULT hr = CheckColorSpaceSupport(ColorSpace, &support);
+
+    if (FAILED(hr))
+      return hr;
+
+    if (!support)
+      return E_INVALIDARG;
+
+    return S_OK;
+  }
+
+  
+  HRESULT STDMETHODCALLTYPE DxgiSwapChain::SetHDRMetaData(
+          DXGI_HDR_METADATA_TYPE    Type,
+          UINT                      Size,
+          void*                     pMetaData) {
+    if (Size && !pMetaData)
+      return E_INVALIDARG;
+
+    switch (Type) {
+      case DXGI_HDR_METADATA_TYPE_NONE:
+        return S_OK;
+
+      case DXGI_HDR_METADATA_TYPE_HDR10:
+        if (Size != sizeof(DXGI_HDR_METADATA_HDR10))
+          return E_INVALIDARG;
+
+        // For some reason this always seems to succeed on Windows
+        Logger::warn("DXGI: HDR not supported");
+        return S_OK;
+
+      default:
+        Logger::err(str::format("DXGI: Invalid HDR metadata type: ", Type));
+        return E_INVALIDARG;
+    }
   }
   
   
@@ -484,7 +541,7 @@ namespace dxvk {
 
 
   HRESULT DxgiSwapChain::EnterFullscreenMode(IDXGIOutput* pTarget) {
-    Com<IDXGIOutput> output = static_cast<DxgiOutput*>(pTarget);
+    Com<IDXGIOutput> output = pTarget;
 
     if (!IsWindow(m_window))
       return DXGI_ERROR_NOT_CURRENTLY_AVAILABLE;
@@ -543,6 +600,7 @@ namespace dxvk {
       SWP_FRAMECHANGED | SWP_SHOWWINDOW | SWP_NOACTIVATE);
     
     m_monitor = desc.Monitor;
+    m_target  = std::move(output);
 
     // Apply current gamma curve of the output
     DXGI_VK_MONITOR_DATA* monitorInfo = nullptr;
@@ -560,9 +618,6 @@ namespace dxvk {
   
   
   HRESULT DxgiSwapChain::LeaveFullscreenMode() {
-    if (!IsWindow(m_window))
-      return DXGI_ERROR_NOT_CURRENTLY_AVAILABLE;
-    
     if (FAILED(RestoreDisplayMode(m_monitor)))
       Logger::warn("DXGI: LeaveFullscreenMode: Failed to restore display mode");
     
@@ -580,6 +635,10 @@ namespace dxvk {
     // Restore internal state
     m_descFs.Windowed = TRUE;
     m_monitor = nullptr;
+    m_target  = nullptr;
+    
+    if (!IsWindow(m_window))
+      return S_OK;
     
     // Only restore the window style if the application hasn't
     // changed them. This is in line with what native DXGI does.
@@ -684,54 +743,20 @@ namespace dxvk {
     
     return DXGI_ERROR_NOT_FOUND;
   }
+
+
+  HRESULT DxgiSwapChain::AcquireMonitorData(
+          HMONITOR                hMonitor,
+          DXGI_VK_MONITOR_DATA**  ppData) {
+    return m_monitorInfo != nullptr
+      ? m_monitorInfo->AcquireMonitorData(hMonitor, ppData)
+      : E_NOINTERFACE;
+  }
+
   
-  
-  HRESULT CreateDxvkSwapChainForHwnd(
-          IDXGIFactory*             pFactory,
-          IDXGIVkPresentDevice*     pDevice,
-          HWND                      hWnd,
-    const DXGI_SWAP_CHAIN_DESC1*    pDesc,
-    const DXGI_SWAP_CHAIN_FULLSCREEN_DESC* pFullscreenDesc,
-          IDXGIOutput*              pRestrictToOutput,
-          IDXGISwapChain1**         ppSwapChain) {
-    // Make sure the back buffer size is not zero
-    DXGI_SWAP_CHAIN_DESC1 desc = *pDesc;
-    
-    GetWindowClientSize(hWnd,
-      desc.Width  ? nullptr : &desc.Width,
-      desc.Height ? nullptr : &desc.Height);
-    
-    // If necessary, set up a default set of
-    // fullscreen parameters for the swap chain
-    DXGI_SWAP_CHAIN_FULLSCREEN_DESC fsDesc;
-    
-    if (pFullscreenDesc) {
-      fsDesc = *pFullscreenDesc;
-    } else {
-      fsDesc.RefreshRate      = { 0, 0 };
-      fsDesc.ScanlineOrdering = DXGI_MODE_SCANLINE_ORDER_UNSPECIFIED;
-      fsDesc.Scaling          = DXGI_MODE_SCALING_UNSPECIFIED;
-      fsDesc.Windowed         = TRUE;
-    }
-    
-    // Create presenter for the device
-    Com<IDXGIVkSwapChain> presenter;
-    
-    HRESULT hr = pDevice->CreateSwapChainForHwnd(
-      hWnd, &desc, &presenter);
-    
-    if (FAILED(hr))
-      return hr;
-    
-    try {
-      // Create actual swap chain object
-      *ppSwapChain = ref(new DxgiSwapChain(
-        pFactory, presenter.ptr(), hWnd, &desc, &fsDesc));
-      return S_OK;
-    } catch (const DxvkError& e) {
-      Logger::err(e.message());
-      return DXGI_ERROR_UNSUPPORTED;
-    }
+  void DxgiSwapChain::ReleaseMonitorData() {
+    if (m_monitorInfo != nullptr)
+      m_monitorInfo->ReleaseMonitorData();
   }
   
 }
