@@ -2,8 +2,16 @@
 
 namespace dxvk {
   
-  DxvkBarrierSet:: DxvkBarrierSet() { }
-  DxvkBarrierSet::~DxvkBarrierSet() { }
+  DxvkBarrierSet:: DxvkBarrierSet(DxvkCmdBuffer cmdBuffer)
+  : m_cmdBuffer(cmdBuffer) {
+
+  }
+
+
+  DxvkBarrierSet::~DxvkBarrierSet() {
+
+  }
+
   
   void DxvkBarrierSet::accessBuffer(
     const DxvkBufferSliceHandle&    bufSlice,
@@ -13,26 +21,15 @@ namespace dxvk {
           VkAccessFlags             dstAccess) {
     DxvkAccessFlags access = this->getAccessTypes(srcAccess);
     
-    m_srcStages |= srcStages;
-    m_dstStages |= dstStages;
-    
     if (srcStages == VK_PIPELINE_STAGE_BOTTOM_OF_PIPE_BIT
      || dstStages == VK_PIPELINE_STAGE_TOP_OF_PIPE_BIT)
       access.set(DxvkAccess::Write);
     
-    if (access.test(DxvkAccess::Write)) {
-      VkBufferMemoryBarrier barrier;
-      barrier.sType               = VK_STRUCTURE_TYPE_BUFFER_MEMORY_BARRIER;
-      barrier.pNext               = nullptr;
-      barrier.srcAccessMask       = srcAccess;
-      barrier.dstAccessMask       = dstAccess;
-      barrier.srcQueueFamilyIndex = VK_QUEUE_FAMILY_IGNORED;
-      barrier.dstQueueFamilyIndex = VK_QUEUE_FAMILY_IGNORED;
-      barrier.buffer              = bufSlice.handle;
-      barrier.offset              = bufSlice.offset;
-      barrier.size                = bufSlice.length;
-      m_bufBarriers.push_back(barrier);
-    }
+    m_srcStages |= srcStages;
+    m_dstStages |= dstStages;
+    
+    m_srcAccess |= srcAccess;
+    m_dstAccess |= dstAccess;
 
     m_bufSlices.push_back({ bufSlice, access });
   }
@@ -49,15 +46,18 @@ namespace dxvk {
           VkAccessFlags             dstAccess) {
     DxvkAccessFlags access = this->getAccessTypes(srcAccess);
 
-    m_srcStages |= srcStages;
-    m_dstStages |= dstStages;
-    
     if (srcStages == VK_PIPELINE_STAGE_BOTTOM_OF_PIPE_BIT
      || dstStages == VK_PIPELINE_STAGE_TOP_OF_PIPE_BIT
      || srcLayout != dstLayout)
       access.set(DxvkAccess::Write);
     
-    if (access.test(DxvkAccess::Write)) {
+    m_srcStages |= srcStages;
+    m_dstStages |= dstStages;
+    
+    if (srcLayout == dstLayout) {
+      m_srcAccess |= srcAccess;
+      m_dstAccess |= dstAccess;
+    } else {
       VkImageMemoryBarrier barrier;
       barrier.sType                       = VK_STRUCTURE_TYPE_IMAGE_MEMORY_BARRIER;
       barrier.pNext                       = nullptr;
@@ -77,23 +77,90 @@ namespace dxvk {
   }
 
 
-  void DxvkBarrierSet::accessMemory(
+  void DxvkBarrierSet::releaseBuffer(
+          DxvkBarrierSet&           acquire,
+    const DxvkBufferSliceHandle&    bufSlice,
+          uint32_t                  srcQueue,
           VkPipelineStageFlags      srcStages,
           VkAccessFlags             srcAccess,
+          uint32_t                  dstQueue,
           VkPipelineStageFlags      dstStages,
           VkAccessFlags             dstAccess) {
-    m_srcStages |= srcStages;
-    m_dstStages |= dstStages;
+    auto& release = *this;
 
-    m_srcAccess |= srcAccess;
-    m_dstAccess |= dstAccess;
+    release.m_srcStages |= srcStages;
+    acquire.m_dstStages |= dstStages;
+
+    VkBufferMemoryBarrier barrier;
+    barrier.sType                       = VK_STRUCTURE_TYPE_BUFFER_MEMORY_BARRIER;
+    barrier.pNext                       = nullptr;
+    barrier.srcAccessMask               = srcAccess;
+    barrier.dstAccessMask               = 0;
+    barrier.srcQueueFamilyIndex         = srcQueue;
+    barrier.dstQueueFamilyIndex         = dstQueue;
+    barrier.buffer                      = bufSlice.handle;
+    barrier.offset                      = bufSlice.offset;
+    barrier.size                        = bufSlice.length;
+    release.m_bufBarriers.push_back(barrier);
+
+    barrier.srcAccessMask               = 0;
+    barrier.dstAccessMask               = dstAccess;
+    acquire.m_bufBarriers.push_back(barrier);
+
+    DxvkAccessFlags access(DxvkAccess::Read, DxvkAccess::Write);
+    release.m_bufSlices.push_back({ bufSlice, access });
+    acquire.m_bufSlices.push_back({ bufSlice, access });
   }
-  
-  
+
+
+  void DxvkBarrierSet::releaseImage(
+          DxvkBarrierSet&           acquire,
+    const Rc<DxvkImage>&            image,
+    const VkImageSubresourceRange&  subresources,
+          uint32_t                  srcQueue,
+          VkImageLayout             srcLayout,
+          VkPipelineStageFlags      srcStages,
+          VkAccessFlags             srcAccess,
+          uint32_t                  dstQueue,
+          VkImageLayout             dstLayout,
+          VkPipelineStageFlags      dstStages,
+          VkAccessFlags             dstAccess) {
+    auto& release = *this;
+
+    release.m_srcStages |= srcStages;
+    acquire.m_dstStages |= dstStages;
+
+    VkImageMemoryBarrier barrier;
+    barrier.sType                       = VK_STRUCTURE_TYPE_IMAGE_MEMORY_BARRIER;
+    barrier.pNext                       = nullptr;
+    barrier.srcAccessMask               = srcAccess;
+    barrier.dstAccessMask               = 0;
+    barrier.oldLayout                   = srcLayout;
+    barrier.newLayout                   = dstLayout;
+    barrier.srcQueueFamilyIndex         = srcQueue;
+    barrier.dstQueueFamilyIndex         = dstQueue;
+    barrier.image                       = image->handle();
+    barrier.subresourceRange            = subresources;
+    barrier.subresourceRange.aspectMask = image->formatInfo()->aspectMask;
+    release.m_imgBarriers.push_back(barrier);
+
+    if (srcQueue == dstQueue)
+      barrier.oldLayout = dstLayout;
+
+    barrier.srcAccessMask               = 0;
+    barrier.dstAccessMask               = dstAccess;
+    acquire.m_imgBarriers.push_back(barrier);
+
+    DxvkAccessFlags access(DxvkAccess::Read, DxvkAccess::Write);
+    release.m_imgSlices.push_back({ image.ptr(), subresources, access });
+    acquire.m_imgSlices.push_back({ image.ptr(), subresources, access });
+  }
+
+
   bool DxvkBarrierSet::isBufferDirty(
     const DxvkBufferSliceHandle&    bufSlice,
           DxvkAccessFlags           bufAccess) {
-    bool result = m_srcAccess || m_dstAccess;
+    bool result = false;
 
     for (uint32_t i = 0; i < m_bufSlices.size() && !result; i++) {
       const DxvkBufferSliceHandle& dstSlice = m_bufSlices[i].slice;
@@ -111,8 +178,7 @@ namespace dxvk {
     const Rc<DxvkImage>&            image,
     const VkImageSubresourceRange&  imgSubres,
           DxvkAccessFlags           imgAccess) {
-    bool result = (m_srcStages & image->info().stages)
-               && (m_srcAccess & image->info().access);
+    bool result = false;
 
     for (uint32_t i = 0; i < m_imgSlices.size() && !result; i++) {
       const VkImageSubresourceRange& dstSubres = m_imgSlices[i].subres;
@@ -126,7 +192,44 @@ namespace dxvk {
 
     return result;
   }
+
+
+  DxvkAccessFlags DxvkBarrierSet::getBufferAccess(
+    const DxvkBufferSliceHandle&    bufSlice) {
+    DxvkAccessFlags access;
+
+    for (uint32_t i = 0; i < m_bufSlices.size(); i++) {
+      const DxvkBufferSliceHandle& dstSlice = m_bufSlices[i].slice;
+
+      if ((bufSlice.handle == dstSlice.handle)
+       && (bufSlice.offset + bufSlice.length > dstSlice.offset)
+       && (bufSlice.offset < dstSlice.offset + dstSlice.length))
+        access = access | m_bufSlices[i].access;
+    }
+
+    return access;
+  }
+
   
+  DxvkAccessFlags DxvkBarrierSet::getImageAccess(
+    const Rc<DxvkImage>&            image,
+    const VkImageSubresourceRange&  imgSubres) {
+    DxvkAccessFlags access;
+
+    for (uint32_t i = 0; i < m_imgSlices.size(); i++) {
+      const VkImageSubresourceRange& dstSubres = m_imgSlices[i].subres;
+
+      if ((image == m_imgSlices[i].image)
+       && (imgSubres.baseArrayLayer < dstSubres.baseArrayLayer + dstSubres.layerCount)
+       && (imgSubres.baseArrayLayer + imgSubres.layerCount     > dstSubres.baseArrayLayer)
+       && (imgSubres.baseMipLevel   < dstSubres.baseMipLevel   + dstSubres.levelCount)
+       && (imgSubres.baseMipLevel   + imgSubres.levelCount     > dstSubres.baseMipLevel))
+        access = access | m_imgSlices[i].access;
+    }
+
+    return access;
+  }
+
 
   void DxvkBarrierSet::recordCommands(const Rc<DxvkCommandList>& commandList) {
     if (m_srcStages | m_dstStages) {
@@ -147,10 +250,12 @@ namespace dxvk {
         pMemBarrier = &memBarrier;
       
       commandList->cmdPipelineBarrier(
-        srcFlags, dstFlags, 0,
+        m_cmdBuffer, srcFlags, dstFlags, 0,
         pMemBarrier ? 1 : 0, pMemBarrier,
-        m_bufBarriers.size(), m_bufBarriers.data(),
-        m_imgBarriers.size(), m_imgBarriers.data());
+        m_bufBarriers.size(),
+        m_bufBarriers.data(),
+        m_imgBarriers.size(),
+        m_imgBarriers.data());
       
       this->reset();
     }
@@ -160,6 +265,9 @@ namespace dxvk {
   void DxvkBarrierSet::reset() {
     m_srcStages = 0;
     m_dstStages = 0;
+
+    m_srcAccess = 0;
+    m_dstAccess = 0;
     
     m_bufBarriers.resize(0);
     m_imgBarriers.resize(0);
