@@ -1,6 +1,8 @@
 #include "dxvk_buffer.h"
 #include "dxvk_device.h"
 
+#include <algorithm>
+
 namespace dxvk {
   
   DxvkBuffer::DxvkBuffer(
@@ -12,18 +14,31 @@ namespace dxvk {
     m_info          (createInfo),
     m_memAlloc      (&memAlloc),
     m_memFlags      (memFlags) {
-    // Align slices to 256 bytes, which guarantees that
-    // we don't violate any Vulkan alignment requirements
+    // Align slices so that we don't violate any alignment
+    // requirements imposed by the Vulkan device/driver
+    VkDeviceSize sliceAlignment = computeSliceAlignment();
     m_physSliceLength = createInfo.size;
-    m_physSliceStride = align(createInfo.size, 256);
-    
-    // Allocate a single buffer slice
-    m_buffer = allocBuffer(1);
+    m_physSliceStride = align(createInfo.size, sliceAlignment);
+    m_physSliceCount  = std::max<VkDeviceSize>(1, 256 / m_physSliceStride);
 
-    m_physSlice.handle = m_buffer.buffer;
-    m_physSlice.offset = 0;
-    m_physSlice.length = m_physSliceLength;
-    m_physSlice.mapPtr = m_buffer.memory.mapPtr(0);
+    // Limit size of multi-slice buffers to reduce fragmentation
+    constexpr VkDeviceSize MaxBufferSize = 4 << 20;
+
+    m_physSliceMaxCount = MaxBufferSize >= m_physSliceStride
+      ? MaxBufferSize / m_physSliceStride
+      : 1;
+
+    // Allocate the initial set of buffer slices
+    m_buffer = allocBuffer(m_physSliceCount);
+
+    DxvkBufferSliceHandle slice;
+    slice.handle = m_buffer.buffer;
+    slice.offset = 0;
+    slice.length = m_physSliceLength;
+    slice.mapPtr = m_buffer.memory.mapPtr(0);
+
+    m_physSlice = slice;
+    m_lazyAlloc = m_physSliceCount > 1;
   }
 
 
@@ -36,48 +51,6 @@ namespace dxvk {
   }
   
   
-  DxvkBufferSliceHandle DxvkBuffer::allocSlice() {
-    std::unique_lock<sync::Spinlock> freeLock(m_freeMutex);
-    
-    // If no slices are available, swap the two free lists.
-    if (m_freeSlices.size() == 0) {
-      std::unique_lock<sync::Spinlock> swapLock(m_swapMutex);
-      std::swap(m_freeSlices, m_nextSlices);
-    }
-
-    // If there are still no slices available, create a new
-    // backing buffer and add all slices to the free list.
-    if (m_freeSlices.size() == 0) {
-      std::unique_lock<sync::Spinlock> swapLock(m_swapMutex);
-      DxvkBufferHandle handle = allocBuffer(m_physSliceCount);
-      
-      for (uint32_t i = 0; i < m_physSliceCount; i++) {
-        DxvkBufferSliceHandle slice;
-        slice.handle = handle.buffer;
-        slice.offset = m_physSliceStride * i;
-        slice.length = m_physSliceLength;
-        slice.mapPtr = handle.memory.mapPtr(slice.offset);
-        m_freeSlices.push_back(slice);
-      }
-      
-      m_buffers.push_back(std::move(handle));
-      m_physSliceCount *= 2;
-    }
-    
-    // Take the first slice from the queue
-    DxvkBufferSliceHandle result = std::move(m_freeSlices.back());
-    m_freeSlices.pop_back();
-    return result;
-  }
-  
-  
-  void DxvkBuffer::freeSlice(const DxvkBufferSliceHandle& slice) {
-    // Add slice to a separate free list to reduce lock contention.
-    std::unique_lock<sync::Spinlock> swapLock(m_swapMutex);
-    m_nextSlices.push_back(slice);
-  }
-  
-  
   DxvkBufferHandle DxvkBuffer::allocBuffer(VkDeviceSize sliceCount) const {
     auto vkd = m_device->vkd();
 
@@ -101,33 +74,40 @@ namespace dxvk {
         "\n  usage: ", info.usage));
     }
     
-    VkMemoryDedicatedRequirementsKHR dedicatedRequirements;
-    dedicatedRequirements.sType                       = VK_STRUCTURE_TYPE_MEMORY_DEDICATED_REQUIREMENTS_KHR;
+    VkMemoryDedicatedRequirements dedicatedRequirements;
+    dedicatedRequirements.sType                       = VK_STRUCTURE_TYPE_MEMORY_DEDICATED_REQUIREMENTS;
     dedicatedRequirements.pNext                       = VK_NULL_HANDLE;
     dedicatedRequirements.prefersDedicatedAllocation  = VK_FALSE;
     dedicatedRequirements.requiresDedicatedAllocation = VK_FALSE;
     
-    VkMemoryRequirements2KHR memReq;
-    memReq.sType = VK_STRUCTURE_TYPE_MEMORY_REQUIREMENTS_2_KHR;
+    VkMemoryRequirements2 memReq;
+    memReq.sType = VK_STRUCTURE_TYPE_MEMORY_REQUIREMENTS_2;
     memReq.pNext = &dedicatedRequirements;
     
-    VkBufferMemoryRequirementsInfo2KHR memReqInfo;
-    memReqInfo.sType  = VK_STRUCTURE_TYPE_BUFFER_MEMORY_REQUIREMENTS_INFO_2_KHR;
+    VkBufferMemoryRequirementsInfo2 memReqInfo;
+    memReqInfo.sType  = VK_STRUCTURE_TYPE_BUFFER_MEMORY_REQUIREMENTS_INFO_2;
     memReqInfo.buffer = handle.buffer;
     memReqInfo.pNext  = VK_NULL_HANDLE;
     
-    VkMemoryDedicatedAllocateInfoKHR dedMemoryAllocInfo;
-    dedMemoryAllocInfo.sType  = VK_STRUCTURE_TYPE_MEMORY_DEDICATED_ALLOCATE_INFO_KHR;
+    VkMemoryDedicatedAllocateInfo dedMemoryAllocInfo;
+    dedMemoryAllocInfo.sType  = VK_STRUCTURE_TYPE_MEMORY_DEDICATED_ALLOCATE_INFO;
     dedMemoryAllocInfo.pNext  = VK_NULL_HANDLE;
     dedMemoryAllocInfo.buffer = handle.buffer;
     dedMemoryAllocInfo.image  = VK_NULL_HANDLE;
 
-    vkd->vkGetBufferMemoryRequirements2KHR(
+    vkd->vkGetBufferMemoryRequirements2(
        vkd->device(), &memReqInfo, &memReq);
 
-    bool useDedicated = dedicatedRequirements.prefersDedicatedAllocation;
+    // Use high memory priority for GPU-writable resources
+    bool isGpuWritable = (m_info.usage & (
+      VK_BUFFER_USAGE_STORAGE_BUFFER_BIT |
+      VK_BUFFER_USAGE_STORAGE_TEXEL_BUFFER_BIT)) != 0;
+    
+    float priority = isGpuWritable ? 1.0f : 0.5f;
+    
+    // Ask driver whether we should be using a dedicated allocation
     handle.memory = m_memAlloc->alloc(&memReq.memoryRequirements,
-      useDedicated ? &dedMemoryAllocInfo : nullptr, m_memFlags);
+      dedicatedRequirements, dedMemoryAllocInfo, m_memFlags, priority);
     
     if (vkd->vkBindBufferMemory(vkd->device(), handle.buffer,
         handle.memory.memory(), handle.memory.offset()) != VK_SUCCESS)
@@ -137,6 +117,40 @@ namespace dxvk {
   }
 
 
+  VkDeviceSize DxvkBuffer::computeSliceAlignment() const {
+    const auto& devInfo = m_device->properties().core.properties;
+
+    VkDeviceSize result = sizeof(uint32_t);
+
+    if (m_info.usage & VK_BUFFER_USAGE_UNIFORM_BUFFER_BIT)
+      result = std::max(result, devInfo.limits.minUniformBufferOffsetAlignment);
+
+    if (m_info.usage & VK_BUFFER_USAGE_STORAGE_BUFFER_BIT)
+      result = std::max(result, devInfo.limits.minStorageBufferOffsetAlignment);
+
+    if (m_info.usage & (VK_BUFFER_USAGE_STORAGE_TEXEL_BUFFER_BIT | VK_BUFFER_USAGE_UNIFORM_TEXEL_BUFFER_BIT)) {
+      result = std::max(result, devInfo.limits.minTexelBufferOffsetAlignment);
+      result = std::max(result, VkDeviceSize(16));
+    }
+
+    if (m_info.usage & (VK_BUFFER_USAGE_TRANSFER_SRC_BIT | VK_BUFFER_USAGE_TRANSFER_DST_BIT)
+     && m_info.size > (devInfo.limits.optimalBufferCopyOffsetAlignment / 2))
+      result = std::max(result, devInfo.limits.optimalBufferCopyOffsetAlignment);
+
+    // For some reason, Warhammer Chaosbane breaks otherwise
+    if (m_info.usage & (VK_BUFFER_USAGE_VERTEX_BUFFER_BIT | VK_BUFFER_USAGE_INDEX_BUFFER_BIT))
+      result = std::max(result, VkDeviceSize(256));
+
+    if (m_memFlags & VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT) {
+      result = std::max(result, devInfo.limits.nonCoherentAtomSize);
+      result = std::max(result, VkDeviceSize(64));
+    }
+
+    return result;
+  }
+
+
+
   
   DxvkBufferView::DxvkBufferView(
     const Rc<vk::DeviceFn>&         vkd,
@@ -209,14 +223,12 @@ namespace dxvk {
   DxvkBufferTracker::~DxvkBufferTracker() { }
   
   
-  void DxvkBufferTracker::freeBufferSlice(
-    const Rc<DxvkBuffer>&         buffer,
-    const DxvkBufferSliceHandle&  slice) {
-    m_entries.push_back({ buffer, slice });
-  }
-  
-  
   void DxvkBufferTracker::reset() {
+    std::sort(m_entries.begin(), m_entries.end(),
+      [] (const Entry& a, const Entry& b) {
+        return a.slice.handle < b.slice.handle;
+      });
+
     for (const auto& e : m_entries)
       e.buffer->freeSlice(e.slice);
       
