@@ -2,6 +2,7 @@
 
 #include "dxvk_instance.h"
 #include "dxvk_openvr.h"
+#include "dxvk_platform_exts.h"
 
 #include <algorithm>
 
@@ -12,23 +13,37 @@ namespace dxvk {
     Logger::info(str::format("DXVK: ", DXVK_VERSION));
 
     m_config = Config::getUserConfig();
-    m_config.merge(Config::getAppConfig(env::getExeName()));
+    m_config.merge(Config::getAppConfig(env::getExePath()));
     m_config.logOptions();
 
-    g_vrInstance.initInstanceExtensions();
+    m_options = DxvkOptions(m_config);
+
+    m_extProviders.push_back(&DxvkPlatformExts::s_instance);
+
+    if (m_options.enableOpenVR)
+      m_extProviders.push_back(&VrInstance::s_instance);
+
+    Logger::info("Built-in extension providers:");
+    for (const auto& provider : m_extProviders)
+      Logger::info(str::format("  ", provider->getName()));
+
+    for (const auto& provider : m_extProviders)
+      provider->initInstanceExtensions();
 
     m_vkl = new vk::LibraryFn();
     m_vki = new vk::InstanceFn(true, this->createInstance());
 
     m_adapters = this->queryAdapters();
-    g_vrInstance.initDeviceExtensions(this);
+
+    for (const auto& provider : m_extProviders)
+      provider->initDeviceExtensions(this);
 
     for (uint32_t i = 0; i < m_adapters.size(); i++) {
-      m_adapters[i]->enableExtensions(
-        g_vrInstance.getDeviceExtensions(i));
+      for (const auto& provider : m_extProviders) {
+        m_adapters[i]->enableExtensions(
+          provider->getDeviceExtensions(i));
+      }
     }
-
-    m_options = DxvkOptions(m_config);
   }
   
   
@@ -72,10 +87,9 @@ namespace dxvk {
   VkInstance DxvkInstance::createInstance() {
     DxvkInstanceExtensions insExtensions;
 
-    std::array<DxvkExt*, 3> insExtensionList = {{
-      &insExtensions.khrGetPhysicalDeviceProperties2,
+    std::array<DxvkExt*, 2> insExtensionList = {{
+      &insExtensions.khrGetSurfaceCapabilities2,
       &insExtensions.khrSurface,
-      &insExtensions.khrWin32Surface,
     }};
 
     DxvkNameSet extensionsEnabled;
@@ -86,9 +100,11 @@ namespace dxvk {
           insExtensionList.data(),
           extensionsEnabled))
       throw DxvkError("DxvkInstance: Failed to create instance");
-    
+
     // Enable additional extensions if necessary
-    extensionsEnabled.merge(g_vrInstance.getInstanceExtensions());
+    for (const auto& provider : m_extProviders)
+      extensionsEnabled.merge(provider->getInstanceExtensions());
+
     DxvkNameList extensionNameList = extensionsEnabled.toNameList();
     
     Logger::info("Enabled instance extensions:");
@@ -102,7 +118,7 @@ namespace dxvk {
     appInfo.pApplicationName      = appName.c_str();
     appInfo.applicationVersion    = 0;
     appInfo.pEngineName           = "DXVK";
-    appInfo.engineVersion         = VK_MAKE_VERSION(0, 9, 6);
+    appInfo.engineVersion         = VK_MAKE_VERSION(1, 5, 4);
     appInfo.apiVersion            = VK_MAKE_VERSION(1, 1, 0);
     
     VkInstanceCreateInfo info;
@@ -118,14 +134,8 @@ namespace dxvk {
     VkInstance result = VK_NULL_HANDLE;
     VkResult status = m_vkl->vkCreateInstance(&info, nullptr, &result);
 
-    if (status == VK_ERROR_INCOMPATIBLE_DRIVER) {
-      Logger::warn("Failed to create Vulkan 1.1 instance, falling back to 1.0");
-      appInfo.apiVersion = 0; /* some very old drivers may not accept 1.0 */
-      status = m_vkl->vkCreateInstance(&info, nullptr, &result);
-    }
-
     if (status != VK_SUCCESS)
-      throw DxvkError("DxvkInstance::createInstance: Failed to create Vulkan instance");
+      throw DxvkError("DxvkInstance::createInstance: Failed to create Vulkan 1.1 instance");
     
     return result;
   }
@@ -144,14 +154,17 @@ namespace dxvk {
     
     std::vector<Rc<DxvkAdapter>> result;
     for (uint32_t i = 0; i < numAdapters; i++) {
-      Rc<DxvkAdapter> adapter = new DxvkAdapter(this, adapters[i]);
-      
-      if (filter.testAdapter(adapter))
-        result.push_back(adapter);
+      VkPhysicalDeviceProperties deviceProperties;
+      m_vki->vkGetPhysicalDeviceProperties(adapters[i], &deviceProperties);
+
+      if (deviceProperties.apiVersion < VK_MAKE_VERSION(1, 1, 0))
+        Logger::warn(str::format("Skipping Vulkan 1.0 adapter: ", deviceProperties.deviceName));
+      else if (filter.testAdapter(deviceProperties))
+        result.push_back(new DxvkAdapter(m_vki, adapters[i]));
     }
     
     std::sort(result.begin(), result.end(),
-      [this] (const Rc<DxvkAdapter>& a, const Rc<DxvkAdapter>& b) -> bool {
+      [] (const Rc<DxvkAdapter>& a, const Rc<DxvkAdapter>& b) -> bool {
         return a->deviceProperties().deviceType == VK_PHYSICAL_DEVICE_TYPE_DISCRETE_GPU
             && b->deviceProperties().deviceType != VK_PHYSICAL_DEVICE_TYPE_DISCRETE_GPU;
       });
