@@ -75,10 +75,12 @@ namespace dxvk {
   DxvkMemory DxvkMemoryChunk::alloc(
           VkMemoryPropertyFlags flags,
           VkDeviceSize          size,
-          VkDeviceSize          align) {
+          VkDeviceSize          align,
+          float                 priority) {
     // Property flags must be compatible. This could
     // be refined a bit in the future if necessary.
-    if (m_memory.memFlags != flags)
+    if (m_memory.memFlags != flags
+     || m_memory.priority != priority)
       return DxvkMemory();
     
     // If the chunk is full, return
@@ -152,22 +154,23 @@ namespace dxvk {
   
   DxvkMemoryAllocator::DxvkMemoryAllocator(const DxvkDevice* device)
   : m_vkd             (device->vkd()),
+    m_device          (device),
     m_devProps        (device->adapter()->deviceProperties()),
-    m_memProps        (device->adapter()->memoryProperties()),
-    m_allowOvercommit (device->config().allowMemoryOvercommit) {
+    m_memProps        (device->adapter()->memoryProperties()) {
     for (uint32_t i = 0; i < m_memProps.memoryHeapCount; i++) {
-      VkDeviceSize heapSize = m_memProps.memoryHeaps[i].size;
-      
       m_memHeaps[i].properties = m_memProps.memoryHeaps[i];
-      m_memHeaps[i].chunkSize  = pickChunkSize(heapSize);
       m_memHeaps[i].stats      = DxvkMemoryStats { 0, 0 };
     }
     
     for (uint32_t i = 0; i < m_memProps.memoryTypeCount; i++) {
       m_memTypes[i].heap       = &m_memHeaps[m_memProps.memoryTypes[i].heapIndex];
+      m_memTypes[i].heapId     = m_memProps.memoryTypes[i].heapIndex;
       m_memTypes[i].memType    = m_memProps.memoryTypes[i];
       m_memTypes[i].memTypeId  = i;
+      m_memTypes[i].chunkSize  = pickChunkSize(i);
     }
+
+    m_restrictAllocations = m_device->isUnifiedMemoryArchitecture();
   }
   
   
@@ -178,22 +181,52 @@ namespace dxvk {
   
   DxvkMemory DxvkMemoryAllocator::alloc(
     const VkMemoryRequirements*             req,
-    const VkMemoryDedicatedAllocateInfoKHR* dedAllocInfo,
-          VkMemoryPropertyFlags             flags) {
+    const VkMemoryDedicatedRequirements&    dedAllocReq,
+    const VkMemoryDedicatedAllocateInfo&    dedAllocInfo,
+          VkMemoryPropertyFlags             flags,
+          float                             priority) {
     std::lock_guard<std::mutex> lock(m_mutex);
 
-    DxvkMemory result = this->tryAlloc(req, dedAllocInfo, flags);
+    // Try to allocate from a memory type which supports the given flags exactly
+    auto dedAllocPtr = dedAllocReq.prefersDedicatedAllocation ? &dedAllocInfo : nullptr;
+    DxvkMemory result = this->tryAlloc(req, dedAllocPtr, flags, priority);
+
+    // If the first attempt failed, try ignoring the dedicated allocation
+    if (!result && dedAllocPtr && !dedAllocReq.requiresDedicatedAllocation) {
+      result = this->tryAlloc(req, nullptr, flags, priority);
+      dedAllocPtr = nullptr;
+    }
+
+    // If that still didn't work, probe slower memory types as well
+    VkMemoryPropertyFlags optFlags = VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT
+                                   | VK_MEMORY_PROPERTY_HOST_CACHED_BIT;
     
-    if (!result && (flags & VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT))
-      result = this->tryAlloc(req, dedAllocInfo, flags & ~VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT);
+    if (!result && (flags & optFlags))
+      result = this->tryAlloc(req, dedAllocPtr, flags & ~optFlags, priority);
     
     if (!result) {
+      DxvkAdapterMemoryInfo memHeapInfo = m_device->adapter()->getMemoryHeapInfo();
+
       Logger::err(str::format(
         "DxvkMemoryAllocator: Memory allocation failed",
         "\n  Size:      ", req->size,
         "\n  Alignment: ", req->alignment,
         "\n  Mem flags: ", "0x", std::hex, flags,
         "\n  Mem types: ", "0x", std::hex, req->memoryTypeBits));
+
+      for (uint32_t i = 0; i < m_memProps.memoryHeapCount; i++) {
+        Logger::err(str::format("Heap ", i, ": ",
+          (m_memHeaps[i].stats.memoryAllocated >> 20), " MB allocated, ",
+          (m_memHeaps[i].stats.memoryUsed      >> 20), " MB used, ",
+          m_device->extensions().extMemoryBudget
+            ? str::format(
+                (memHeapInfo.heaps[i].memoryAllocated >> 20), " MB allocated (driver), ",
+                (memHeapInfo.heaps[i].memoryBudget    >> 20), " MB budget (driver), ",
+                (m_memHeaps[i].properties.size        >> 20), " MB total")
+            : str::format(
+                (m_memHeaps[i].properties.size        >> 20), " MB total")));
+      }
+
       throw DxvkError("DxvkMemoryAllocator: Memory allocation failed");
     }
     
@@ -201,24 +234,11 @@ namespace dxvk {
   }
   
   
-  DxvkMemoryStats DxvkMemoryAllocator::getMemoryStats() {
-    std::lock_guard<std::mutex> lock(m_mutex);
-
-    DxvkMemoryStats totalStats;
-    
-    for (size_t i = 0; i < m_memProps.memoryHeapCount; i++) {
-      totalStats.memoryAllocated += m_memHeaps[i].stats.memoryAllocated;
-      totalStats.memoryUsed      += m_memHeaps[i].stats.memoryUsed;
-    }
-      
-    return totalStats;
-  }
-  
-  
   DxvkMemory DxvkMemoryAllocator::tryAlloc(
     const VkMemoryRequirements*             req,
-    const VkMemoryDedicatedAllocateInfoKHR* dedAllocInfo,
-          VkMemoryPropertyFlags             flags) {
+    const VkMemoryDedicatedAllocateInfo*    dedAllocInfo,
+          VkMemoryPropertyFlags             flags,
+          float                             priority) {
     DxvkMemory result;
 
     for (uint32_t i = 0; i < m_memProps.memoryTypeCount && !result; i++) {
@@ -227,7 +247,7 @@ namespace dxvk {
       
       if (supported && adequate) {
         result = this->tryAllocFromType(&m_memTypes[i],
-          flags, req->size, req->alignment, dedAllocInfo);
+          flags, req->size, req->alignment, priority, dedAllocInfo);
       }
     }
     
@@ -240,30 +260,38 @@ namespace dxvk {
           VkMemoryPropertyFlags             flags,
           VkDeviceSize                      size,
           VkDeviceSize                      align,
-    const VkMemoryDedicatedAllocateInfoKHR* dedAllocInfo) {
+          float                             priority,
+    const VkMemoryDedicatedAllocateInfo*    dedAllocInfo) {
+    // Prevent unnecessary external host memory fragmentation
+    bool isDeviceLocal = (flags & VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT) != 0;
+
+    if (!isDeviceLocal)
+      priority = 0.0f;
+
     DxvkMemory memory;
 
-    if ((size >= type->heap->chunkSize / 4) || dedAllocInfo) {
+    if (size >= type->chunkSize || dedAllocInfo) {
       DxvkDeviceMemory devMem = this->tryAllocDeviceMemory(
-        type, flags, size, dedAllocInfo);
+        type, flags, size, priority, dedAllocInfo);
 
       if (devMem.memHandle != VK_NULL_HANDLE)
         memory = DxvkMemory(this, nullptr, type, devMem.memHandle, 0, size, devMem.memPointer);
     } else {
       for (uint32_t i = 0; i < type->chunks.size() && !memory; i++)
-        memory = type->chunks[i]->alloc(flags, size, align);
+        memory = type->chunks[i]->alloc(flags, size, align, priority);
       
       if (!memory) {
-        DxvkDeviceMemory devMem = tryAllocDeviceMemory(
-          type, flags, type->heap->chunkSize, nullptr);
-
-        if (devMem.memHandle == VK_NULL_HANDLE)
-          return DxvkMemory();
+        DxvkDeviceMemory devMem;
         
-        Rc<DxvkMemoryChunk> chunk = new DxvkMemoryChunk(this, type, devMem);
-        memory = chunk->alloc(flags, size, align);
+        for (uint32_t i = 0; i < 6 && (type->chunkSize >> i) >= size && !devMem.memHandle; i++)
+          devMem = tryAllocDeviceMemory(type, flags, type->chunkSize >> i, priority, nullptr);
 
-        type->chunks.push_back(std::move(chunk));
+        if (devMem.memHandle) {
+          Rc<DxvkMemoryChunk> chunk = new DxvkMemoryChunk(this, type, devMem);
+          memory = chunk->alloc(flags, size, align, priority);
+
+          type->chunks.push_back(std::move(chunk));
+        }
       }
     }
 
@@ -278,19 +306,27 @@ namespace dxvk {
           DxvkMemoryType*                   type,
           VkMemoryPropertyFlags             flags,
           VkDeviceSize                      size,
-    const VkMemoryDedicatedAllocateInfoKHR* dedAllocInfo) {
-    if ((type->memType.propertyFlags & VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT)
-     && (type->heap->stats.memoryAllocated + size > type->heap->properties.size)
-     && (!m_allowOvercommit))
-      return DxvkDeviceMemory();
+          float                             priority,
+    const VkMemoryDedicatedAllocateInfo*    dedAllocInfo) {
+    bool useMemoryPriority = (flags & VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT)
+                          && (m_device->features().extMemoryPriority.memoryPriority);
     
+    if (m_restrictAllocations && type->heap->stats.memoryAllocated + size > type->heap->properties.size)
+      return DxvkDeviceMemory();
+
     DxvkDeviceMemory result;
     result.memSize  = size;
     result.memFlags = flags;
+    result.priority = priority;
+
+    VkMemoryPriorityAllocateInfoEXT prio;
+    prio.sType            = VK_STRUCTURE_TYPE_MEMORY_PRIORITY_ALLOCATE_INFO_EXT;
+    prio.pNext            = dedAllocInfo;
+    prio.priority         = priority;
 
     VkMemoryAllocateInfo info;
     info.sType            = VK_STRUCTURE_TYPE_MEMORY_ALLOCATE_INFO;
-    info.pNext            = dedAllocInfo;
+    info.pNext            = useMemoryPriority ? &prio : prio.pNext;
     info.allocationSize   = size;
     info.memoryTypeIndex  = type->memTypeId;
 
@@ -302,11 +338,13 @@ namespace dxvk {
 
       if (status != VK_SUCCESS) {
         Logger::err(str::format("DxvkMemoryAllocator: Mapping memory failed with ", status));
+        m_vkd->vkFreeMemory(m_vkd->device(), result.memHandle, nullptr);
         return DxvkDeviceMemory();
       }
     }
 
     type->heap->stats.memoryAllocated += size;
+    m_device->adapter()->notifyHeapMemoryAlloc(type->heapId, size);
     return result;
   }
 
@@ -346,17 +384,30 @@ namespace dxvk {
           DxvkDeviceMemory      memory) {
     m_vkd->vkFreeMemory(m_vkd->device(), memory.memHandle, nullptr);
     type->heap->stats.memoryAllocated -= memory.memSize;
+    m_device->adapter()->notifyHeapMemoryFree(type->heapId, memory.memSize);
   }
 
 
-  VkDeviceSize DxvkMemoryAllocator::pickChunkSize(VkDeviceSize heapSize) const {
-    // Pick a reasonable chunk size depending on the memory
-    // heap size. Small chunk sizes can reduce fragmentation
-    // and are therefore preferred for small memory heaps.
-    constexpr VkDeviceSize MaxChunkSize  = 64 * 1024 * 1024;
-    constexpr VkDeviceSize MinChunkCount = 16;
+  VkDeviceSize DxvkMemoryAllocator::pickChunkSize(uint32_t memTypeId) const {
+    VkMemoryType type = m_memProps.memoryTypes[memTypeId];
+    VkMemoryHeap heap = m_memProps.memoryHeaps[type.heapIndex];
 
-    return std::min(heapSize / MinChunkCount, MaxChunkSize);
+    // Default to a chunk size of 128 MiB
+    VkDeviceSize chunkSize = 128 << 20;
+
+    // Try to waste a bit less system memory in 32-bit
+    // applications due to address space constraints
+    #ifndef _WIN64
+    if (type.propertyFlags & VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT)
+      chunkSize = 32 << 20;
+    #endif
+
+    // Reduce the chunk size on small heaps so
+    // we can at least fit in 15 allocations
+    while (chunkSize * 15 > heap.size)
+      chunkSize >>= 1;
+
+    return chunkSize;
   }
   
 }
\ No newline at end of file
