@@ -31,6 +31,7 @@ namespace dxvk {
     void*                 memPointer = nullptr;
     VkDeviceSize          memSize    = 0;
     VkMemoryPropertyFlags memFlags   = 0;
+    float                 priority   = 0.0f;
   };
 
   
@@ -42,7 +43,6 @@ namespace dxvk {
    */
   struct DxvkMemoryHeap {
     VkMemoryHeap      properties;
-    VkDeviceSize      chunkSize;
     DxvkMemoryStats   stats;
   };
 
@@ -61,6 +61,8 @@ namespace dxvk {
     VkMemoryType      memType;
     uint32_t          memTypeId;
 
+    VkDeviceSize      chunkSize;
+
     std::vector<Rc<DxvkMemoryChunk>> chunks;
   };
   
@@ -120,6 +122,15 @@ namespace dxvk {
       return reinterpret_cast<char*>(m_mapPtr) + offset;
     }
 
+    /**
+     * \brief Returns length of memory allocated
+     * 
+     * \returns Memory size
+     */
+    VkDeviceSize length() const {
+      return m_length;
+    }
+
     /**
      * \brief Checks whether the memory slice is defined
      * 
@@ -170,12 +181,14 @@ namespace dxvk {
      * \param [in] flags Requested memory flags
      * \param [in] size Number of bytes to allocate
      * \param [in] align Required alignment
+     * \param [in] priority Requested priority
      * \returns The allocated memory slice
      */
     DxvkMemory alloc(
             VkMemoryPropertyFlags flags,
             VkDeviceSize          size,
-            VkDeviceSize          align);
+            VkDeviceSize          align,
+            float                 priority);
     
     /**
      * \brief Frees memory
@@ -212,7 +225,7 @@ namespace dxvk {
    * Allocates device memory for Vulkan resources.
    * Memory objects will be destroyed automatically.
    */
-  class DxvkMemoryAllocator : public RcObject {
+  class DxvkMemoryAllocator {
     friend class DxvkMemory;
     friend class DxvkMemoryChunk;
   public:
@@ -236,52 +249,64 @@ namespace dxvk {
      * \brief Allocates device memory
      * 
      * \param [in] req Memory requirements
-     * \param [in] flats Memory type flags
+     * \param [in] dedAllocReq Dedicated allocation requirements
+     * \param [in] dedAllocInfo Dedicated allocation info
+     * \param [in] flags Memory type flags
+     * \param [in] priority Device-local memory priority
      * \returns Allocated memory slice
      */
     DxvkMemory alloc(
       const VkMemoryRequirements*             req,
-      const VkMemoryDedicatedAllocateInfoKHR* dedAllocInfo,
-            VkMemoryPropertyFlags             flags);
+      const VkMemoryDedicatedRequirements&    dedAllocReq,
+      const VkMemoryDedicatedAllocateInfo&    dedAllocInfo,
+            VkMemoryPropertyFlags             flags,
+            float                             priority);
     
     /**
      * \brief Queries memory stats
      * 
-     * Returns the total amount of device memory
-     * allocated and used by all available heaps.
-     * \returns Global memory stats
+     * Returns the total amount of memory
+     * allocated and used for a given heap.
+     * \param [in] heap Heap index
+     * \returns Memory stats for this heap
      */
-    DxvkMemoryStats getMemoryStats();
+    DxvkMemoryStats getMemoryStats(uint32_t heap) const {
+      return m_memHeaps[heap].stats;
+    }
     
   private:
 
     const Rc<vk::DeviceFn>                 m_vkd;
-    const Rc<DxvkAdapter>                  m_adapter;
+    const DxvkDevice*                      m_device;
     const VkPhysicalDeviceProperties       m_devProps;
     const VkPhysicalDeviceMemoryProperties m_memProps;
-    const bool                             m_allowOvercommit;
     
     std::mutex                                      m_mutex;
     std::array<DxvkMemoryHeap, VK_MAX_MEMORY_HEAPS> m_memHeaps;
     std::array<DxvkMemoryType, VK_MAX_MEMORY_TYPES> m_memTypes;
+
+    bool m_restrictAllocations;
     
     DxvkMemory tryAlloc(
       const VkMemoryRequirements*             req,
-      const VkMemoryDedicatedAllocateInfoKHR* dedAllocInfo,
-            VkMemoryPropertyFlags             flags);
+      const VkMemoryDedicatedAllocateInfo*    dedAllocInfo,
+            VkMemoryPropertyFlags             flags,
+            float                             priority);
     
     DxvkMemory tryAllocFromType(
             DxvkMemoryType*                   type,
             VkMemoryPropertyFlags             flags,
             VkDeviceSize                      size,
             VkDeviceSize                      align,
-      const VkMemoryDedicatedAllocateInfoKHR* dedAllocInfo);
+            float                             priority,
+      const VkMemoryDedicatedAllocateInfo*    dedAllocInfo);
     
     DxvkDeviceMemory tryAllocDeviceMemory(
             DxvkMemoryType*                   type,
             VkMemoryPropertyFlags             flags,
             VkDeviceSize                      size,
-      const VkMemoryDedicatedAllocateInfoKHR* dedAllocInfo);
+            float                             priority,
+      const VkMemoryDedicatedAllocateInfo*    dedAllocInfo);
     
     void free(
       const DxvkMemory&           memory);
@@ -297,7 +322,7 @@ namespace dxvk {
             DxvkDeviceMemory      memory);
     
     VkDeviceSize pickChunkSize(
-            VkDeviceSize          heapSize) const;
+            uint32_t              memTypeId) const;
 
   };
   
