@@ -1,14 +1,19 @@
+#include "dxvk_device.h"
 #include "dxvk_meta_copy.h"
 
+#include <dxvk_fullscreen_geom.h>
+#include <dxvk_fullscreen_vert.h>
+#include <dxvk_fullscreen_layer_vert.h>
+
 #include <dxvk_copy_color_1d.h>
 #include <dxvk_copy_color_2d.h>
 #include <dxvk_copy_color_ms.h>
 #include <dxvk_copy_depth_1d.h>
 #include <dxvk_copy_depth_2d.h>
 #include <dxvk_copy_depth_ms.h>
-
-#include <dxvk_resolve_vert.h>
-#include <dxvk_resolve_geom.h>
+#include <dxvk_copy_depth_stencil_1d.h>
+#include <dxvk_copy_depth_stencil_2d.h>
+#include <dxvk_copy_depth_stencil_ms.h>
 
 namespace dxvk {
 
@@ -16,12 +21,14 @@ namespace dxvk {
     const Rc<vk::DeviceFn>&   vkd,
     const Rc<DxvkImageView>&  dstImageView,
     const Rc<DxvkImageView>&  srcImageView,
+    const Rc<DxvkImageView>&  srcStencilView,
           bool                discardDst)
-  : m_vkd         (vkd),
-    m_dstImageView(dstImageView),
-    m_srcImageView(srcImageView),
-    m_renderPass  (createRenderPass(discardDst)),
-    m_framebuffer (createFramebuffer()) {
+  : m_vkd           (vkd),
+    m_dstImageView  (dstImageView),
+    m_srcImageView  (srcImageView),
+    m_srcStencilView(srcStencilView),
+    m_renderPass    (createRenderPass(discardDst)),
+    m_framebuffer   (createFramebuffer()) {
 
   }
   
@@ -35,31 +42,20 @@ namespace dxvk {
   VkRenderPass DxvkMetaCopyRenderPass::createRenderPass(bool discard) const {
     auto aspect = m_dstImageView->info().aspect;
 
-    std::array<VkSubpassDependency, 2> subpassDeps = {{
-      { VK_SUBPASS_EXTERNAL, 0,
-        m_dstImageView->imageInfo().stages,
-        VK_PIPELINE_STAGE_COLOR_ATTACHMENT_OUTPUT_BIT,
-        0, VK_ACCESS_COLOR_ATTACHMENT_WRITE_BIT, 0 },
-      { 0, VK_SUBPASS_EXTERNAL,
-        VK_PIPELINE_STAGE_COLOR_ATTACHMENT_OUTPUT_BIT,
-        m_dstImageView->imageInfo().stages,
-        VK_ACCESS_COLOR_ATTACHMENT_WRITE_BIT,
-        m_dstImageView->imageInfo().access, 0 },
-    }};
-    
     VkAttachmentDescription attachment;
     attachment.flags            = 0;
     attachment.format           = m_dstImageView->info().format;
     attachment.samples          = m_dstImageView->imageInfo().sampleCount;
     attachment.loadOp           = VK_ATTACHMENT_LOAD_OP_LOAD;
     attachment.storeOp          = VK_ATTACHMENT_STORE_OP_STORE;
-    attachment.stencilLoadOp    = VK_ATTACHMENT_LOAD_OP_DONT_CARE;
-    attachment.stencilStoreOp   = VK_ATTACHMENT_STORE_OP_DONT_CARE;
+    attachment.stencilLoadOp    = VK_ATTACHMENT_LOAD_OP_LOAD;
+    attachment.stencilStoreOp   = VK_ATTACHMENT_STORE_OP_STORE;
     attachment.initialLayout    = m_dstImageView->imageInfo().layout;
     attachment.finalLayout      = m_dstImageView->imageInfo().layout;
 
     if (discard) {
       attachment.loadOp         = VK_ATTACHMENT_LOAD_OP_DONT_CARE;
+      attachment.stencilLoadOp  = VK_ATTACHMENT_LOAD_OP_DONT_CARE;
       attachment.initialLayout  = VK_IMAGE_LAYOUT_UNDEFINED;
     }
     
@@ -96,8 +92,8 @@ namespace dxvk {
     info.pAttachments           = &attachment;
     info.subpassCount           = 1;
     info.pSubpasses             = &subpass;
-    info.dependencyCount        = subpassDeps.size();
-    info.pDependencies          = subpassDeps.data();
+    info.dependencyCount        = 0;
+    info.pDependencies          = nullptr;
 
     VkRenderPass result = VK_NULL_HANDLE;
     if (m_vkd->vkCreateRenderPass(m_vkd->device(), &info, nullptr, &result) != VK_SUCCESS)
@@ -124,16 +120,14 @@ namespace dxvk {
     
     VkFramebuffer result = VK_NULL_HANDLE;
     if (m_vkd->vkCreateFramebuffer(m_vkd->device(), &fboInfo, nullptr, &result) != VK_SUCCESS)
-      throw DxvkError("DxvkMetaMipGenRenderPass: Failed to create target framebuffer");
+      throw DxvkError("DxvkMetaCopyRenderPass: Failed to create target framebuffer");
     return result;
   }
 
 
-  DxvkMetaCopyObjects::DxvkMetaCopyObjects(const Rc<vk::DeviceFn>& vkd)
-  : m_vkd         (vkd),
+  DxvkMetaCopyObjects::DxvkMetaCopyObjects(const DxvkDevice* device)
+  : m_vkd         (device->vkd()),
     m_sampler     (createSampler()),
-    m_shaderVert  (createShaderModule(dxvk_resolve_vert)),
-    m_shaderGeom  (createShaderModule(dxvk_resolve_geom)),
     m_color {
       createShaderModule(dxvk_copy_color_1d),
       createShaderModule(dxvk_copy_color_2d),
@@ -142,7 +136,19 @@ namespace dxvk {
       createShaderModule(dxvk_copy_depth_1d),
       createShaderModule(dxvk_copy_depth_2d),
       createShaderModule(dxvk_copy_depth_ms) } {
+    if (device->extensions().extShaderViewportIndexLayer) {
+      m_shaderVert = createShaderModule(dxvk_fullscreen_layer_vert);
+    } else {
+      m_shaderVert = createShaderModule(dxvk_fullscreen_vert);
+      m_shaderGeom = createShaderModule(dxvk_fullscreen_geom);
+    }
     
+    if (device->extensions().extShaderStencilExport) {
+      m_depthStencil = {
+        createShaderModule(dxvk_copy_depth_stencil_1d),
+        createShaderModule(dxvk_copy_depth_stencil_2d),
+        createShaderModule(dxvk_copy_depth_stencil_ms) };
+    }
   }
 
 
@@ -154,6 +160,9 @@ namespace dxvk {
       m_vkd->vkDestroyRenderPass(m_vkd->device(), pair.second.renderPass, nullptr);
     }
 
+    m_vkd->vkDestroyShaderModule(m_vkd->device(), m_depthStencil.fragMs, nullptr);
+    m_vkd->vkDestroyShaderModule(m_vkd->device(), m_depthStencil.frag2D, nullptr);
+    m_vkd->vkDestroyShaderModule(m_vkd->device(), m_depthStencil.frag1D, nullptr);
     m_vkd->vkDestroyShaderModule(m_vkd->device(), m_depth.fragMs, nullptr);
     m_vkd->vkDestroyShaderModule(m_vkd->device(), m_depth.frag2D, nullptr);
     m_vkd->vkDestroyShaderModule(m_vkd->device(), m_depth.frag1D, nullptr);
@@ -265,7 +274,7 @@ namespace dxvk {
     const DxvkMetaCopyPipelineKey&  key) {
     DxvkMetaCopyPipeline pipeline;
     pipeline.renderPass = this->createRenderPass(key);
-    pipeline.dsetLayout = this->createDescriptorSetLayout();
+    pipeline.dsetLayout = this->createDescriptorSetLayout(key);
     pipeline.pipeLayout = this->createPipelineLayout(pipeline.dsetLayout);
     pipeline.pipeHandle = this->createPipelineObject(key, pipeline.pipeLayout, pipeline.renderPass);
     return pipeline;
@@ -282,8 +291,8 @@ namespace dxvk {
     attachment.samples          = key.samples;
     attachment.loadOp           = VK_ATTACHMENT_LOAD_OP_LOAD;
     attachment.storeOp          = VK_ATTACHMENT_STORE_OP_STORE;
-    attachment.stencilLoadOp    = VK_ATTACHMENT_LOAD_OP_DONT_CARE;
-    attachment.stencilStoreOp   = VK_ATTACHMENT_STORE_OP_DONT_CARE;
+    attachment.stencilLoadOp    = VK_ATTACHMENT_LOAD_OP_LOAD;
+    attachment.stencilStoreOp   = VK_ATTACHMENT_STORE_OP_STORE;
     attachment.initialLayout    = VK_IMAGE_LAYOUT_GENERAL;
     attachment.finalLayout      = VK_IMAGE_LAYOUT_GENERAL;
     
@@ -330,20 +339,29 @@ namespace dxvk {
   }
 
   
-  VkDescriptorSetLayout DxvkMetaCopyObjects::createDescriptorSetLayout() const {
-    VkDescriptorSetLayoutBinding binding;
-    binding.binding             = 0;
-    binding.descriptorType      = VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER;
-    binding.descriptorCount     = 1;
-    binding.stageFlags          = VK_SHADER_STAGE_FRAGMENT_BIT;
-    binding.pImmutableSamplers  = &m_sampler;
+  VkDescriptorSetLayout DxvkMetaCopyObjects::createDescriptorSetLayout(
+    const DxvkMetaCopyPipelineKey&  key) const {
+    std::array<VkDescriptorSetLayoutBinding, 2> bindings;
+
+    for (uint32_t i = 0; i < 2; i++) {
+      bindings[i].binding             = i;
+      bindings[i].descriptorType      = VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER;
+      bindings[i].descriptorCount     = 1;
+      bindings[i].stageFlags          = VK_SHADER_STAGE_FRAGMENT_BIT;
+      bindings[i].pImmutableSamplers  = &m_sampler;
+    }
     
     VkDescriptorSetLayoutCreateInfo info;
     info.sType                  = VK_STRUCTURE_TYPE_DESCRIPTOR_SET_LAYOUT_CREATE_INFO;
     info.pNext                  = nullptr;
     info.flags                  = 0;
     info.bindingCount           = 1;
-    info.pBindings              = &binding;
+    info.pBindings              = bindings.data();
+
+    auto format = imageFormatInfo(key.format);
+
+    if (format->aspectMask == (VK_IMAGE_ASPECT_DEPTH_BIT | VK_IMAGE_ASPECT_STENCIL_BIT))
+      info.bindingCount = 2;
     
     VkDescriptorSetLayout result = VK_NULL_HANDLE;
     if (m_vkd->vkCreateDescriptorSetLayout(m_vkd->device(), &info, nullptr, &result) != VK_SUCCESS)
@@ -382,8 +400,9 @@ namespace dxvk {
     auto aspect = imageFormatInfo(key.format)->aspectMask;
 
     std::array<VkPipelineShaderStageCreateInfo, 3> stages;
+    uint32_t stageCount = 0;
     
-    VkPipelineShaderStageCreateInfo& vsStage = stages[0];
+    VkPipelineShaderStageCreateInfo& vsStage = stages[stageCount++];
     vsStage.sType               = VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_CREATE_INFO;
     vsStage.pNext               = nullptr;
     vsStage.flags               = 0;
@@ -392,16 +411,18 @@ namespace dxvk {
     vsStage.pName               = "main";
     vsStage.pSpecializationInfo = nullptr;
     
-    VkPipelineShaderStageCreateInfo& gsStage = stages[1];
-    gsStage.sType               = VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_CREATE_INFO;
-    gsStage.pNext               = nullptr;
-    gsStage.flags               = 0;
-    gsStage.stage               = VK_SHADER_STAGE_GEOMETRY_BIT;
-    gsStage.module              = m_shaderGeom;
-    gsStage.pName               = "main";
-    gsStage.pSpecializationInfo = nullptr;
+    if (m_shaderGeom) {
+      VkPipelineShaderStageCreateInfo& gsStage = stages[stageCount++];
+      gsStage.sType               = VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_CREATE_INFO;
+      gsStage.pNext               = nullptr;
+      gsStage.flags               = 0;
+      gsStage.stage               = VK_SHADER_STAGE_GEOMETRY_BIT;
+      gsStage.module              = m_shaderGeom;
+      gsStage.pName               = "main";
+      gsStage.pSpecializationInfo = nullptr;
+    }
     
-    VkPipelineShaderStageCreateInfo& psStage = stages[2];
+    VkPipelineShaderStageCreateInfo& psStage = stages[stageCount++];
     psStage.sType               = VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_CREATE_INFO;
     psStage.pNext               = nullptr;
     psStage.flags               = 0;
@@ -410,9 +431,23 @@ namespace dxvk {
     psStage.pName               = "main";
     psStage.pSpecializationInfo = nullptr;
 
-    auto shaderSet = (aspect & VK_IMAGE_ASPECT_COLOR_BIT) ? &m_color : &m_depth;
+    std::array<std::pair<const FragShaders*, VkImageAspectFlags>, 3> shaderSets = {{
+      { &m_color,        VK_IMAGE_ASPECT_COLOR_BIT },
+      { &m_depth,        VK_IMAGE_ASPECT_DEPTH_BIT },
+      { &m_depthStencil, VK_IMAGE_ASPECT_DEPTH_BIT | VK_IMAGE_ASPECT_STENCIL_BIT },
+    }};
+
+    const FragShaders* shaderSet = nullptr;
     
-    if (key.viewType == VK_IMAGE_VIEW_TYPE_1D)
+    for (const auto& pair : shaderSets) {
+      if (pair.second == aspect)
+        shaderSet = pair.first;
+    }
+
+    if (!shaderSet)
+      throw DxvkError("DxvkMetaCopyObjects: Unsupported aspect mask");
+
+    if (key.viewType == VK_IMAGE_VIEW_TYPE_1D_ARRAY)
       psStage.module = shaderSet->frag1D;
     else if (key.samples == VK_SAMPLE_COUNT_1_BIT)
       psStage.module = shaderSet->frag2D;
@@ -444,7 +479,7 @@ namespace dxvk {
     iaState.sType               = VK_STRUCTURE_TYPE_PIPELINE_INPUT_ASSEMBLY_STATE_CREATE_INFO;
     iaState.pNext               = nullptr;
     iaState.flags               = 0;
-    iaState.topology            = VK_PRIMITIVE_TOPOLOGY_POINT_LIST;
+    iaState.topology            = VK_PRIMITIVE_TOPOLOGY_TRIANGLE_LIST;
     iaState.primitiveRestartEnable = VK_FALSE;
     
     VkPipelineViewportStateCreateInfo vpState;
@@ -508,9 +543,9 @@ namespace dxvk {
       cbState.blendConstants[i] = 0.0f;
     
     VkStencilOpState stencilOp;
-    stencilOp.failOp            = VK_STENCIL_OP_KEEP;
-    stencilOp.passOp            = VK_STENCIL_OP_KEEP;
-    stencilOp.depthFailOp       = VK_STENCIL_OP_KEEP;
+    stencilOp.failOp            = VK_STENCIL_OP_REPLACE;
+    stencilOp.passOp            = VK_STENCIL_OP_REPLACE;
+    stencilOp.depthFailOp       = VK_STENCIL_OP_REPLACE;
     stencilOp.compareOp         = VK_COMPARE_OP_ALWAYS;
     stencilOp.compareMask       = 0xFFFFFFFF;
     stencilOp.writeMask         = 0xFFFFFFFF;
@@ -524,7 +559,7 @@ namespace dxvk {
     dsState.depthWriteEnable    = VK_TRUE;
     dsState.depthCompareOp      = VK_COMPARE_OP_ALWAYS;
     dsState.depthBoundsTestEnable = VK_FALSE;
-    dsState.stencilTestEnable   = VK_FALSE;
+    dsState.stencilTestEnable   = VK_TRUE;
     dsState.front               = stencilOp;
     dsState.back                = stencilOp;
     dsState.minDepthBounds      = 0.0f;
@@ -534,7 +569,7 @@ namespace dxvk {
     info.sType                  = VK_STRUCTURE_TYPE_GRAPHICS_PIPELINE_CREATE_INFO;
     info.pNext                  = nullptr;
     info.flags                  = 0;
-    info.stageCount             = stages.size();
+    info.stageCount             = stageCount;
     info.pStages                = stages.data();
     info.pVertexInputState      = &viState;
     info.pInputAssemblyState    = &iaState;
