@@ -8,13 +8,12 @@ namespace dxvk {
     DxvkOptions() { }
     DxvkOptions(const Config& config);
 
-    /// Allow allocating more memory from
-    /// a heap than the device supports.
-    bool allowMemoryOvercommit;
-
     /// Enable state cache
     bool enableStateCache;
 
+    /// Enables OpenVR loading
+    bool enableOpenVR;
+
     /// Number of compiler threads
     /// when using the state cache
     int32_t numCompilerThreads;
@@ -22,6 +21,9 @@ namespace dxvk {
     /// Shader-related options
     Tristate useRawSsbo;
     Tristate useEarlyDiscard;
+
+    /// HUD elements
+    std::string hud;
   };
 
 }
\ No newline at end of file
