@@ -65,6 +65,14 @@ namespace dxvk {
     const DxvkDescriptorSlot* bindingInfos() const {
       return m_descriptorSlots.data();
     }
+
+    /**
+     * \brief Push constant range
+     * \returns Push constant range
+     */
+    VkPushConstantRange pushConstRange() const {
+      return m_pushConstRange;
+    }
     
     /**
      * \brief Defines a new slot
@@ -73,18 +81,24 @@ namespace dxvk {
      * defined by another shader stage, this will extend
      * the stage mask by the given stage. Otherwise, an
      * entirely new binding is added.
-     * \param [in] slot Resource slot
-     * \param [in] type Resource type
-     * \param [in] view Image view type
      * \param [in] stage Shader stage
-     * \param [in] access Access flags
+     * \param [in] desc Slot description
      */
     void defineSlot(
-            uint32_t              slot,
-            VkDescriptorType      type,
-            VkImageViewType       view,
             VkShaderStageFlagBits stage,
-            VkAccessFlags         access);
+      const DxvkResourceSlot&     desc);
+
+    /**
+     * \brief Defines new push constant range
+     *
+     * \param [in] stage Shader stage
+     * \param [in] offset Range offset
+     * \param [in] size Range size
+     */
+    void definePushConstRange(
+            VkShaderStageFlagBits stage,
+            uint32_t              offset,
+            uint32_t              size);
     
     /**
      * \brief Gets binding ID for a slot
@@ -112,6 +126,7 @@ namespace dxvk {
   private:
     
     std::vector<DxvkDescriptorSlot> m_descriptorSlots;
+    VkPushConstantRange             m_pushConstRange = { };
 
     uint32_t countDescriptors(
             VkDescriptorType      type) const;
@@ -135,8 +150,7 @@ namespace dxvk {
     
     DxvkPipelineLayout(
       const Rc<vk::DeviceFn>&   vkd,
-            uint32_t            bindingCount,
-      const DxvkDescriptorSlot* bindingInfos,
+      const DxvkDescriptorSlotMapping& slotMapping,
             VkPipelineBindPoint pipelineBindPoint);
     
     ~DxvkPipelineLayout();
@@ -167,6 +181,14 @@ namespace dxvk {
       return m_bindingSlots.data();
     }
     
+    /**
+     * \brief Push constant range
+     * \returns Push constant range
+     */
+    const VkPushConstantRange& pushConstRange() const {
+      return m_pushConstRange;
+    }
+    
     /**
      * \brief Descriptor set layout handle
      * \returns Descriptor set layout handle
@@ -213,13 +235,12 @@ namespace dxvk {
      * \brief Checks for static buffer bindings
      * 
      * Returns \c true if there is at least one
-     * descriptor of the static uniform or storage
-     * buffer type.
+     * descriptor of the static uniform buffer
+     * type.
      */
     bool hasStaticBufferBindings() const {
-      return m_descriptorTypes.any(
-        VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER,
-        VK_DESCRIPTOR_TYPE_STORAGE_BUFFER);
+      return m_descriptorTypes.test(
+        VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER);
     }
     
     /**
@@ -245,6 +266,7 @@ namespace dxvk {
     
     Rc<vk::DeviceFn> m_vkd;
     
+    VkPushConstantRange             m_pushConstRange      = { };
     VkDescriptorSetLayout           m_descriptorSetLayout = VK_NULL_HANDLE;
     VkPipelineLayout                m_pipelineLayout      = VK_NULL_HANDLE;
     VkDescriptorUpdateTemplateKHR   m_descriptorTemplate  = VK_NULL_HANDLE;
