@@ -1,10 +1,11 @@
 #include <algorithm>
 
+#include "dxvk_device.h"
 #include "dxvk_renderpass.h"
 
 namespace dxvk {
   
-  bool DxvkRenderPassFormat::matches(const DxvkRenderPassFormat& fmt) const {
+  bool DxvkRenderPassFormat::eq(const DxvkRenderPassFormat& fmt) const {
     bool eq = sampleCount == fmt.sampleCount;
     
     for (uint32_t i = 0; i < MaxNumRenderTargets && eq; i++) {
@@ -17,6 +18,21 @@ namespace dxvk {
     
     return eq;
   }
+
+
+  size_t DxvkRenderPassFormat::hash() const {
+    DxvkHashState state;
+    state.add(uint32_t(sampleCount));
+
+    for (uint32_t i = 0; i < MaxNumRenderTargets; i++) {
+      state.add(uint32_t(color[i].format));
+      state.add(uint32_t(color[i].layout));
+    }
+
+    state.add(uint32_t(depth.format));
+    state.add(uint32_t(depth.layout));
+    return state;
+  }
   
   
   DxvkRenderPass::DxvkRenderPass(
@@ -39,7 +55,7 @@ namespace dxvk {
   
   
   bool DxvkRenderPass::hasCompatibleFormat(const DxvkRenderPassFormat& fmt) const {
-    return m_format.matches(fmt);
+    return m_format.eq(fmt);
   }
   
   
@@ -66,9 +82,6 @@ namespace dxvk {
     // Render passes may not require the previous
     // contents of the attachments to be preserved.
     for (uint32_t i = 0; i < MaxNumRenderTargets; i++) {
-      colorRef[i].attachment = VK_ATTACHMENT_UNUSED;
-      colorRef[i].layout     = VK_IMAGE_LAYOUT_UNDEFINED;
-      
       if (m_format.color[i].format != VK_FORMAT_UNDEFINED) {
         VkAttachmentDescription desc;
         desc.flags            = 0;
@@ -85,6 +98,9 @@ namespace dxvk {
         colorRef[i].layout     = m_format.color[i].layout;
         
         attachments.push_back(desc);
+      } else {
+        colorRef[i].attachment = VK_ATTACHMENT_UNUSED;
+        colorRef[i].layout     = VK_IMAGE_LAYOUT_UNDEFINED;
       }
     }
     
@@ -104,6 +120,9 @@ namespace dxvk {
       depthRef.layout     = m_format.depth.layout;
       
       attachments.push_back(desc);
+    } else {
+      depthRef.attachment = VK_ATTACHMENT_UNUSED;
+      depthRef.layout     = VK_IMAGE_LAYOUT_UNDEFINED;
     }
     
     VkSubpassDescription subpass;
@@ -140,10 +159,11 @@ namespace dxvk {
           VK_PIPELINE_STAGE_ALL_GRAPHICS_BIT |
           VK_PIPELINE_STAGE_ALL_COMMANDS_BIT)) {
       subpassDeps[subpassDepCount++] = { 0, 0,
+        VK_PIPELINE_STAGE_COLOR_ATTACHMENT_OUTPUT_BIT,
         VK_PIPELINE_STAGE_FRAGMENT_SHADER_BIT,
-        VK_PIPELINE_STAGE_FRAGMENT_SHADER_BIT,
-        VK_ACCESS_SHADER_WRITE_BIT,
-        VK_ACCESS_SHADER_READ_BIT, 0 };
+        VK_ACCESS_COLOR_ATTACHMENT_WRITE_BIT,
+        VK_ACCESS_SHADER_READ_BIT,
+        VK_DEPENDENCY_BY_REGION_BIT };
     }
 
     if (ops.barrier.srcStages && ops.barrier.dstStages) {
@@ -205,8 +225,8 @@ namespace dxvk {
   }
   
   
-  DxvkRenderPassPool::DxvkRenderPassPool(const Rc<vk::DeviceFn>& vkd)
-  : m_vkd(vkd) {
+  DxvkRenderPassPool::DxvkRenderPassPool(const DxvkDevice* device)
+  : m_vkd(device->vkd()) {
     
   }
   
@@ -216,17 +236,17 @@ namespace dxvk {
   }
   
   
-  Rc<DxvkRenderPass> DxvkRenderPassPool::getRenderPass(const DxvkRenderPassFormat& fmt) {
+  DxvkRenderPass* DxvkRenderPassPool::getRenderPass(const DxvkRenderPassFormat& fmt) {
     std::lock_guard<std::mutex> lock(m_mutex);
+
+    auto entry = m_renderPasses.find(fmt);
+    if (entry != m_renderPasses.end())
+      return &entry->second;
     
-    for (const auto& r : m_renderPasses) {
-      if (r->hasCompatibleFormat(fmt))
-        return r;
-    }
-    
-    Rc<DxvkRenderPass> rp = new DxvkRenderPass(m_vkd, fmt);
-    m_renderPasses.push_back(rp);
-    return rp;
+    auto result = m_renderPasses.emplace(std::piecewise_construct,
+      std::tuple(fmt),
+      std::tuple(m_vkd, fmt));
+    return &result.first->second;
   }
   
 }
\ No newline at end of file
