@@ -8,6 +8,7 @@
 #include "dxvk_shader_key.h"
 
 #include "../spirv/spirv_code_buffer.h"
+#include "../spirv/spirv_compression.h"
 
 namespace dxvk {
   
@@ -30,12 +31,23 @@ namespace dxvk {
     // Specialization constants for pipeline state
     SpecConstantRangeStart      = ColorComponentMappings + MaxNumRenderTargets * 4,
     RasterizerSampleCount       = SpecConstantRangeStart + 0,
-
-    /// Lowest and highest known spec constant IDs
-    SpecConstantIdMin           = RasterizerSampleCount,
-    SpecConstantIdMax           = RasterizerSampleCount,
+    FirstPipelineConstant
   };
-  
+
+  /**
+   * \brief Shader flags
+   *
+   * Provides extra information about the features
+   * used by a shader.
+   */
+  enum DxvkShaderFlag : uint64_t {
+    HasSampleRateShading,
+    HasTransformFeedback,
+    ExportsStencilRef,
+    ExportsViewportIndexLayerFromVertexStage,
+  };
+
+  using DxvkShaderFlags = Flags<DxvkShaderFlag>;
   
   /**
    * \brief Shader interface slots
@@ -45,8 +57,10 @@ namespace dxvk {
    * purely for validation purposes.
    */
   struct DxvkInterfaceSlots {
-    uint32_t inputSlots  = 0;
-    uint32_t outputSlots = 0;
+    uint32_t inputSlots      = 0;
+    uint32_t outputSlots     = 0;
+    uint32_t pushConstOffset = 0;
+    uint32_t pushConstSize   = 0;
   };
 
 
@@ -107,7 +121,8 @@ namespace dxvk {
    * \brief Shader module create info
    */
   struct DxvkShaderModuleCreateInfo {
-    bool fsDualSrcBlend;
+    bool      fsDualSrcBlend  = false;
+    uint32_t  undefinedInputs = 0;
   };
   
   
@@ -128,7 +143,7 @@ namespace dxvk {
             uint32_t                slotCount,
       const DxvkResourceSlot*       slotInfos,
       const DxvkInterfaceSlots&     iface,
-      const SpirvCodeBuffer&        code,
+            SpirvCodeBuffer         code,
       const DxvkShaderOptions&      options,
             DxvkShaderConstData&&   constData);
     
@@ -143,16 +158,12 @@ namespace dxvk {
     }
     
     /**
-     * \brief Checks whether a capability is enabled
-     * 
-     * If the shader contains an \c OpCapability
-     * instruction with the given capability, it
-     * is considered enabled. This may be required
-     * to correctly set up certain pipeline states.
-     * \param [in] cap The capability to check
-     * \returns \c true if \c cap is enabled
+     * \brief Retrieves shader flags
+     * \returns Shader flags
      */
-    bool hasCapability(spv::Capability cap);
+    DxvkShaderFlags flags() const {
+      return m_flags;
+    }
     
     /**
      * \brief Adds resource slots definitions to a mapping
@@ -173,7 +184,7 @@ namespace dxvk {
      * \param [in] info Module create info
      * \returns The shader module
      */
-    Rc<DxvkShaderModule> createShaderModule(
+    DxvkShaderModule createShaderModule(
       const Rc<vk::DeviceFn>&          vkd,
       const DxvkDescriptorSlotMapping& mapping,
       const DxvkShaderModuleCreateInfo& info);
@@ -223,6 +234,7 @@ namespace dxvk {
      */
     void setShaderKey(const DxvkShaderKey& key) {
       m_key = key;
+      m_hash = key.hash();
     }
 
     /**
@@ -232,6 +244,18 @@ namespace dxvk {
     DxvkShaderKey getShaderKey() const {
       return m_key;
     }
+
+    /**
+     * \brief Get lookup hash
+     * 
+     * Retrieves a non-unique hash value derived from the
+     * shader key which can be used to perform lookups.
+     * This is better than relying on the pointer value.
+     * \returns Hash value for map lookups
+     */
+    size_t getHash() const {
+      return m_hash;
+    }
     
     /**
      * \brief Retrieves debug name
@@ -240,22 +264,38 @@ namespace dxvk {
     std::string debugName() const {
       return m_key.toString();
     }
+
+    /**
+     * \brief Get lookup hash for a shader
+     *
+     * Convenience method that returns \c 0 for a null
+     * pointer, and the shader's lookup hash otherwise.
+     * \param [in] shader The shader
+     * \returns The shader's lookup hash, or 0
+     */
+    static size_t getHash(const Rc<DxvkShader>& shader) {
+      return shader != nullptr ? shader->getHash() : 0;
+    }
     
   private:
     
     VkShaderStageFlagBits m_stage;
-    SpirvCodeBuffer       m_code;
+    SpirvCompressedBuffer m_code;
     
     std::vector<DxvkResourceSlot> m_slots;
     std::vector<size_t>           m_idOffsets;
     DxvkInterfaceSlots            m_interface;
+    DxvkShaderFlags               m_flags;
     DxvkShaderOptions             m_options;
     DxvkShaderConstData           m_constData;
     DxvkShaderKey                 m_key;
+    size_t                        m_hash = 0;
 
     size_t m_o1IdxOffset = 0;
     size_t m_o1LocOffset = 0;
-    
+
+    static void eliminateInput(SpirvCodeBuffer& code, uint32_t location);
+
   };
   
 
@@ -267,9 +307,13 @@ namespace dxvk {
    * context will create pipeline objects on the
    * fly when executing draw calls.
    */
-  class DxvkShaderModule : public RcObject {
+  class DxvkShaderModule {
     
   public:
+
+    DxvkShaderModule();
+
+    DxvkShaderModule(DxvkShaderModule&& other);
     
     DxvkShaderModule(
       const Rc<vk::DeviceFn>&     vkd,
@@ -277,14 +321,8 @@ namespace dxvk {
       const SpirvCodeBuffer&      code);
     
     ~DxvkShaderModule();
-    
-    /**
-     * \brief Shader module handle
-     * \returns Shader module handle
-     */
-    VkShaderModule handle() const {
-      return m_module;
-    }
+
+    DxvkShaderModule& operator = (DxvkShaderModule&& other);
     
     /**
      * \brief Shader stage creation info
@@ -293,29 +331,24 @@ namespace dxvk {
      * \returns Shader stage create info
      */
     VkPipelineShaderStageCreateInfo stageInfo(
-      const VkSpecializationInfo* specInfo) const;
+      const VkSpecializationInfo* specInfo) const {
+      VkPipelineShaderStageCreateInfo stage = m_stage;
+      stage.pSpecializationInfo = specInfo;
+      return stage;
+    }
     
     /**
-     * \brief Shader object
-     * \returns The shader
+     * \brief Checks whether module is valid
+     * \returns \c true if module is valid
      */
-    Rc<DxvkShader> shader() const {
-      return m_shader;
-    }
-
-    /**
-     * \brief Retrieves shader key
-     * \returns Unique shader key
-     */
-    DxvkShaderKey getShaderKey() const {
-      return m_shader->getShaderKey();
+    operator bool () const {
+      return m_stage.module != VK_NULL_HANDLE;
     }
     
   private:
     
-    Rc<vk::DeviceFn>      m_vkd;
-    Rc<DxvkShader>        m_shader;
-    VkShaderModule        m_module;
+    Rc<vk::DeviceFn>                m_vkd;
+    VkPipelineShaderStageCreateInfo m_stage;
     
   };
   
