@@ -1,5 +1,7 @@
 #pragma once
 
+#include <unordered_set>
+
 #include "spirv_code_buffer.h"
 
 namespace dxvk {
@@ -26,6 +28,10 @@ namespace dxvk {
     uint32_t sSampleId     = 0;
     uint32_t sMinLod       = 0;
   };
+
+  constexpr uint32_t spvVersion(uint32_t major, uint32_t minor) {
+    return (major << 16) | (minor << 8);
+  }
   
   /**
    * \brief SPIR-V module
@@ -39,7 +45,8 @@ namespace dxvk {
     
   public:
     
-    SpirvModule();
+    explicit SpirvModule(uint32_t version);
+
     ~SpirvModule();
     
     SpirvCodeBuffer compile() const;
@@ -137,24 +144,57 @@ namespace dxvk {
             int32_t                 y,
             int32_t                 z,
             int32_t                 w);
+
+    uint32_t constvec4b32(
+            bool                    x,
+            bool                    y,
+            bool                    z,
+            bool                    w);
     
     uint32_t constvec4u32(
             uint32_t                x,
             uint32_t                y,
             uint32_t                z,
             uint32_t                w);
-    
+
+    uint32_t constvec2f32(
+            float                   x,
+            float                   y);
+
+    uint32_t constvec3f32(
+            float                   x,
+            float                   y,
+            float                   z);
+
     uint32_t constvec4f32(
             float                   x,
             float                   y,
             float                   z,
             float                   w);
+
+    uint32_t constfReplicant(
+            float                   replicant,
+            uint32_t                count);
+
+    uint32_t constbReplicant(
+            bool                    replicant,
+            uint32_t                count);
     
     uint32_t constComposite(
             uint32_t                typeId,
             uint32_t                constCount,
       const uint32_t*               constIds);
     
+    uint32_t constUndef(
+            uint32_t                typeId);
+    
+    uint32_t lateConst32(
+            uint32_t                typeId);
+
+    void setLateConst(
+            uint32_t                constId,
+      const uint32_t*               argIds);
+
     uint32_t specConstBool(
             bool                    v);
     
@@ -212,6 +252,16 @@ namespace dxvk {
             uint32_t                structId,
             uint32_t                memberId,
             spv::BuiltIn            builtIn);
+
+    void memberDecorate(
+            uint32_t                structId,
+            uint32_t                memberId,
+            spv::Decoration         decoration);
+
+    void memberDecorateMatrixStride(
+            uint32_t                structId,
+            uint32_t                memberId,
+            uint32_t                stride);
     
     void memberDecorateOffset(
             uint32_t                structId,
@@ -301,7 +351,7 @@ namespace dxvk {
             uint32_t                parameterType);
     
     void functionEnd();
-    
+
     uint32_t opAccessChain(
             uint32_t                resultType,
             uint32_t                composite,
@@ -586,6 +636,17 @@ namespace dxvk {
     uint32_t opFAbs(
             uint32_t                resultType,
             uint32_t                operand);
+
+    uint32_t opFMix(
+            uint32_t                resultType,
+            uint32_t                x,
+            uint32_t                y,
+            uint32_t                a);
+
+  uint32_t opCross(
+            uint32_t                resultType,
+            uint32_t                x,
+            uint32_t                y);
     
     uint32_t opIAdd(
             uint32_t                resultType,
@@ -641,6 +702,34 @@ namespace dxvk {
             uint32_t                resultType,
             uint32_t                a,
             uint32_t                b);
+
+    uint32_t opVectorTimesScalar(
+            uint32_t                resultType,
+            uint32_t                vector,
+            uint32_t                scalar);
+
+    uint32_t opMatrixTimesMatrix(
+            uint32_t                resultType,
+            uint32_t                a,
+            uint32_t                b);
+
+    uint32_t opMatrixTimesVector(
+            uint32_t                resultType,
+            uint32_t                matrix,
+            uint32_t                vector);
+
+    uint32_t opVectorTimesMatrix(
+            uint32_t                resultType,
+            uint32_t                vector,
+            uint32_t                matrix);
+
+    uint32_t opTranspose(
+            uint32_t                resultType,
+            uint32_t                matrix);
+
+    uint32_t opInverse(
+            uint32_t                resultType,
+            uint32_t                matrix);
     
     uint32_t opFFma(
             uint32_t                resultType,
@@ -824,14 +913,36 @@ namespace dxvk {
     uint32_t opInverseSqrt(
             uint32_t                resultType,
             uint32_t                operand);
+
+    uint32_t opNormalize(
+            uint32_t                resultType,
+            uint32_t                operand);
+
+    uint32_t opReflect(
+            uint32_t                resultType,
+            uint32_t                incident,
+            uint32_t                normal);
+
+    uint32_t opLength(
+            uint32_t                resultType,
+            uint32_t                operand);
     
     uint32_t opExp2(
             uint32_t                resultType,
             uint32_t                operand);
+
+    uint32_t opExp(
+            uint32_t                resultType,
+            uint32_t                operand);
     
     uint32_t opLog2(
             uint32_t                resultType,
             uint32_t                operand);
+
+    uint32_t opPow(
+            uint32_t                resultType,
+            uint32_t                base,
+            uint32_t                exponent);
     
     uint32_t opFract(
             uint32_t                resultType,
@@ -874,6 +985,10 @@ namespace dxvk {
             uint32_t                condition,
             uint32_t                operand1,
             uint32_t                operand2);
+
+    uint32_t opIsNan(
+            uint32_t                resultType,
+            uint32_t                operand);
     
     uint32_t opFunctionCall(
             uint32_t                resultType,
@@ -982,6 +1097,18 @@ namespace dxvk {
             uint32_t                sampledImage,
             uint32_t                coordinates,
       const SpirvImageOperands&     operands);
+
+    uint32_t opImageSampleProjImplicitLod(
+            uint32_t                resultType,
+            uint32_t                sampledImage,
+            uint32_t                coordinates,
+      const SpirvImageOperands&     operands);
+
+    uint32_t opImageSampleProjExplicitLod(
+            uint32_t                resultType,
+            uint32_t                sampledImage,
+            uint32_t                coordinates,
+      const SpirvImageOperands&     operands);
     
     uint32_t opImageSampleDrefImplicitLod(
             uint32_t                resultType,
@@ -997,6 +1124,20 @@ namespace dxvk {
             uint32_t                reference,
       const SpirvImageOperands&     operands);
 
+    uint32_t opImageSampleProjDrefImplicitLod(
+            uint32_t                resultType,
+            uint32_t                sampledImage,
+            uint32_t                coordinates,
+            uint32_t                reference,
+      const SpirvImageOperands&     operands);
+
+    uint32_t opImageSampleProjDrefExplicitLod(
+            uint32_t                resultType,
+            uint32_t                sampledImage,
+            uint32_t                coordinates,
+            uint32_t                reference,
+      const SpirvImageOperands&     operands);
+
     uint32_t opGroupNonUniformBallot(
             uint32_t                resultType,
             uint32_t                execution,
@@ -1008,12 +1149,14 @@ namespace dxvk {
             uint32_t                operation,
             uint32_t                ballot);
     
-    uint32_t opGroupNonUniformLogicalAnd(
+    uint32_t opGroupNonUniformElect(
+            uint32_t                resultType,
+            uint32_t                execution);
+    
+    uint32_t opGroupNonUniformBroadcastFirst(
             uint32_t                resultType,
             uint32_t                execution,
-            uint32_t                operation,
-            uint32_t                value,
-            uint32_t                clusterSize);
+            uint32_t                value);
     
     void opControlBarrier(
             uint32_t                execution,
@@ -1055,6 +1198,8 @@ namespace dxvk {
     void opReturn();
     
     void opKill();
+
+    void opDemoteToHelperInvocation();
     
     void opEmitVertex(
             uint32_t                streamId);
@@ -1064,6 +1209,7 @@ namespace dxvk {
     
   private:
     
+    uint32_t m_version;
     uint32_t m_id             = 1;
     uint32_t m_instExtGlsl450 = 0;
     
@@ -1078,6 +1224,8 @@ namespace dxvk {
     SpirvCodeBuffer m_typeConstDefs;
     SpirvCodeBuffer m_variables;
     SpirvCodeBuffer m_code;
+
+    std::unordered_set<uint32_t> m_lateConsts;
     
     uint32_t defType(
             spv::Op                 op, 
