@@ -25,10 +25,24 @@ namespace dxvk::env {
    */
   std::string getExeName();
   
+  /**
+   * \brief Gets full path to executable
+   * \returns Path to executable
+   */
+  std::string getExePath();
+  
   /**
    * \brief Sets name of the calling thread
    * \param [in] name Thread name
    */
   void setThreadName(const std::string& name);
+
+  /**
+   * \brief Creates a directory
+   * 
+   * \param [in] path Path to directory
+   * \returns \c true on success
+   */
+  bool createDirectory(const std::string& path);
   
 }
