@@ -1,14 +1,8 @@
 #include "util_string.h"
 
-#ifdef CP_UNIXCP
-static constexpr int cp = CP_UNIXCP;
-#else
-static constexpr int cp = CP_ACP;
-#endif
-
 namespace dxvk::str {
   std::string fromws(const WCHAR *ws) {
-    size_t len = ::WideCharToMultiByte(cp,
+    size_t len = ::WideCharToMultiByte(CP_UTF8,
       0, ws, -1, nullptr, 0, nullptr, nullptr);
 
     if (len <= 1)
@@ -18,8 +12,16 @@ namespace dxvk::str {
 
     std::string result;
     result.resize(len);
-    ::WideCharToMultiByte(cp, 0, ws, -1,
+    ::WideCharToMultiByte(CP_UTF8, 0, ws, -1,
       &result.at(0), len, nullptr, nullptr);
     return result;
   }
+
+
+  void tows(const char* mbs, WCHAR* wcs, size_t wcsLen) {
+    ::MultiByteToWideChar(
+      CP_UTF8, 0, mbs, -1,
+      wcs, wcsLen);
+  }
+
 }
