@@ -2,12 +2,20 @@
 
 #include <string>
 #include <sstream>
+#include <vector>
 
 #include "./com/com_include.h"
 
 namespace dxvk::str {
   
   std::string fromws(const WCHAR *ws);
+
+  void tows(const char* mbs, WCHAR* wcs, size_t wcsLen);
+
+  template <size_t N>
+  void tows(const char* mbs, WCHAR (&wcs)[N]) {
+    return tows(mbs, wcs, N);
+  }
   
   inline void format1(std::stringstream&) { }
 
